package Bowtie;
#!/usr/bin/perl
use strict;
use warnings;
use Cwd;
use File::Copy;
use File::Basename;


# My Library
use MyMath;
use MyFasta;




# =============================
# Function: change the format of Bowtie to required format for Mireap
# Input: infile for Bowtie default output
# 		 outfile for result
# Author: Yuangen Yao
# Date: 2013-2-20
# ֤MIREAPеǴ1ʼ
# =============================

sub bowtie2Mireap {
	
	my $infile = shift or die;
	my $outfile = shift or die;
	
	unless(-e $infile){
		print STDERR "$infile not exists\n";
		die;
	}
	
	open IN,"<$infile" or die;
	open OUT,">$outfile" or die;
	
	while( defined( my $line = <IN> ) ){
		chomp( $line  );
		next if $line =~ /^\s*$/;				
		my @tmp = split /\t/,$line;
		my $read_id 	= $tmp[0];
		my $strand		= $tmp[1];
		my $refId		= $tmp[2];
		my $beg			= $tmp[3];
		my $read_seq	= $tmp[4];
		
		$beg++;
		my $end = $beg + length( $read_seq ) - 1;
		
		my $tag_name = MyFasta::getTagName($read_id);
		my $tag_freq = MyFasta::getTagFreq($read_id);
		
		print OUT "$tag_name\t$refId\t$beg\t$end\t$strand\n";
	}
	
	close IN;
	close OUT;
	
}



# =============================
# Function: get expression level of mRNA according to mapping
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub bowtie2bed {
	
	my $infile_mapping = shift or die;
	my $outfile_bed = shift or die;
	
	unless(-e $infile_mapping){
		print STDERR "Error:$infile_mapping not exists\n";
		die;
	}
	
	open IN,"<$infile_mapping" or die;
	open OUT,">$outfile_bed" or die;
	
	while( defined( my $line = <IN> ) ){
		chomp( $line  );
		next if $line =~ /^\s*$/;				
		my @tmp = split /\t/,$line;
		my $read_id 	= $tmp[0];
		my $strand		= $tmp[1];
		my $refId		= $tmp[2];
		my $beg			= $tmp[3];
		my $read_seq	= $tmp[4];
		my $mapTime		= $tmp[6];	
		my $mis	  		= $tmp[7];
	
		if ( defined $mis ) {
			my @tmp = split /,/,$mis;
			$mis = scalar @tmp;
		} else {
			$mis = 0;
		}
		
		$beg++;
		my $end = $beg + length( $read_seq ) - 1;
		
		print OUT "$refId\t$beg\t$end\t$read_id\t$mis\t$strand\n";
		
	}
	
	close IN;
	close OUT;
	
}




# =============================
# Function: parse default Bowtie output
# Input: default Bowtie output 
# Return: filename array.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub parser1 {		
	
	my $infile   = shift or die;
	my $hash_dis = shift or die;
	
	my $given_mis 	= shift;
	my $adaptor   	= shift;
	my $hash_refseq = shift;

	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}

	open IN,"<$infile" or die;
	while( defined( my $line = <IN> ) ){
		chomp( $line  );
		next if $line =~ /^\s*$/;				
		my @tmp = split /\t/,$line;
		my $read_id 	= $tmp[0];
		my $strand		= $tmp[1];
		my $refId		= $tmp[2];
		my $beg			= $tmp[3];
		my $read_seq	= $tmp[4];
		my $mapTime		= $tmp[6];	
		my $mis_string	= $tmp[7];
			
		$beg++;
		my $end = $beg + length( $read_seq ) - 1;
		
		my $count = MyFasta::getTagFreq($read_id);
		$mapTime++;
		$count = MyMath::round($count/$mapTime,6);
		
		# ****************************************************** #
		
		if ((defined $given_mis) and (defined $adaptor) and (defined $hash_refseq)){
			# head.
			my $mis = 0;
			if( $beg < length($adaptor) + 1 ){
				$mis += abs($beg - length($adaptor) + 1);
			}
			
			my ($ref_seq,$ref_end);
			if(exists $hash_refseq->{$refId}){
				$ref_seq = $hash_refseq->{$refId} ;
				$ref_end = length($adaptor) + length($ref_seq);
			} else {
				print STDERR "Error:$refId\n";
				die;
			}
			
			# tail.
			if ( $end > $ref_end ){
				$mis += abs($end - $ref_end);
			}
			
			# middle.
			my $mis_mid = 0;
			if ( defined $mis_string ) {
				my @mis_pos = split /,/,$mis_string;
				for my $p_str ( @mis_pos ){
					if($p_str=~/(\d+):/){
						my $p = $1;
						unless(&is_number($p)){
							print STDERR "Error:$p not a numeric\n";
							die;
						}
						if( $p >= length($adaptor) + 1 and $p <= $ref_end ){
							$mis_mid++;
						}
					} else {
						die;
					}
				}
			}	
			
			$mis += $mis_mid;
			if( $mis > $given_mis ){
				next;
			} 
			
		}
		
		# ****************************************************** #
		
		unless ( exists $hash_dis->{$refId}{tag} ) {
			$hash_dis->{$refId}{tag} = 1;
			$hash_dis->{$refId}{read} = $count;
		} else {
			$hash_dis->{$refId}{tag}++;
			$hash_dis->{$refId}{read} += $count;
		}
		
	}
	close IN;	
}





# =============================
# Function: parse default Bowtie output
# Input: default Bowtie output 
# Return: filename array.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub parser2 {	
	
	my $infile = shift or die;
	my $hash_map = shift or die;	

	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}
	
	open IN,"<$infile" or die;
	while( defined( my $line = <IN> ) ){
		chomp( $line  );
		next if $line =~ /^\s*$/;	
			
		my @tmp = split /\t/,$line;
		my $id 		= $tmp[0];
		my $strand 	= $tmp[1];
		my $chrom  	= $tmp[2];
		my $beg		= $tmp[3];
		my $seq	 	= $tmp[4];
		my $mis	  	= $tmp[7];
		my $lng     = length( $seq );	
		
		if ( defined $mis ) {
			my @tmp = split /,/,$mis;
			$mis = scalar @tmp;
		} else {
			$mis = 0;
		}
		
		$beg++;
		my $end  = $beg + $lng - 1;
				
		unless ( exists $hash_map->{$chrom}{$strand}{$id}{$beg} ){
			$hash_map->{$chrom}{$strand}{$id}{$beg} = "$mis:$seq";
		} else {
			die;
		}
		
	}
	close IN;	
}







# =============================
# Function: parse default Bowtie output
# Input: default Bowtie output 
# Return: filename array.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================


sub parser3 {	
	
	my $infile 	 = shift or die;
	my $hash_map = shift or die;	
	my $hash_tag = shift or die;
	my $hash_tag_count = shift or die;
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}
	
	open IN,"<$infile" or die;
	while( defined( my $line = <IN> ) ){
		chomp( $line  );
		next if $line =~ /^\s*$/;	
			
		my @tmp = split /\t/,$line;
		my $id 		= $tmp[0];
		my $strand 	= $tmp[1];
		my $refId	= $tmp[2];
		my $beg		= $tmp[3];
		my $seq	 	= $tmp[4];
		my $mapTime	= $tmp[6];	
		my $mis	  	= $tmp[7];
		my $lng     = length( $seq );	
		
		unless ( defined $mis ) {
			$mis = 0;
		}	
		
		$beg++;
		my $end = $beg + length($seq) - 1;	
		if ( $strand ne "+" ){
			next;
		}
		
		my $count = MyFasta::getTagFreq($id);
		$mapTime++;
		$count = MyMath::round($count/$mapTime,6);
		
		unless(exists $hash_tag_count->{$id}){
			$hash_tag_count->{$id} = $count;
		}
		
		unless ( exists $hash_map->{$refId}{$id}{$beg}{$end} ) {
			$hash_map->{$refId}{$id}{$beg}{$end} = $mis;
		} else {
			print STDERR "Error:($id,$refId,$beg,$end) repeat\n";
			die;
		}	
		
		unless( exists $hash_tag->{$id} ){
			$hash_tag->{$id} = $seq;
		}
			
	}
	close IN;	
}





# =============================
# Function: parse default Bowtie output
# Input: default Bowtie output 
# Return: filename array.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================


sub parser4 {	
	
	my $infile 	 = shift or die;
	my $hash_map = shift or die;	
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}
	
	open IN,"<$infile" or die;
	while( defined( my $line = <IN> ) ){
		chomp( $line  );
		next if $line =~ /^\s*$/;	
			
		my @tmp = split /\t/,$line;
		my $id 		= $tmp[0];
		my $strand 	= $tmp[1];
		my $refId	= $tmp[2];
		my $beg		= $tmp[3];
		my $seq	 	= $tmp[4];
		my $mapTime	= $tmp[6];	
		my $mis	  	= $tmp[7];
		my $lng     = length( $seq );	
		
		unless ( defined $mis ) {
			$mis = 0;
		}	
		
		next if $strand ne "+";
		
		$beg++;
		my $end = $beg + length($seq) - 1;	
		
		unless ( exists $hash_map->{$refId}{$id}{$beg}{$end} ) {
			$hash_map->{$refId}{$id}{$beg}{$end} = 1;
		} else {
			print STDERR "Error:($id,$refId,$beg,$end) repeat\n";
			die;
		}	
		
	}
	close IN;	
}








# =============================
# Function: parse default Bowtie output
# Input: default Bowtie output 
# Return: filename array.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub parser5 {	
	
	my $infile 	 = shift or die;
	my $hash_map = shift or die;	
	my $hash_tag = shift or die;
	my $hash_maptime = shift or die;
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}
	
	open IN,"<$infile" or die;
	while( defined( my $line = <IN> ) ){
		chomp( $line  );
		next if $line =~ /^\s*$/;	
			
		my @tmp = split /\t/,$line;
		my $id 		= $tmp[0];
		my $strand 	= $tmp[1];
		my $refId	= $tmp[2];
		my $beg		= $tmp[3];
		my $seq	 	= $tmp[4];
		my $mapTime	= $tmp[6];	
		my $mis	  	= $tmp[7];
		my $lng     = length( $seq );	
		
		unless ( defined $mis ) {
			$mis = 0;
		}	
		
		$beg++;
		my $end = $beg + length($seq) - 1;	
		if ( $strand ne "+" ){
			next;
		}
		
		$mapTime++;
		
		unless(exists $hash_maptime->{$id}){
			$hash_maptime->{$id} = $mapTime;
		}
			
		unless ( exists $hash_map->{$refId}{$id}{$beg}{$end} ) {
			$hash_map->{$refId}{$id}{$beg}{$end} = $mis;
		} else {
			print STDERR "Error:($id,$refId,$beg,$end) repeat\n";
			die;
		}	
		
		unless( exists $hash_tag->{$id} ){
			$hash_tag->{$id} = $seq;
		}
			
	}
	close IN;	
}








# =============================
# Function: parse default Bowtie output
# Input: default Bowtie output 
# Return: filename array.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub parser6 {	
	
	my $infile 	 = shift or die;
	my $hash_map = shift or die;	

	
	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}
	
	open IN,"<$infile" or die;
	while( defined( my $line = <IN> ) ){
		chomp( $line  );
		next if $line =~ /^\s*$/;	
			
		my @tmp = split /\t/,$line;
		my $id 		= $tmp[0];
		my $strand 	= $tmp[1];
		my $refId	= $tmp[2];
		my $beg		= $tmp[3];
		my $seq	 	= $tmp[4];
		my $mapTime	= $tmp[6];	
		my $mis	  	= $tmp[7];
		
		$beg++;
		my $end = $beg + length($seq) - 1;	
			
		unless ( exists $hash_map->{$refId}{$strand}{$beg}{$end} ) {
			$hash_map->{$refId}{$strand}{$beg}{$end} = 1;
		} else {
			$hash_map->{$refId}{$strand}{$beg}{$end}++;
		}	
		
	}
	close IN;	
}






# =============================
# Function: parse default Bowtie output
# Input: default Bowtie output 
# Return: filename array.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub  parser7 {	
	
	my $file = shift or die;
	my $hash_map = shift or die;
	
	my $MIN_LEN = 18;
	my $MAX_LEN = 26;
		
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/MIN_LEN/i) {$MIN_LEN=shift @_}
		if ($argument=~/MAX_LEN/i) {$MAX_LEN=shift @_}
	}


	open IN,"<$file" or die;
	while( defined( my $line = <IN> ) ){
		chomp( $line  );
		next if $line =~ /^\s*$/;	
			
		my @tmp = split /\t/,$line;
		my $id 		= $tmp[0];
		my $strand 	= $tmp[1];
		my $chrom	= $tmp[2];
		my $beg		= $tmp[3];
		my $seq	 	= $tmp[4];
		my $mapTime	= $tmp[6];	
		my $mis	  	= $tmp[7];
		my $lng     = length( $seq );	
		
		unless ( defined $mis ) {
			$mis = 0;
		}	
		
		$beg++;
		my $end = $beg + length($seq) - 1;	
		if ( $strand ne "+" ){
			next;
		}
		
		my $count = MyFasta::getTagFreq($id);
		$mapTime++;
		$count = MyMath::round($count/$mapTime,6);
		
		
		next if ( $lng < $MIN_LEN or $lng > $MAX_LEN );	
		
		unless ( exists $hash_map->{$chrom}{$strand}{$beg}{$end} ){
			$hash_map->{$chrom}{$strand}{$beg}{$end} = $count;
		} else {
			$hash_map->{$chrom}{$strand}{$beg}{$end}+= $count;
		}
	
	}
	close IN;	
}










# =============================
# Function: parse default Bowtie output
# Input: default Bowtie output 
# Return: filename array.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub  parser8 {	
	
	my $file = shift or die;
	my $hash_map = shift or die;
	
	open IN,"<$file" or die;
	while( defined( my $line = <IN> ) ){
		chomp( $line  );
		next if $line =~ /^\s*$/;	
			
		my @tmp = split /\t/,$line;
		my $id 		= $tmp[0];
		my $strand 	= $tmp[1];
		my $refId	= $tmp[2];
		my $beg		= $tmp[3];
		my $seq	 	= $tmp[4];
		my $mapTime	= $tmp[6];	
		my $mis	  	= $tmp[7];
	
		unless ( defined $mis ) {
			$mis = 0;
		}
		
		$beg++;
		my $end = $beg + length($seq) - 1;	
		
		unless ( exists $hash_map->{$id}{$refId} ){
			$hash_map->{$id}{$refId} = $mis;
		} 
	}
	close IN;	
}





# =============================
# Function: parse default Bowtie output
# Input: default Bowtie output 
# Return: filename array.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub  parser9 {	
	
	my $file = shift or die;
	my $hash_map = shift or die;
	
	
	my $MIN_LEN = 0;
	my $MAX_LEN = 10000;
	my $multimapping = 1;
	
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/MIN_LEN/i) {$MIN_LEN=shift @_}
		if ($argument=~/MAX_LEN/i) {$MAX_LEN=shift @_}
		if ($argument=~/multimapping/i) {$multimapping=shift @_}
	}
	
		
	open IN,"<$file" or die;
	while( defined( my $line = <IN> ) ){
		chomp( $line  );
		next if $line =~ /^\s*$/;	
			
		my @tmp = split /\t/,$line;
		my $id 		= $tmp[0];
		my $strand 	= $tmp[1];
		my $chrom	= $tmp[2];
		my $beg		= $tmp[3];
		my $seq	 	= $tmp[4];
		my $mapTime	= $tmp[6];	
		my $mis	  	= $tmp[7];
		my $lng     = length( $seq );	
		
		unless ( defined $mis ) {
			$mis = 0;
		}	
		
		$beg++;
		my $end = $beg + length($seq) - 1;	
		
		next if ( $lng < $MIN_LEN or $lng > $MAX_LEN );	
		
		my $count = MyFasta::getTagFreq($id);
			
			
		if ( $multimapping ){
			$mapTime++;
			$count = MyMath::round($count/$mapTime,6);
		}
		
		
		unless ( exists $hash_map->{$chrom}{$strand}{$beg}{$end} ){
			$hash_map->{$chrom}{$strand}{$beg}{$end} = $count;
		} else {
			$hash_map->{$chrom}{$strand}{$beg}{$end}+= $count;
		}
	
	}
	close IN;	
}





# =============================
# Function: parse default Bowtie output
# Input: default Bowtie output 
# Return: filename array.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub  parser10 {	
	
	my $file = shift or die;
	my $hash_map = shift or die;
	
	
	my $MIN_LEN = 0;
	my $MAX_LEN = 10000;
	my $multimapping = 1;
	
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/MIN_LEN/i) {$MIN_LEN=shift @_}
		if ($argument=~/MAX_LEN/i) {$MAX_LEN=shift @_}
		if ($argument=~/multimapping/i) {$multimapping=shift @_}
	}
	
	
	open IN,"<$file" or die;
	while( defined( my $line = <IN> ) ){
		chomp( $line  );
		next if $line =~ /^\s*$/;	
			
		my @tmp = split /\t/,$line;
		my $id 		= $tmp[0];
		my $strand 	= $tmp[1];
		my $chrom	= $tmp[2];
		my $beg		= $tmp[3];
		my $seq	 	= $tmp[4];
		my $mapTime	= $tmp[6];	
		my $mis	  	= $tmp[7];
		my $lng     = length( $seq );	
		
		unless ( defined $mis ) {
			$mis = 0;
		}	
		
		$beg++;
		my $end = $beg + length($seq) - 1;	
		
		if ( $mapTime ){
			next;
		}
		
		next if ( $lng < $MIN_LEN or $lng > $MAX_LEN );	
		
		my $count = MyFasta::getTagFreq($id);
		
		
		if ( $multimapping ){
			$mapTime++;
			$count = MyMath::round($count/$mapTime,6);
		}
		
		
		unless ( exists $hash_map->{$chrom}{$strand}{$beg}{$end} ){
			$hash_map->{$chrom}{$strand}{$beg}{$end} = $count;
		} else {
			$hash_map->{$chrom}{$strand}{$beg}{$end}+= $count;
		}
	
	}
	close IN;	
}



# =============================
# Function:execute Bowtie mapping
# Input: reference of array of fasta file
#        reference file
#		 Bowtie command
#		 Bowtie bin path 
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub mapper {
	
	my $ref_file  	= shift or die;
	my $read_files 	= shift or die;
	my $cmd			= shift or die;
	my $bowtie_dir	= shift or die;
		

	# ****************************************************** #
	
	my $dest_dir;
	my $adaptor;
	my $hash_refseq;
	my $analysis = 1;	
	my $del_index = 0;
	my $genome_stat;
	
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/dest_dir/i) {$dest_dir=shift @_}
		if ($argument=~/adaptor/i) {$adaptor=shift @_}
		if ($argument=~/hash_refseq/i) {$hash_refseq=shift @_}
		if ($argument=~/analysis/i) {$analysis=shift @_}
		if ($argument=~/del_index/i) {$del_index=shift @_}
		if ($argument=~/genome_stat/i) {$genome_stat=shift @_}
	}
	
	print "\n\nBowtie cmd:$cmd\n\n";
	
	# ****************************************************** #
	
	unless( defined $cmd ){
		print STDERR "Error:cmd not defined\n\n\n";
		die;
	}
	unless(-e "$bowtie_dir/bowtie"){
		print STDERR "Error:bowtie dir not exists\n";
		die;
	}
	
	# ****************************************************** #
	
	my $given_mis = "";
	if($cmd=~/-v\s+(\d+)/){
		my $mismatch = $1;
		if ( MyMath::is_numeric($mismatch) ){
			$given_mis = $mismatch;
		} else {
			print STDERR "Error:$mismatch not a numeric\n";
			die;
		}
	} else {
		$given_mis = 2;
	}
	
	# ****************************************************** #
	
	
	my @suffix = qw(.fa .fq .txt);
	my $ref_dirname = dirname($ref_file);
	my $ref_basename = basename($ref_file,@suffix);
	my $ebwt_base_name = "$ref_dirname/$ref_basename";
	my @ebwt_files = ();
	
	# check Bowtie indexes
	my $build_flag = 1;
	for my $i ( 1 .. 4 ){
		unless( -e $ebwt_base_name.".$i.ebwt" ){
			push(@ebwt_files,$ebwt_base_name.".$i.ebwt");
			$build_flag = 0;
		}
	}
	unless( -e $ebwt_base_name.".rev.1.ebwt" ){
		push(@ebwt_files,$ebwt_base_name.".rev.1.ebwt");
		$build_flag = 0;
	}
	unless( -e $ebwt_base_name.".rev.2.ebwt" ){	
		push(@ebwt_files,$ebwt_base_name.".rev.2.ebwt");		
		$build_flag = 0;
	}
	
	# build index	
	unless( $build_flag ){							
		my $ret_build = `$bowtie_dir/bowtie-build -f $ref_file $ebwt_base_name`;					
	}
	
	if ( $build_flag ){
		$del_index = 0;
	}
	
	# ****************************************************** #
	
	my $current_dir = getcwd;
	chdir($bowtie_dir);
	
	
	
		
	# mapping	
	foreach my $read_file ( @$read_files ){	
	
		if ( &is_fastq_fasta($read_file) eq "fasta" ){
			if ($cmd !~ /\b-f\b/){
				$cmd .= " -f ";
			}
		} else {
			$cmd =~ s/\b-f\b//ig;
		}
			
		my $read_basename = basename($read_file,@suffix);
		my ($outfile_mapping,$outfile_mapped,$outfile_unmapped);
		if ( defined $dest_dir ){
			$outfile_mapping  = "$dest_dir/$ref_basename.$read_basename.mapping.txt";
			$outfile_mapped   = "$dest_dir/$ref_basename.$read_basename.mapped.fa";
			$outfile_unmapped = "$dest_dir/$ref_basename.$read_basename.unmapped.fa";
		} else {
			my $dirname = dirname($read_file);
			$outfile_mapping  = "$dirname/$ref_basename.$read_basename.mapping.txt";
			$outfile_mapped   = "$dirname/$ref_basename.$read_basename.mapped.fa";
			$outfile_unmapped = "$dirname/$ref_basename.$read_basename.unmapped.fa";		
		}
	
		die if system("$bowtie_dir/$cmd --un $outfile_unmapped --al ".
		"$outfile_mapped $ebwt_base_name $read_file $outfile_mapping");
		
		print "\n\n";
		
	}
	
	chdir($current_dir);
	
	
	if ( $analysis ){
	
		# ****************************************************** #
		
		# parse	
		my $header 			= "";
		my $read_dirname 	= "";
		my @sampleNames 	= ();
		my %hash_stat   	= ();
		my %hash_locus  	= ();
		my %hash_locus_sum 	= ();
		
		foreach my $read_file ( @$read_files ){	
		
			my $read_basename = basename($read_file,@suffix);
			
			my ($outfile_mapping,$outfile_mapped,$outfile_unmapped);
			if ( defined $dest_dir ){
				$read_dirname = $dest_dir;
				$outfile_mapping  = "$dest_dir/$ref_basename.$read_basename.mapping.txt";
				$outfile_mapped   = "$dest_dir/$ref_basename.$read_basename.mapped.fa";
				$outfile_unmapped = "$dest_dir/$ref_basename.$read_basename.unmapped.fa";
			} else {		
				my $dirname = dirname($read_file);
				$read_dirname = $dirname;
				$outfile_mapping  = "$dirname/$ref_basename.$read_basename.mapping.txt";
				$outfile_mapped   = "$dirname/$ref_basename.$read_basename.mapped.fa";
				$outfile_unmapped = "$dirname/$ref_basename.$read_basename.unmapped.fa";
			}
				
			
			my $sample = $read_basename;
			$header .= "\t$sample";
			push(@sampleNames,$sample);
		
			unless(-e $outfile_mapping){
				print STDERR "Error:$outfile_mapping not exists\n";
				die;
			}
			
			my %hash_dis = ();
			if ( (defined $hash_refseq) and (defined $adaptor) ){
				&parser1($outfile_mapping,\%hash_dis,$given_mis,$adaptor,$hash_refseq);
			} else {				
				&parser1($outfile_mapping,\%hash_dis);
			}
			
			# total 
			my ($total_read,$total_tag);		
			if( $cmd =~ /\s+-f\s+/ ){
				($total_read,$total_tag) = MyFasta::countTag($read_file);
			} else {
				$total_read = MyFasta::countfastq($read_file);
				$total_tag  = $total_read;
			}
			
			$hash_stat{total_tag}{$sample} = $total_tag;
			$hash_stat{total_read}{$sample} = $total_read;
			
			# mapped 
			if(-e $outfile_mapped){
				my ($mapped_read,$mapped_tag);
				if( $cmd =~ /\s+-f\s+/ ){
					($mapped_read,$mapped_tag) = MyFasta::countTag($outfile_mapped);
				} else {
					$mapped_read = MyFasta::countfastq($outfile_mapped);
					$mapped_tag  = $mapped_read;
				}
				$hash_stat{mapped_tag}{$sample} = $mapped_tag;
				$hash_stat{mapped_read}{$sample} = $mapped_read;
				$hash_stat{mapped_tag_ratio}{$sample} = MyMath::round($mapped_tag/$total_tag,6);
				$hash_stat{mapped_read_ratio}{$sample} = MyMath::round($mapped_read/$total_read,6);
			} else {
				$hash_stat{mapped_tag}{$sample} = 0;
				$hash_stat{mapped_read}{$sample} = 0;
				$hash_stat{mapped_tag_ratio}{$sample} = 0;
				$hash_stat{mapped_read_ratio}{$sample} = 0;
			}
			
			# unmapped
			if(-e $outfile_unmapped){
				my ($unmapped_read,$unmapped_tag);
				if( $cmd =~ /\s+-f\s+/ ){
					($unmapped_read,$unmapped_tag) = MyFasta::countTag($outfile_unmapped);	
				} else {
					$unmapped_read = MyFasta::countfastq($outfile_unmapped);
					$unmapped_tag  = $unmapped_read;
				}
				$hash_stat{unmapped_tag}{$sample} = $unmapped_tag;
				$hash_stat{unmapped_read}{$sample} = $unmapped_read;
			} else {
				$hash_stat{unmapped_tag}{$sample} = 0;
				$hash_stat{unmapped_read}{$sample} = 0;
			}
			
			# locus.
			my $sum_tag = 0;
			my $sum_read = 0;
			for my $refId ( sort keys %hash_dis ){	
				$sum_tag += $hash_dis{$refId}{tag};
				$sum_read += $hash_dis{$refId}{read};
				$hash_locus{$refId}{$sample}{tag} = $hash_dis{$refId}{tag};	
				$hash_locus{$refId}{$sample}{read} = $hash_dis{$refId}{read};
			}
			$hash_locus_sum{$sample}{tag} = $sum_tag;
			$hash_locus_sum{$sample}{read} = $sum_read;
		
		}
		
		# ****************************************************** #
		
		my $outfile_mapping_stat = "$read_dirname/$ref_basename.stat.txt";
		open  STAT,">$outfile_mapping_stat" or die;
		print STAT "Item$header\n";
		
		foreach my $item ( sort keys %hash_stat ){
			my $str = $item;
			foreach my $sample ( @sampleNames ){
				if(exists $hash_stat{$item}{$sample}){
					$str .= "\t$hash_stat{$item}{$sample}";
				} else {
					$str .= "\t0";
				}
			}
			print STAT "$str\n";
		}
		
		close STAT;
				
		# ****************************************************** #
		
		$header = "Locus".$header;
		
		my $outfile_tag_distribution  = "$read_dirname/$ref_basename.tag.txt";
		my $outfile_read_distribution = "$read_dirname/$ref_basename.read.txt";
		open TAG,">$outfile_tag_distribution" or die;
		open READ,">$outfile_read_distribution" or die;
		
		my $outfile_tag_percent_distribution  = "$read_dirname/$ref_basename.tag(percent).txt";
		my $outfile_read_percent_distribution = "$read_dirname/$ref_basename.read(percent).txt";
		open TAGPER,">$outfile_tag_percent_distribution" or die;
		open READPER,">$outfile_read_percent_distribution" or die;
		
		print TAG "$header\n";
		print READ "$header\n";
		
		print TAGPER "$header\n";
		print READPER "$header\n";
		
		foreach my $refId ( sort keys %hash_locus ){
			
			my $str_tag = $refId;
			my $str_read = $refId;
			
			my $str_tag_per = $refId;
			my $str_read_per = $refId;
			
			foreach my $sample ( @sampleNames ){
				
				if( (exists $hash_locus{$refId}{$sample}{tag}) and ($hash_locus_sum{$sample}{tag} > 0) ){
					$str_tag .= "\t$hash_locus{$refId}{$sample}{tag}";
					my $per = MyMath::round($hash_locus{$refId}{$sample}{tag}/$hash_locus_sum{$sample}{tag},6);	
					$str_tag_per .= "\t$per";
				} else {
					$str_tag .= "\t0";
					$str_tag_per .= "\t0";
				}
				
				if( (exists $hash_locus{$refId}{$sample}{read}) and ($hash_locus_sum{$sample}{read} > 0)){
					$str_read .= "\t$hash_locus{$refId}{$sample}{read}";
					my $per = MyMath::round($hash_locus{$refId}{$sample}{read}/$hash_locus_sum{$sample}{read},6);
					$str_read_per .= "\t$per";
				} else {
					$str_read .= "\t0";
					$str_read_per .= "\t0";
				}
			}
			print TAG "$str_tag\n";
			print READ "$str_read\n";
			
			print TAGPER "$str_tag_per\n";
			print READPER "$str_read_per\n";
		}
		
		close TAG;
		close READ;
		
		close TAGPER;
		close READPER;
		
		# ****************************************************** #
			
		if ( (defined $genome_stat) && (-e $genome_stat) ){	
			
			my %hash_row = ();
			MyMath::toMatrix($genome_stat,\%hash_row);
			
			my $outfile_tag_RPM_distribution  = "$read_dirname/$ref_basename.tag.RPM.txt";
			my $outfile_read_RPM_distribution = "$read_dirname/$ref_basename.read.RPM.txt";
			open TAG,">$outfile_tag_RPM_distribution" or die;
			open READ,">$outfile_read_RPM_distribution" or die;
			
			print TAG "$header\n";
			print READ "$header\n";
					
			foreach my $refId ( sort keys %hash_locus ){
						
				my $str_tag = $refId;
				my $str_read = $refId;	
							
				foreach my $sample ( @sampleNames ){					
					if( (exists $hash_locus{$refId}{$sample}{tag}) and ($hash_locus_sum{$sample}{tag} > 0) ){
						my $t = $hash_locus{$refId}{$sample}{tag} / $hash_row{mapped_tag}{$sample};
						$t *= 1000000;
						$t = MyMath::round($t,6);
						$str_tag .= "\t$t";
					} else {
						$str_tag .= "\t0";
					}
					
					if( (exists $hash_locus{$refId}{$sample}{read}) and ($hash_locus_sum{$sample}{read} > 0)){
						my $t = $hash_locus{$refId}{$sample}{read} / $hash_row{mapped_read}{$sample};
						$t *= 1000000;
						$t = MyMath::round($t,6);
						$str_read .= "\t$t";
					} else {
						$str_read .= "\t0";
					}
				}
				
				print TAG "$str_tag\n";
				print READ "$str_read\n";
			}
			
			close TAG;
			close READ;	
		}
			
		# ****************************************************** #
		
	}
	
	# ****************************************************** #
	
	if ( $del_index ){
		for my $file ( @ebwt_files ){
			unlink($file);
		}
	}
	
	# ****************************************************** #
}


# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub is_fastq_fasta {
	
	my $infile = shift or die;
	
	unless( -e $infile ){
		print STDERR "Error:$infile not exists\n";
		die;
	}
		
	my $str;
	open IN,"<$infile" or die;
	while( <IN> ){
		chomp;
		next if /^\s*$/;
		if( /^>/ ){
			$str = "fasta";
			last;
		} elsif ( /^\@/ ) {
			$str = "fastq";
			last;
		} else {
			print STDERR "Error:$infile format error\n";
			print STDERR "Only fastq and fasta format are valid\n"; 
			die;
		}
	}
	close IN;
	
	return $str;
	
}

		
##############################################################
1;		
		
		
		
		
		
	
		
	