package Feature;
#!/usr/bin/perl
use strict;
use warnings;
use File::Copy;
use File::Basename;

# My Library
use MyMath;
use Libsvm;
use MyFile;




# =============================
# Function: parse the output structure of RNAfold
# Input: a RNAfold structure file 
#		 (output) Reference of hash for structure
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub parse_struct {
	
	my $infile_struct = shift or die;
	my $hash_struct	= shift or die;
	
	unless(-e $infile_struct){
		print STDERR "Error:$infile_struct not exists\n";
		die;
	}
	
	open IN,"<$infile_struct" or die;
	
	my $id = "";
	my %hash_tmp = ();
	
	while( defined(my $line = <IN> ) ){
		chomp( $line );
		next if $line =~ /^\s*$/;	
			
		if( $line =~/>/ ){
			$id = substr($line,1);
		} elsif ( $line =~ /[ACGUTN]+/i ) {
			
			$line = uc( $line );
			$hash_tmp{$id}{seq} = $line;
				
		} elsif ( $line =~ /\.|\(|\)/  ){				
			
			my @tmp = split /\s+\(/,$line;	
			$hash_tmp{$id}{struct} = $tmp[0];
			
			my $mfe = $tmp[1];
			$mfe =~ s/\(|\)//g;			
			$hash_tmp{$id}{mfe}= $mfe;
			
		}
	}
	
	close IN;
	
	
	for my $id ( keys %hash_tmp ){
		
		my $mfe = $hash_tmp{$id}{mfe};
		my $seq = $hash_tmp{$id}{seq};
		my $struct = $hash_tmp{$id}{struct};
		
		unless( $struct =~ /\(/ ) {
			next;
		}
		
		unless( $struct =~ /\)/ ) {
			next;
		}
		
		if (length($struct) != length($seq)){
			next;
		}
		
		unless( MyMath::is_numeric($mfe) ) {
			next;
		}
		
		$hash_struct->{$id}{mfe} = $mfe;
		$hash_struct->{$id}{seq} = $seq;
		$hash_struct->{$id}{struct} = $struct;
		
	}

}




# =============================
# Function: whether bifurcation exists
# Input: a structure string 
# Return: 1 (bifurcation) or 0 (no bifurcation)
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub is_bifurcation {
	
    my $struct	= shift or die;
    
    unless ( $struct =~ /^((\.|\()+..(\.|\))+)$/ ){
    	return 1;
    } else {
		return 0; 
	}
 
}


# =============================
# Function: make a hash table for paired bases
# Input: a structure 
#		 (output) Reference of hash for paired base
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub basePair { 
	
	my $struct  = shift or die;
	my $hash_bp	= shift or die;
	
	my @strs = split('', $struct);
	
	my @leftBrackets = ();
	
	for my $i ( 0 .. $#strs ) {
		if ( $strs[$i] eq "(" ) {
			push @leftBrackets, $i+1;
		} elsif ( $strs[$i] eq ")" ) {
			my $ch = pop @leftBrackets;
			my $rp = $i + 1;
			$hash_bp->{ $ch } = $rp;
			$hash_bp->{ $rp } = $ch;
		}
	}
	
	if( @leftBrackets ){
		warn "unbalanced $struct\n";
	}
}





# =============================
# Function: translate codon into amino acid
# Input: a triplet 
# Return: amino acid
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub codon2aa {
	
	my $codon = shift or die;
	
	$codon =~ s/\s*//g;
	$codon =~ tr/[acgtuUn\.]/[ACGTTTNN]/;
	
	my %hash_translation = (
	'TCA'=>'S','TCC'=>'S','TCG'=>'S','TCT'=>'S','TTC'=>'F','GGT'=>'G','TTT'=>'F',
	'TTA'=>'L','TTG'=>'L','TAC'=>'Y','TAT'=>'Y','TAA'=>'-','TAG'=>'-','TGC'=>'C',
	'TGT'=>'C','TGA'=>'-','TGG'=>'W','CTA'=>'L','CTC'=>'L','CTG'=>'L','CTT'=>'L',
	'CCA'=>'P','CCC'=>'P','CCG'=>'P','CCT'=>'P','CAC'=>'H','CAT'=>'H','CAA'=>'Q',
	'CAG'=>'Q','CGA'=>'R','CGC'=>'R','CGG'=>'R','CGT'=>'R','ATA'=>'I','ATC'=>'I',
	'ATT'=>'I','ATG'=>'M','ACA'=>'T','ACC'=>'T','ACG'=>'T','ACT'=>'T','AAC'=>'N',
	'AAT'=>'N','AAA'=>'K','AAG'=>'K','AGC'=>'S','AGT'=>'S','AGA'=>'R','AGG'=>'R',
	'GTA'=>'V','GTC'=>'V','GTG'=>'V','GTT'=>'V','GCA'=>'A','GCC'=>'A','GCG'=>'A',
	'GCT'=>'A','GAC'=>'D','GAT'=>'D','GAA'=>'E','GAG'=>'E','GGA'=>'G','GGC'=>'G',
	'GGG'=>'G'
	);
	
	if( defined $hash_translation{$codon} ){
		return $hash_translation{$codon};
	} else {
		return "-";
	}
	
}






# =============================
# Function: Needleman-Wunsch alignment
# Input: two sequences
# Return: alignment result 
# Date: 2013-2-20
# =============================

sub Needleman_Wunsch {
	
	my $seqA  = shift or die;
	my $seqB  = shift or die;

	
	my $MATCH     =  1; 
	my $MISMATCH  = -1;
	my $GAP       = -1; 


	my @lattice = ();
	$lattice[0][0]{score}   = 0;
	$lattice[0][0]{pointer} = "none";
	
	for(my $j = 1; $j <= length($seqA); $j++) {
	    $lattice[0][$j]{score}   = $GAP * $j;
	    $lattice[0][$j]{pointer} = "left";
	}
	
	for (my $i = 1; $i <= length($seqB); $i++) {
	    $lattice[$i][0]{score}   = $GAP * $i;
	    $lattice[$i][0]{pointer} = "up";
	}

	
	for(my $i = 1; $i <= length($seqB); $i++) {
	    for(my $j = 1; $j <= length($seqA); $j++) {
	        my ($diagonal_score, $left_score, $up_score);
   
	        my $letterA = substr($seqA, $j-1, 1);
	        my $letterB = substr($seqB, $i-1, 1);     
	                               
	        if ($letterA eq $letterB) {
	            $diagonal_score = $lattice[$i-1][$j-1]{score} + $MATCH;
	         } else {
	            $diagonal_score = $lattice[$i-1][$j-1]{score} + $MISMATCH;
	         }

	        
	        $up_score   = $lattice[$i-1][$j]{score} + $GAP;
	        $left_score = $lattice[$i][$j-1]{score} + $GAP;

	        
	        if ($diagonal_score >= $up_score) {
	            if ($diagonal_score >= $left_score) {
	                $lattice[$i][$j]{score}   = $diagonal_score;
	                $lattice[$i][$j]{pointer} = "diagonal";
	             } else {
	                $lattice[$i][$j]{score}   = $left_score;
	                $lattice[$i][$j]{pointer} = "left";
	             }
	         } else {
	            if ($up_score >= $left_score) {
	                $lattice[$i][$j]{score}   = $up_score;
	                $lattice[$i][$j]{pointer} = "up";
	             } else {
	                $lattice[$i][$j]{score}   = $left_score;
	                $lattice[$i][$j]{pointer} = "left";
	             }
	         }
	     }
	}


	my $alignA = "";
	my $alignB = "";

	my $j = length($seqA);
	my $i = length($seqB);

	while ( 1 ) {
		
	    last if $lattice[$i][$j]{pointer} eq "none"; 

	    if ($lattice[$i][$j]{pointer} eq "diagonal") {
	        $alignA .= substr($seqA, $j-1, 1);
	        $alignB .= substr($seqB, $i-1, 1);
	        $i--;
	        $j--;
	     } elsif ($lattice[$i][$j]{pointer} eq "left") {
	        $alignA .= substr($seqA, $j-1, 1);
	        $alignB .= "-";
	        $j--;
	     } elsif ($lattice[$i][$j]{pointer} eq "up") {
	        $alignA .= "-";
	        $alignB .= substr($seqB, $i-1, 1);
	        $i--;
	     }    
	     
	}

	$alignA = reverse $alignA;
	$alignB = reverse $alignB;
	
	return ($alignA,$alignB);
}






# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
# ++++++++++++++++++++++ sequence feature +++++++++++++++++ #
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #






# =============================
# %(G+C) = (|C|+|G|)/L * 100
# Function: calculate the percent of G and C base in the sequence
# Input: a string of RNA or DNA
# Return: GC content
# Reference: microPred: effective classification 
# of pre-miRNAs for human miRNA gene prediction
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub GC_content {	
	
	my $seq = shift or die;
	
	my @t = split ( '', $seq );	
	
	my $m = 0;
	my $n = 0;
	
	for my $base ( @t ) {
		if ( $base eq "G" or $base eq "g" ) {
			$m++;
		} elsif ( $base eq "C" or $base eq "c" ) {
			$m++;
		}
		$n++;
	}
	
	return ($m/$n) * 100;
	
}





# =============================
# dinucleotide frequencies
# %XY = |XY| * 100 / ( L - 1 )
# where X and Y belong to (A C G U)
# Function: calculate dinucleotide frequencies
# Input: a string of structure dinucleotide
#		 reference of array for output
# Reference: Evidence that microRNA precursors, 
# unlike other non-coding RNAs, 
# have lower folding free energies than random sequences
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub dinucleotide_frequency {
	
	my $seq = shift or die;
	my $dinucleotide = shift or die;
	my $dinucleotide_str = shift or die;
	
	$seq = uc($seq);
	$seq =~ tr/T/U/;
	
	my $n = 0;
	my @tmp = ();
	my @bases = qw ( A C G U );
	for my $i ( @bases ){
		for my $j ( @bases ){
			$tmp[$n++] = $i.$j;
		}
	}
		
	my $i = 0;
	for my $di ( @tmp ){
		my $n = 0;
		while( $seq =~ /$di/g ) {
			$n++;
		}
		$dinucleotide->[$i] = ($n/(length($seq)-1)) * 100;
		$dinucleotide_str->[$i] = $di;
		$i++;
	}
	
	if ( $i != 16 ){
		print STDERR "Error: lack dinucleotide\n";
		die;
	}
	
}





# =============================
# Function: calculate k-mer ocurrence frequency 
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub k_mer {
	
	my $k = shift or die;
	my $seq = shift or die;
	my $hash_kmer = shift or die;
	
	if ( $k < 0 ){
		print STDERR "Error:$k < 0\n";
		die;
	}
		
	$seq =~ tr/[acgtTun\.]/[ACGUUUNN]/;	
	
	my @bases = qw( A C G U );
	
	my @querys = ();
	MyMath::permutation(\@bases,\@querys,$k-1);
	
	my $all_possible_k_mer = MyMath::factorial( $k );
	
	for my $query ( @querys ) {
		my $n = 0;
		my $i = 0;
		while($i < length($seq) ){		
			my $subseq = substr($seq,$i,$k);
			if ( $subseq eq $query ){
				$n++;
			}
			$i++;
		}
	
		unless(exists $hash_kmer->{$query}){
			$hash_kmer->{$query} = $n/$all_possible_k_mer;
		} else {
			print STDERR "Error:$query repeat\n";
			die;
		}
	}	
}





# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
# ++++++++++++++++++++++ structure feature +++++++++++++++++ #
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #




# =============================
# Function: the size of biggest bulge in structure
# Input: a structure
# Return: max of bulge
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub biggest_bulge_size {
	
	my $struct = shift or die;
	
	my $max_bulge_size = 0;
	while ( $struct =~ /(\.+)/g ) {
		my $size = length $1;
		if ( $size > $max_bulge_size ) {
			$max_bulge_size = $size;
		}
	}
	
	return $max_bulge_size;
	
}



# =============================
# Function: get asymmetry of structure
# Input: a structure
# Return: asymmetry
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub asymmetry {
	
	my $struct = shift or die;
	
	my %hash_bp = ();
	&basePair($struct,\%hash_bp);
	
	my $sum = 0;
	my ($ip,$jp);
	
	for my $i ( 0 .. length( $struct ) ) {
		if ( defined $hash_bp{$i} ) {
			my $j = $hash_bp{$i};
			if ( defined $ip and defined $jp ) {
				$sum += abs( ( $i - $ip ) + ( $j - $jp ) );
			}
			$ip = $i;
			$jp = $j;
		}
	}
	
	return $sum;
	
}




# =============================
# Function: calculate the ratio of G and C base in the sequence
# Input: a structure
# Return: ratio of paired base and unpaired base
# 		  ratio of unpaired base to paired base
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub paired_unpaired_base {
	
	my $struct = shift or die;
	
	my $sum = 0;
	while( $struct =~ /\(|\)|\./g ) {
		$sum++;
	}
	
	my $pair = 0;
	while( $struct =~ /\(|\)/g ) {
		$pair++;
	}
	
	my $unpair = 0;
	while( $struct =~ /\./g ) {
		$unpair++;
	}
	
	my $pair2unpairRatio = 0;
	if( $unpair ){
		$pair2unpairRatio = $pair/$unpair;
	}
	
	return ($pair/$sum,$unpair/$sum,$pair2unpairRatio);
	
}







# =============================
# Function: calculate the distribution of 
# unpaired base to paired base in secondary structure
# Input: a structure
#		 the number of bin
#  		 reference of array for distribution
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub unpaired2paired_distribution {
	
	my $struct 	= shift or die;
	my $nbin 	= shift or die;
	my $dis		= shift or die;
	
	
	my $bin = (length $struct) / $nbin;	
	if( $bin - int( $bin ) < 1.e-10 ){
		$bin = int ( $bin );
	} else {
		$bin = int( $bin ) + 1;
	}
	
	my $i = 0;
	my $j = 0;
	
	while ( $i < (length $struct) ) {
		
		my $substruct = substr( $struct, $i, $bin );
				
		my $pair = 0;
		while( $substruct =~ /\(|\)/g ) {
			$pair++;
		}
		
		my $unpair = 0;
		while( $substruct =~ /\./g ) {
			$unpair++;
		}	
		
		my $ratio = 0;
		if( $pair ){
			$ratio = $unpair / $pair;	
		}
		
		$dis->[$j] = $ratio;
		$i += $bin;
		$j++;
		
	}	
	
	while ( $j < $nbin  ){
		$dis->[$j++] = 0;
	}	
	
}

	




# =============================
# Function: calculate the maximum of consecutive paired base
# Input:  a string of structure
# Return: asymmetry
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub biggest_consecutive_paired_base {
	
	my $struct = shift or die;
	
	my %hash_bp = ();
	&basePair($struct,\%hash_bp);
	
	my $max_consecutive_lng = 0;
	for my $i ( sort {$a<=>$b} keys %hash_bp ){
		my $j = $i;
		while ( $j <= (length $struct) ){
			if ( defined $hash_bp{$j} ){
				$j++;
			} else {
				last;
			}
		}
		if ( $max_consecutive_lng < $j - $i ){
			$max_consecutive_lng = $j - $i;
		}
	}	
	
	return $max_consecutive_lng;
	
}



# =============================
# A stem is a structural motif containing more than three contiguous base pairs.
# n_stems is the number of stems in the secondary structure. 
# Function: calculate n_stems
# Input: structure
# Return: n_stems
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub count_n_stems {
	
	my $struct = shift or die;
	
	my $n = 0;
	while( $struct =~ /(\(\(\()+/g ) {
		$n++;
	}
	
	my $m = 0;
	while( $struct =~ /(\)\)\))+/g ) {
		$m++;
	}

	my $n_stem = $n > $m ? $n : $m;
	
	return $n_stem;
	
}









# =============================
# A bulge is a structural motif containing more than three contiguous unpaired bases.
# n_bulges is the number of bulge in the secondary structure. 
# Function: calculate n_bulges
# Input: structure
# Return: n_bulges
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub count_n_bulges {
	
	my $struct = shift or die;
	
	my $n = 0;
	while( $struct =~ /(\.\.\.)+/g ) {
		$n++;
	}
	
	return $n;
	
}






# =============================
# Normalized base-paring propensity (dP)
# dP = tot_bases / L
# where tot_bases is the total number of base pairs in the secondary structure
# Function: calculate Normalized base-paring propensity
# Input: structure
# Return: n_stems
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub normalized_base_paring_propensity {
	
	my $struct = shift or die;
	
	my $tot_bases = 0;
	while( $struct =~ /\)/g ){
		$tot_bases++;
	}
	
	return $tot_bases /(length $struct);
	
}






# =============================
# n _loops is the number of loops in the secondary structure
# Function: calculate n _loops 
# Input: structure
# Return: n_loops
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub count_n_loops {
	
	my $struct = shift or die;
	
	my %hash_bp = ();
	&basePair($struct,\%hash_bp);
	
	my @t = sort ( {$a<=>$b} keys %hash_bp );
	
	my $n_loops = 0;
	for my $i ( 1 .. $#t ){	
		if( $hash_bp{ $t[$i] } == $t[$i-1] && $hash_bp{ $t[$i-1] } == $t[$i] ) {
			$n_loops++;
		}
	}
	
	return $n_loops;
	
}
	





# =============================
# normalized_n_stems = n_stems / L
# Function: get normalized_n_stems
# Input:  structure
# Return: normalized n_stems
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================


sub normalized_n_stems {
	
	my $struct = shift or die;
	
	my $n_stems = &count_n_stems($struct);
	
	return $n_stems/(length $struct);
	
}







# =============================
# normalized_n_bulges = n_bulges / L
# Function: get normalized_n_bulges
# Input:  structure
# Return: normalized_n_bulges
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================


sub normalized_n_bulges {
	
	my $struct = shift or die;
	
	my $n_bulges = &count_n_bulges($struct);
	
	return $n_bulges/(length $struct);
	
}




# =============================
# normalized_n_loops = n_loops / L
# Function: get normalized_n_loops
# Input:  structure
# Return: normalized_n_loops
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================


sub normalized_n_loops {
	
	my $struct = shift or die;
	
	my $n_loops = &count_n_loops($struct);
	
	return $n_loops/(length $struct);
	
}





# =============================
# Average base pairs per stem:
# avg_bp_stem = tot_bases / n_stems
# where tot_bases is the total number of base pairs in secondary structure
# A stem is a structural motif containing more than three contiguous base pairs
# Function: get avg_bp_stem
# Input:  structure
# Return: avg_bp_stem
# Reference: microPred: effective classification 
# of pre-miRNAs for human miRNA gene prediction
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub avg_bp_stem {
	
	my $struct = shift or die;
	
	my $tot_bases = 0;
	while( $struct =~ /\)/g ){
		$tot_bases++;
	}
	
	my $n_stems = &count_n_stems($struct);
	
	if( $n_stems ){
		return $tot_bases / $n_stems;
	} else {
		return 0;
	}
	
}








# =============================
# Average unpaired bases per bulge:
# avg_unbp_bulge = tot_unpaired_bases / n_bulges
# where tot_unpaired_bases is the total number of unpaired bases in structure
# a bulge is the number of bulge in the secondary structure. 
# Function: get avg_unbp_bulge
# Input:  structure
# Return: avg_unbp_bulge
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub avg_unbp_bulge {
	
	my $struct = shift or die;
	
	my $tot_unpaired_bases = 0;
	while( $struct =~ /\./g ){
		$tot_unpaired_bases++;
	}
	
	my $n_bulges = &count_n_bulges($struct);
	
	if( $n_bulges ){
		return $tot_unpaired_bases / $n_bulges;
	} else {
		return 0;
	}
	
}




	
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
# ++++++++++++++++++++++ structure and sequence feature +++++++++++++++++ #
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #




# =============================
# Normalized base pair counts:
# |A-U|/L, |G-C|/L and |G-U|/L
# where |X-Y| is the number of (X-Y) base pairs 
# in the secondary structure
# 
# AND
#
# %(A-U)/n_stems, %(G-C)/n_stems and %(G-U)/n_stems
# where %(X-Y) = |X-Y|/tot_bases. 
# tot_bases is the total number of base pairs in the secondary structure
# 
# Function: calculate Normalized base pair counts
# Input:  sequence
#	      structure
#         reference of array for normalized base pair
#		  reference of array for normalized base pair per stems
# Reference:microPred: effective classification 
# of pre-miRNAs for human miRNA gene prediction
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub normalized_base_pair_count {
	
	my $seq = shift or die;
	my $struct = shift or die;
	
	my $ref_array_nbp = shift or die;
	my $ref_array_nbp_stem = shift or die;
	
	my $ref_array_nbp_str = shift or die;
	my $ref_array_nbp_stem_str = shift or die;
	


	$seq = uc ( $seq );
	$seq =~ tr /T/U/;
	
	my %hash_bp = ();
	&basePair($struct,\%hash_bp);
	
	
	my %hash_tmp = ();
	for my $i ( sort {$a<=>$b} keys %hash_bp ){
		my $ch_a = substr($seq,$i-1,1);
		my $ch_b = substr($seq,$hash_bp{$i}-1,1);
		my @tmp = ();
		push(@tmp,$ch_a);
		push(@tmp,$ch_b);
		@tmp = sort {$a cmp $b} @tmp;
		my $ch = join('',@tmp);
		unless( exists $hash_tmp{$ch} ){
			$hash_tmp{$ch} = 1;
		} else {
			$hash_tmp{$ch}++;
		}
	}
	
	my %bp_tmp = ();
	$bp_tmp{AU} = 1;
	$bp_tmp{CG} = 1;
	$bp_tmp{GU} = 1;

	for my $bp ( sort keys %bp_tmp ){
		unless ( exists $hash_tmp{ $bp } ){
			$hash_tmp{ $bp } = 0;
		}
	}
	
	
	my $n = 0;
	for my $bp ( sort keys %hash_tmp ){
		$ref_array_nbp->[$n] = $hash_tmp{ $bp } / (length $seq);
		$ref_array_nbp_str->[$n] = $bp;
		$n++;
	}
	
	
	if ( $n != 3 ){
		print STDERR "Error:lack base pair counts ($n,$seq)\n";
		for my $bp ( sort keys %hash_tmp ){
			print STDERR "$bp\n";
		}
		die;
	}
	
	
	# ************************************** #
	
	my $tot_bases = 0;
	while( $struct =~ /\)/g ){
		$tot_bases++;
	}
	
	my $n_stems = &count_n_stems($struct);
	
	$n = 0;
	for my $bp ( sort keys %hash_tmp ){
		if( $n_stems ){
			$ref_array_nbp_stem->[$n] = ($hash_tmp{ $bp } / $tot_bases) / $n_stems;
		} else {
			$ref_array_nbp_stem->[$n] = 0;
		}
		$ref_array_nbp_stem_str->[$n] = $bp;
		$n++;
	}
	
	# ************************************** #
	
	
}
	





# =============================
# Function: extract binary elements
# Input: a string of RNA or DNA
#		 a string of structure
#		 reference of array for result
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub binary {
	
	my $seq = shift or die;
	my $struct = shift or die;
	my $binary_array = shift or die;
	my $binary_array_str = shift or die;
	
	my $i = 0;
	my $k_mer = 3;
	my %binary = ();
	
	if( not ($seq =~/^([A|C|G|T|U|a|c|g|t|u]+)$/) ){
		print "Error: The $seq contains characters others than [acgtuACGTU]\n";
		die;
	}
	
	
	while ( $i < (length $seq) ) {
		
		my $subseq = substr( $seq, $i, $k_mer );
		my $substruct = substr( $struct, $i, $k_mer );
		$i++;
		
		my $aa = &codon2aa($subseq);
		my $bracket_dot = substr($substruct,1,1);
		$bracket_dot =~ s/\)/\(/g;
			
		my $str = $bracket_dot.$aa;
		
		next if length( $str ) < 2;
		
		unless( exists $binary{ $str } ){
			$binary{ $str } = 1;
		} else {
			$binary{ $str }++;
		}	
	}
	
	# ********************************** #
	
	my $k = 0;
	my @bracket_dot = qw/ ( . /;
	my @amino_acid = qw( A R N D C Q E G H I L K M F P S T W Y V - );
	
	my @allbinary = ();
	for my $bracket_dot ( @bracket_dot ) {
		for my $i ( 0 .. $#amino_acid ){
			$allbinary[$k++] = $bracket_dot.$amino_acid[$i];
		}
	}
	
	my $nbinary = 0;
	for my $binary ( @allbinary ){
		unless( exists $binary{ $binary } ) {
			$binary{ $binary } = 0;
		}
		$nbinary++;
	}
	
	# ********************************** #
	
	my $j = 0;
	for my $binary ( keys %binary ) {
		$binary_array->[$j] = $binary{$binary}/$nbinary;
		$binary_array_str->[$j] = $binary;
		$j++;
	}
	
	if ( $j != 42 ){
		print "Error:lack binary ($seq\t$struct)\n";
		die;
	}
	
}






# =============================
# Function: extract triplet elements
# Input: a string of RNA or DNA
#		 a string of structure
#		 reference of array for result
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub triplet {
	
	my $seq = shift or die;
	my $struct = shift or die;
	my $triplet_array = shift or die;
	my $triplet_array_str = shift or die;
	
	my $i = 0;
	my $k_mer = 3;
	my %triplet = ();
	
	if( not ($seq =~/^([A|C|G|T|U|a|c|g|t|u]+)$/) ){
		print "Error: The $seq contains characters others than [acgtuACGTU]\n";
		die;
	}
	
	
	while ( $i < (length $seq) ) {
		
		my $subseq = substr( $seq, $i, $k_mer );
		my $substruct = substr( $struct, $i, $k_mer );
		$i++;
		
		my $ch = substr( $subseq, 1, 1 );
		
		my $str = $ch."$substruct";
		$str =~ s/\)/\(/g;
		
		next if length( $str ) < $k_mer + 1;
		
		unless( exists $triplet{ $str } ){
			$triplet{ $str } = 1;
		} else {
			$triplet{ $str }++;
		}	
	}
	
	# ********************************** #
	
	my $k = 0;
	my @chars = qw( A C G U );
	my @bracket_dot = ( "(((", "((.", "(..", "(.(", ".((", ".(.", "..(", "..." );
	
	my @allTriplet = ();
	for my $base ( @chars ) {
		for my $i ( 0 ..$#bracket_dot ){
			$allTriplet[$k++] = $base.$bracket_dot[$i];
		}
	}
	
	my $ntriplet = 0;
	for my $triplet ( @allTriplet ){
		unless( exists $triplet{ $triplet } ) {
			$triplet{ $triplet } = 0;
		}
		$ntriplet++;
	}
	
	# ********************************** #
	
	my $j = 0;
	for my $triplet ( keys %triplet ) {
		$triplet_array->[$j] = $triplet{$triplet}/$ntriplet;
		$triplet_array_str->[$j] = $triplet;
		$j++;
	}
	
	if ( $j != ( 2 ** $k_mer ) * 4 ){
		print "Error:lack triplet ($seq\t$struct)\n";
		die;
	}
	
}





# =============================
# Function: 
# reverse tructure strings and character strings, then 
# aligne the strings with their reversed counterparts.
# the gaps between these strings were filled with "-"

# the aligned strings were divided into ten equal regions,
# and  the ratios of each nucleotide pair type were calculated as
# input featuresaps between these strings were filled with "-"
# 
# Input:  a string of RNA or DNA
#		  a string of structure
# 		  reference of hash for result
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub reverse_align_bp {
	
	my $seq = shift or die;
	my $struct = shift or die;
	my $n_segment = shift or die;
	my $hash_rabp = shift or die;
	
	
	my $rev_seq = "";
	my $rev_struct = "";
	my @seqs = split ('',$seq);
	my @structs = split ('',$struct);

	# "reverse" sequence and struct.
	for( my $i = $#seqs; $i >= 0; $i-- ){
		$rev_seq .= $seqs[$i];	
		if ( $structs[$i] eq ")" ){
			$rev_struct .= "(";
		} else {
			$rev_struct .= $structs[$i];
		}
	}
	

	# change struct string into sequence string,
	# then excute Needleman Wunsch alignement.
	$struct =~ tr/\)/\(/;
	$struct =~ tr/\(\./AT/;
	$rev_struct =~ tr/\(\./AT/;
	($struct, $rev_struct) = &Needleman_Wunsch( $struct, $rev_struct );
	$struct =~ tr/AT/\(\./;
	$rev_struct =~ tr/AT/\(\./;


	# get the sequence corresponding to struct.
	@seqs = split('',$seq);
	@structs = split('',$struct);
	
	my $j = 0;
	my $result_seq = "";
	
	foreach my $i ( 0 .. $#structs ){
		if ( $structs[$i] eq "-" ){
			$result_seq .= "-";
		} else {
			$result_seq .= $seqs[$j];
			$j++;
		}
	}
	
	
	my @rev_seqs = split('',$rev_seq);
	my @rev_structs = split('',$rev_struct);
	
	$j = 0;
	my $result_rev_seq = "";
	
	foreach my $i ( 0 .. $#rev_structs ){
		if ( $rev_structs[$i] eq "-" ){
			$result_rev_seq .= "-";
		} else {
			$result_rev_seq .= $rev_seqs[$j];
			$j++;
		}
	}
	
	
	
	# ****************************************** #
	
	
	my $strA = uc ( $result_seq );
	my $strB = uc ( $result_rev_seq );
	$strA =~ tr/T/U/;
	$strB =~ tr/T/U/;
	
	
	# all possible combination between A C G U -
	my @bases = qw(A C G U -);	
	my %hash_all_bp = ();
	for my $i ( @bases ) {
		for my $j ( @bases ) {
			if ( $i ne "-" or $j ne "-" ){
				my @tmp = ($i, $j);
				@tmp = sort ( @tmp );
				my $bp = join('',@tmp);
				unless( exists $hash_all_bp{ $bp } ){
					$hash_all_bp{ $bp } = 1;
				} 
			}
		}
	}
	

	if ( length ( $strA ) ne length( $strB ) ){
		print STDERR "Error:the length of two strings is different\n";
		die;
	}
	
	my $string_lng = length ( $strA );
	my $segment_lng = int ( $string_lng / $n_segment ) + 1;
	
	my @arrayA = split ('',$strA);
	my @arrayB = split ('',$strB);
	
	my %hash_total = ();
	for(my $i = 0; $i <= $#arrayA; $i++ ){
		my $t = int ( $i / $segment_lng );

		my $chA = $arrayA[$i];
		my $chB = $arrayB[$i];
		my @tmp = ($chA,$chB);
		@tmp = sort (@tmp);
		my $bp = join('',@tmp);		
		unless ( exists $hash_rabp->{$t}{$bp} ){
			$hash_rabp->{$t}{$bp} = 1;
		} else {
			$hash_rabp->{$t}{$bp}++;
		}
		unless ( exists $hash_total{$bp} ){
			$hash_total{$bp} = 1;
		} else {
			$hash_total{$bp}++;
		}
	}
	

	for my $i (0 ..$n_segment-1){
		for my $j ( sort keys %hash_all_bp ){
			unless ( exists $hash_rabp->{$i}{$j} ){
				$hash_rabp->{$i}{$j} = 0;
			} 
			unless ( exists $hash_total{$j} ){
				$hash_total{$j} = 1;
			}
		}
	}
	
	
	# normalization 
	my $sum = 0;
	for my $i ( sort {$a<=>$b} keys %$hash_rabp ){
		for my $j ( sort  keys %{ $hash_rabp->{ $i } } ){
			$hash_rabp->{ $i }{ $j } /= $hash_total{ $j };
			$sum++;
		}
	}
	
}
	



# +++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
# ++++++++++++++++++++++ energy feature +++++++++++++++++ #
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++ #




# =============================
# normalized minimum free energy of folding( dG ):
# dG = MFE / L
# Function: calculate normalized minimum free energy of folding
# Input: a sequence
#		 minimum folding energy
# Return: normalized minimum free energy
# Reference:microPred: effective classification of 
# pre-miRNAs for human miRNA gene prediction
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub normalized_mfe {
	
	my $struct = shift or die;
	my $mfe = shift or die;
	
	return $mfe/(length($struct));
	
}
	
	




# =============================
# MFE Index 1 = dG / %(G+C)
# Function: calculate MFE Index 1.
# Input: 
#		 minimum folding energy
# Reference:microPred: effective classification 
# of pre-miRNAs for human miRNA gene predictio
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub mfe_index_1 {
	
	my $seq = shift or die;
	my $mfe = shift or die;
	
	my $dG = &normalized_mfe($seq,$mfe);
	
	my $GCc = &GC_content($seq);
	
	if( $GCc ) {
		return $dG / $GCc;
	} else {
		return 0;
	}
	
}



# =============================
# MFE Index 2 = dG / n_stems
# where n_stems is the number of stems in the secondary structure. A stem 
# stems is a structural motif containing more than three contiguous base pairs. 
# Function: calculate MFE Index 2.
# Input: structure
#		 minimum folding energy
# Return: MFE Index 2
# Reference:microPred: effective classification 
# of pre-miRNAs for human miRNA gene predictio
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub mfe_index_2 {
	
	my $struct = shift or die;
	my $mfe = shift or die;
	
	my $dG = &normalized_mfe($struct,$mfe);
	
	my $n_stems = &count_n_stems($struct);
	
	if( $n_stems ){
		return $dG / $n_stems;
	} else {
		return 0;
	}
	
}






# =============================
# MFE Index 3 = dG / n_loops
# n_loops is the number of loops in the secondary structure 
# Function: calculate MFE Index 3.
# Input: structure
#		 minimum folding energy
# Return: MFE Index 3
# Reference:microPred: effective classification 
# of pre-miRNAs for human miRNA gene predictio
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub mfe_index_3 {
	
	my $struct = shift or die;
	my $mfe = shift or die;
	
	my $dG = &normalized_mfe($struct,$mfe);

	my $n_loops = &count_n_loops($struct);
	
	if( $n_loops ){
		return $dG / $n_loops;
	} else {
		return 0;
	}
}
	



# =============================
# MFE Index 4 = dG / tot_bases
# tot_bases is the total number of base pairs in the secondary structure 
# Function: calculate MFE Index 4
# Input: structure
#		 minimum folding energy
# Return: MFE Index 4
# Reference:microPred: effective classification 
# of pre-miRNAs for human miRNA gene predictio
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub mfe_index_4 {
	
	my $struct = shift or die;
	my $mfe = shift or die;
	
	my $tot_bases = 0;
	while( $struct =~ /\)/g ){
		$tot_bases++;
	}
	

	my $dG = &normalized_mfe($struct,$mfe);
	
	if( $tot_bases ){
		return $dG / $tot_bases;
	} else {
		return 0;
	}
	
}






# =============================
# MFE Index 5 = dG / n_bulges
# where n_bulges is the number of bulge in the secondary structure. A stem 
# bulge is a structural motif containing more than three contiguous unpaired bases. 
# Function: calculate MFE Index 5.
# Input: structure
#		 minimum folding energy
# Return: MFE Index 5
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub mfe_index_5 {
	
	my $struct = shift or die;
	my $mfe = shift or die;
	
	my $dG = &normalized_mfe($struct,$mfe);
	
	my $n_bulges = &count_n_bulges($struct);
	
	if( $n_bulges ){
		return $dG / $n_bulges;
	} else {
		return 0;
	}
	
}





# +++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++ #




# =============================
# Input: infile for positive fasta
#		 ratio of negative sample to positive sample
#		 outfile for train fasta
#		 outifle for train struct
#		 [ CDS fasta ]
#		 [ CDS structure file ]
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub get_train_data {
	
	my $ratio_negative_to_positive = shift or die;
	my $infile_positive_struct = shift or die;
	my $outfile_negative_fasta = shift or die;
	my $dir = shift or die;


	my ($cds_fasta,$cds_struct);
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/cds_fasta/i) {$cds_fasta=shift @_}
		if ($argument=~/cds_struct/i) {$cds_struct=shift @_}
	}
	

	# ************************************************* #
	
	
	my %hash_positive_struct = ();
	Feature::parse_struct($infile_positive_struct,\%hash_positive_struct);
	
	my $bifurcation_positive = 0;
	my $no_bifurcation_positive = 0;
	my $max_mfe_positive = -10000000;
	my $min_paired_base_positive = 1000000;
	my %hash_positive_length = ();
	
	
	for my $id ( keys %hash_positive_struct ){
		
		my $seq = $hash_positive_struct{$id}{seq};
		my $struct = $hash_positive_struct{$id}{struct};
	
		$seq = uc($seq);
		$seq =~ s/^\s*//g;
		$seq =~ s/\s*$//g; 
		
		if( not ($seq =~/^([A|C|G|U]+)$/) ){
			print STDERR "Error:$id\n";
			die;
		}
	
	
		my $n_bp = 0;
		while( $struct =~ /\)/g ){
			$n_bp++;
		}
		
		if( $n_bp < $min_paired_base_positive ){
			$min_paired_base_positive = $n_bp;
		}
		
		if( $hash_positive_struct{$id}{mfe} > $max_mfe_positive ){
			$max_mfe_positive = $hash_positive_struct{$id}{mfe};
		}
		
		my $lng = length($struct);
		if( Feature::is_bifurcation($struct) ){
			$bifurcation_positive++;
			unless(exists $hash_positive_length{$lng}{1}){			
				$hash_positive_length{$lng}{1} = 1;
			} else {
				$hash_positive_length{$lng}{1}++;
			}
		} else {
			$no_bifurcation_positive++;
			unless(exists $hash_positive_length{$lng}{0}){			
				$hash_positive_length{$lng}{0} = 1;
			} else {
				$hash_positive_length{$lng}{0}++;
			}
		}
		
	}
		
	
	# ************************************************* #
	
	
	my %hash_segment_struct = ();
	if ( defined $cds_fasta ){
		
		unless(-e $cds_fasta){
			print STDERR "Error:$cds_fasta not exists\n";
			die;
		}
		
		my @suffix = qw(.fa .txt);
		my $basename = basename($cds_fasta,@suffix);	
		
		
		my %hash_seq = ();
		MyFasta::readfasta($cds_fasta,\%hash_seq);
		
		
		my @shuffleIDs = keys %hash_seq;	
		MyMath::shuffle(\@shuffleIDs);
		
		
		my $long_negative_seq = "";
		for my $id ( @shuffleIDs ){
			my $t = $hash_seq{$id};
			$t =~s/\s//g;
			$long_negative_seq .= $t;
		}
		

		my $i = 0;
		my $exit_loop = 0;
		my %hash_segment_seq = ();
		
		while ( $i < length($long_negative_seq) ){
			for my $j ( sort {$a<=>$b} keys %hash_positive_length ){	
				if( ($i + $j) >= length($long_negative_seq) - 1 ){
					$exit_loop = 1;
					last;
				}	
				my $subseq = substr($long_negative_seq, $i, $j);	
						
				$subseq = uc( $subseq );
				$subseq =~ tr/T/U/;
				$subseq =~ s/^\s*//g;
				$subseq =~ s/\s*$//g; 
				unless( not ($subseq =~/^([A|C|G|U]+)$/) ){
					unless(exists $hash_segment_seq{$subseq} ){
						$hash_segment_seq{$subseq} = $i+1;
					}
				}
				
				$i += $j;
							
			}
			if( $exit_loop ){
				last;
			}
		}
		
		my $segment_fasta = "$dir/$basename.segment-seq.fa";
		open OUT,">$segment_fasta" or die;	
		for my $seq ( keys %hash_segment_seq ){
			print OUT ">$hash_segment_seq{$seq}\n$seq\n";
		}	
		close OUT;
			
		my $segment_struct_file = "$dir/$basename.segment-struct.txt";
		die if system("perl fold_linux.pl $segment_fasta $segment_struct_file RNA");
		
		
		Feature::parse_struct($segment_struct_file,\%hash_segment_struct);
	
	} elsif(defined $cds_struct) {
		
		unless(-e $cds_struct){
			print STDERR "Error:$cds_struct not exists\n";
			die;
		}
		
		Feature::parse_struct($cds_struct,\%hash_segment_struct);
		
	} 
		
	
	# ************************************************* #
	
	
	my $bifurcation_negative = 0;
	my $no_bifurcation_negative = 0;
	my %hash_valided_segment = ();
	my %hash_valided_segment_length = ();
	my %hash_screen_segment = ();
	
	for my $id ( keys %hash_segment_struct ){
		
		my $mfe = $hash_segment_struct{$id}{mfe};
		my $seq = $hash_segment_struct{$id}{seq};
		my $struct = $hash_segment_struct{$id}{struct};
		
		$seq = uc($seq);
		$seq =~ s/^\s*//g;
		$seq =~ s/\s*$//g; 
		
		if( not ($seq =~/^([A|C|G|U]+)$/) ){
			next;
		}

		my $n_bp = 0;
		while( $struct =~ /\)/g ){
			$n_bp++;
		}
		
		if(($n_bp >= $min_paired_base_positive) && ($mfe <= $max_mfe_positive)){
			my $lng = length( $hash_segment_struct{$id}{seq} );
			if( Feature::is_bifurcation($struct) ){
				$bifurcation_negative++;
				$hash_valided_segment{$id} = 1;
				unless(exists $hash_valided_segment_length{$lng}{1}){
					$hash_valided_segment_length{$lng}{1} = 1;
					$hash_screen_segment{$lng}{1} = 0;
				} else {
					$hash_valided_segment_length{$lng}{1}++;
				}			
			} else {
				$no_bifurcation_negative++;
				$hash_valided_segment{$id} = 0;
				unless(exists $hash_valided_segment_length{$lng}{0}){
					$hash_valided_segment_length{$lng}{0} = 1;
					$hash_screen_segment{$lng}{0} = 0;
				} else {
					$hash_valided_segment_length{$lng}{0}++;
				}
			}
		}	
	}

	print "bifurcation candidate:$bifurcation_negative\n";
	print "no bifurcation candidate:$no_bifurcation_negative\n";

	my $min_r_c = scalar keys %hash_segment_struct;
	for my $lng ( keys %hash_valided_segment_length ){
		if(exists $hash_positive_length{$lng}{1} ){
			my $r = $hash_valided_segment_length{$lng}{1} / $hash_positive_length{$lng}{1};
			if( $r < $min_r_c ){
				$min_r_c = $r;
			}
		}
		if(exists $hash_positive_length{$lng}{0} ){
			my $r = $hash_valided_segment_length{$lng}{0} / $hash_positive_length{$lng}{0};
			if( $r < $min_r_c ){
				$min_r_c = $r;
			}
		}
	}
	
	my $min_r = $min_r_c < $ratio_negative_to_positive ? $min_r_c : $ratio_negative_to_positive;	
	$min_r = int( $min_r );	
	$min_r_c = int( $min_r_c );
	print "The real and candidate ratio of negative to positive:$min_r,$min_r_c\n";
	
	# ************************************ #
	
	
	open NEGATIVEFASTA,">$outfile_negative_fasta" or die;
	
	for my $r ( 0 ..1 ){	
		
		my @ID_array = ();
		for my $id ( keys %hash_valided_segment ){
			if( $hash_valided_segment{$id} == $r ){
				push(@ID_array,$id);
			}
		}
	
		
		MyMath::shuffle(\@ID_array);
			
	
		for my $id ( @ID_array ){	
			
			my $mfe = $hash_segment_struct{$id}{mfe};
			my $seq = $hash_segment_struct{$id}{seq};
			my $struct = $hash_segment_struct{$id}{struct};
			
			my $n_bp = 0;
			while( $struct =~ /\)/g ){
				$n_bp++;
			}
			
			if( $mfe > $max_mfe_positive || $n_bp < $min_paired_base_positive ){
				print STDERR "Error:$id\n";
				die;
			}
			
			my $lng = length($seq);	
			if(exists $hash_positive_length{$lng}{$r}){
				if( $hash_screen_segment{$lng}{$r} < $min_r * $hash_positive_length{$lng}{$r} ){
					$hash_screen_segment{$lng}{$r}++;
					$seq = uc( $seq );
					$seq =~ tr/T/U/;
					$seq =~ s/^\s*//g;
					$seq =~ s/\s*$//g; 
					unless( not ($seq =~/^([A|C|G|U]+)$/) ){
						print NEGATIVEFASTA ">$id\n$seq\n";
					} else {
						print STDERR "Error:$id\n";
						die;
					}
				}
			}
		}
	}
	close NEGATIVEFASTA;
	
}		





# +++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++ #





# =============================
# Function: calculate F-score 
# Reference: Combining SVMs with various feature selection strategies
# Author: Yuangen Yao
# Date: 2013-3-31
# =============================	

sub F_score {
	
	my $infile_feature = shift or die;
	my $hash_fscore = shift or die;
	

	unless(-e $infile_feature){
		print STDERR "Error:$infile_feature not exists\n";
		die;
	}
	
	
	my %hash_row = ();
	my @cols = MyMath::toMatrix($infile_feature,\%hash_row);
	
	
	my @rows = ();
	for my $row ( sort {$a cmp $b} keys %hash_row ){
		if ( $row !~/^\bname\b/i ){
			push(@rows,$row);
		}
	}
	
	
	my $feature_num = 0;
	for my $col ( 2 .. $#cols ){
	
		my $whole_sum 	 = 0;
		my $positive_sum = 0;
		my $negative_sum = 0;
		my $positive_num = 0;
		my $negative_num = 0;
		$feature_num++;
		
		for my $row ( @rows ){
			
			my $v = $hash_row{$row}{ $cols[$col] };
			my $label = $hash_row{$row}{ $cols[1] };
			
			$whole_sum += $v;
			if ( $label == 1 ){		
				$positive_sum += $v;
				$positive_num++;		
			} elsif ( $label == -1 ){		
				$negative_sum += $v;
				$negative_num++;	
			} else {
				print STDERR "Error:($row,$cols[$col],$v)\n";
				die;
			}
		}
		
		my $whole_av 	= $whole_sum / ($positive_num + $negative_num);
		my $positive_av = $positive_sum / $positive_num;
		my $negative_av = $negative_sum / $negative_num;
		
		my $numerator = ( $positive_av - $whole_av ) * ( $positive_av - $whole_av );
		$numerator   += ( $negative_av - $whole_av ) * ( $negative_av - $whole_av );
		
		my $positive_variance_sum = 0;
		my $negative_variance_sum = 0;

		for my $row ( @rows ){
			
			my $v = $hash_row{$row}{ $cols[$col] };
			my $label = $hash_row{$row}{ $cols[1] };
			
			if ( $label == 1 ){			
				my $t = ( $v - $positive_av ) * ( $v - $positive_av );
				$positive_variance_sum += $t;	
			} elsif ( $label == -1 ){		
				my $t = ( $v - $negative_av ) * ( $v - $negative_av );
				$negative_variance_sum += $t;	
			} else {
				print STDERR "Error:($row,$cols[$col],$v)\n";
				die;
			}
		}
		
		my $f_score = 0;	
		if ( $positive_num > 1 && $negative_num > 1 ){
			my $denominator = $positive_variance_sum / ( $positive_num - 1 );
			$denominator   += $negative_variance_sum / ( $negative_num - 1 );
			if ( $denominator ){
				$f_score = $numerator / $denominator;
			}
		}
		
		$hash_fscore->{$cols[$col]} = $f_score;
		
	}
	
	return $feature_num;
	
}




# +++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++ #


###########################################################
1;






	