package MyFasta;
#!/usr/bin/perl
use strict;
use warnings;
use File::Basename;








# =============================
# Function: read one or more fasta sequencs into hash table.
# Input: a fasta file and reference of hash table.
# Return: the number of fasta sequence.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub readfasta {
	
	my $usage = "< fasta file > < hash reference >";
	my $infile = shift or die $usage;
	my $hash_seq = shift or die $usage;
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists";
		die;
	}
	
	open IN, $infile || die;
	
	my $c=0;
	my $seqId;
	while (defined (my $line=<IN>)) {
		chomp($line);
		next if $line=~/^\s*$/;
		if ($line =~/^>/) {
			$line =~s/\s*$//g;	
			$line =~s/^\s*//g;	
			$seqId = substr($line,1);
			$c++;
		} else {
			$line =~s/\s*//g;
			$hash_seq->{$seqId}.=$line;
		}
	}
	close IN;
	
	return $c;
	
}




# =============================
# Function: read one or more fasta sequencs into hash table.
# Input: a fasta file and reference of hash table.
# Return: the number of fasta sequence.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub readfastaId {
	
	my $usage = "< fasta file > < hash reference >";
	my $infile = shift or die $usage;
	my $hash_seq = shift or die $usage;
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists";
		die;
	}
	
	open IN, $infile || die;
	
	my $c=0;
	my $seqId;
	while (defined (my $line=<IN>)) {
		chomp($line);
		next if $line=~/^\s*$/;
		if ($line =~/^>/) {
			$line =~s/\s*$//g;	
			$line =~s/^\s*//g;	
			$seqId = substr($line,1);
			$hash_seq->{$seqId} = 1;
			$c++;
		}
	}
	close IN;
	
	return $c;
	
}





# =============================
# Function: Collapses reads in the fasta file to make each sequence entry unique.
# Input: a fasta file and reference of hash table.
# Return: the number of fasta sequence.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub seqkey {
	
	my $infile 	 = shift or die;
	my $hash_seq = shift or die;
	  
	unless(-e $infile){
		print STDERR "Error:$infile not exists";
		die;
	}
	  
    my $c = 0;
    my($seqId, $seq) = ();
    
    open FASTA,"<$infile" or die;
    while (<FASTA>) {
        chomp;
        if (/^>(\S+)/){
		    $seq = "";
		    $seqId = $1;
		    while (<FASTA>){
				chomp;
				if (/^>(\S+)/){						
					my $freq = &getTagFreq($seqId);					
					$seq =~ tr/[acgtuUn\.]/[ACGTTTNN]/;			
					$hash_seq->{$seq} += $freq;										
					$seqId = $1;
					
					$seq  = "";
					$c++;
					next;
				}
				s/\s*//g;
				$seq .= $_;
		    }
		}
    }
    
    close FASTA;
    
	my $freq = &getTagFreq($seqId);		
	$seq =~ tr/[acgtuUn\.]/[ACGTTTNN]/;
	$hash_seq->{$seq} += $freq;													
	$c++;
	
    return $c;
    
}







# =============================
# Function: Collapses reads in the fasta file to make each sequence entry unique.
# Input: a fasta file and reference of hash table.
# Return: the number of fasta sequence.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub seqkeyIDvalue {
	
	my $infile 	 = shift or die;
	my $hash_seq = shift or die;
	  
	unless(-e $infile){
		print STDERR "Error:$infile not exists";
		die;
	}
	  
    my $c = 0;
    my($seqId, $seq) = ();
    
    open FASTA,"<$infile" or die;
    while (<FASTA>) {
        chomp;
        if (/^>(\S+)/){
		    $seq = "";
		    $seqId = $1;
		    while (<FASTA>){
				chomp;
				if (/^>(\S+)/){									
					$seq =~ tr/[acgtuUn\.]/[ACGTTTNN]/;	
					unless(exists $hash_seq->{$seq}){		
						$hash_seq->{$seq} = $seqId;	
					} else {
						$hash_seq->{$seq}.= "\t$seqId";
					}									
					$seqId = $1;
					
					$seq  = "";
					$c++;
					next;
				}
				s/\s*//g;
				$seq .=  $_;
		    }
		}
    }
    
    close FASTA;
    
	$seq =~ tr/[acgtuUn\.]/[ACGTTTNN]/;
	unless(exists $hash_seq->{$seq}){		
		$hash_seq->{$seq} = $seqId;	
	} else {
		$hash_seq->{$seq}.= "\t$seqId";
	}											
	$c++;
	
    return $c;
    
}




# =============================
# Function: change one or more fasta sequencs into a sequence of characters.
# Input: a fasta file.
# Return: a string of sequence.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub fasta2string {
	
	my $usage = "< fasta file >";
	my $infile = shift or die $usage;
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists";
		die;
	}
	
	open IN, $infile || die;
	
	my $seq = "";
	while (<IN>) {
		chomp;
		unless (/^>/) {
			$_=~s/\s//g;
			$seq .= $_;
		}
	}
	close IN;
	
	$seq =~ s/\s//g;
	
	return $seq;
	
}


# =============================
# Function: change Fastq into Fasta.
# Input: a fastq file And a filename for output fasta file.
# Return: the number of processed fastq.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub fastq2fasta {
	
	my $usage = "< fastq file > < filename for output fasta file >";
	my $infile = shift or die $usage;
	my $outfile = shift or die $usage;
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists";
		die;
	}

	open IN, "<$infile" or die;
	open OUT,">$outfile" or die;
	
	my $c = 0;
	my $processed = 0;
	my $seq = "";

	while( <IN> ) {
	    chomp;
	    next if /^\s*$/;
	    $c++;
	    if($c == 1){
	        $processed++;
	    }elsif($c == 2){
	        $seq = $_;
	    }elsif($c == 4){
	        $c = 0;
	        print OUT ">seq_$processed\n$seq\n";
	        $seq = "";
	    }else{}
	}
	close IN;
	close OUT;
	
	print STDERR "$processed reads processed\n\n";
	
	return $processed;
	
}






# =============================
# Function: trim a string of character
# Input: a string of character 
# Return: a string of trimd character
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub trim {
	my $str = shift or die;
	$str =~ s/^\s*//g;
	$str =~ s/\s*$//g;
	return $str;
}




# =============================
# Function: reverse complement of RNA or DNA sequence.
# Input: a DNA or RNA sequence.
# Return: reverse complement of input sequence.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub revcom{  
	
    my($sequence)=@_;
    
    my $revcom = reverse(comp($sequence));
    
    return $revcom;
    
}




# =============================
# Function: complement of RNA or DNA sequence.
# Input: a DNA or RNA sequence.
# Return: uppercase complement of input sequence.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub comp{
	
    my($sequence)=@_;
    
    $sequence=~tr/acgtuACGTU/TGCAATGCAA/;   
    
    return $sequence;
    
}





# =============================
# Function: count the fasta.
# Input: a fasta file.
# Return: the number of fasta sequence.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub countfasta {
	
	my $usage = "< fasta file >";
	my $infile = shift or die $usage;
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists";
		die;
	}
	
	open IN, $infile || die;
	
	my $c = 0;
	while (<IN>) {
		if (/^>/) {
			$c++;
		} 
	}
	close IN;
	
	return $c;
	
}




# =============================
# Function: count the fastq.
# Input: a fastq file.
# Return: the number of fastq.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub countfastq {
	
	my $usage = "< fasta file >";
	my $infile = shift or die $usage;
		
	unless(-e $infile){
		print STDERR "Error:$infile not exists";
		die;
	}
	
	open IN, "<$infile" or die;
	
	my $c = 0;
	my $processed = 0;
	
	while( <IN> ) {
	    chomp;
	    $c++;
	    if($c == 1){
	        $processed++;
	    }elsif($c == 4){
	        $c = 0;
	    }
	}
	close IN;

	return $processed;
	
}



# =============================
# Function: get frequency of a tag.
# Input: a tag file.
# Return: the number of total read and unique read.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub getTagFreq {
	
	my $usage = "< tag >";
	my $tag = shift or die $usage;
	
	my $count = 1;
	if($tag =~ /_x(\S+)/){
		$count = $1;
	} elsif ($tag =~ /\S+\s+(\S+)/){
		$count = $1;
	} 
	
	unless(&is_numeric($count)){
		print STDERR "Error:$count not a numeric\n";
		print STDERR "Error:format wrong";
		die;
	}
	
	return $count;
	
}
	
	


# =============================
# Function: get name of a tag.
# Input: a tag file.
# Return: the number of total read and unique read.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub getTagName {
	
	my $usage = "< tag >";
	my $tag = shift or die $usage;
	
	my $count = 0;
	my $name = "";
	
	if($tag =~ /(\S+)_x(\S+)/){
		$name = $1;
		$count = $2;
	} elsif ($tag =~ /(\S+)\s+(\S+)/){
		$name = $1;
		$count = $2;
	} else {
		print STDERR "Error:format wrong";
		die;
	}
	
	unless(&is_numeric($count)){
		print STDERR "Error:$count not a numeric\n";
		print STDERR "Error:format wrong";
		die;
	}
	
	return $name;
	
}


# =============================
# Function: count the Tag.
# Input: a tag file.
# Return: the number of total read and unique read.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub countTag {

	my $usage = "< tag file >";
	my $infile = shift or die $usage;
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists";
		die;
	}
	
	open IN, $infile || die;
	
	my $total_count  = 0;
	my $unique_count = 0;
	
	while (<IN>) {
		chomp;
		if (/^>/) {
			my $count = &getTagFreq($_);
			$total_count += $count;
			$unique_count++;
		} 
	}
	close IN;
	
	return ($total_count,$unique_count);
	
}







# =============================
# Function: get the length of fasta.
# Input: a fasta file and reference of hash.
# Return: the number of fasta.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub getFastaLength {

	my $usage = "<fasta file > < reference of hash >";
	my $infile = shift or die $usage;
	my $hash_lng = shift or die $usage;
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}
	
	open IN, $infile || die;
	
	my $c=0;
	my $seq = "";
	
	while (<IN>) {
		chomp;
		if (/^>/) {
			$seq =~ s/\s//g;
			my $lng = length( $seq );
			if ( $lng > 0 ){
				unless( exists $hash_lng->{$lng} ){
					$hash_lng->{$lng} = 1;
				} else {
					$hash_lng->{$lng}++;
				}
			}
			$seq = "";
			$c++;
		} else {
			$_=~s/\s//g;
			$seq .= $_;
		}
	}
	close IN;	
	
	$seq =~ s/\s//g;
	my $lng = length( $seq );
	if ( $lng > 0 ){
		unless( exists $hash_lng->{$lng} ){
			$hash_lng->{$lng} = 1;
		} else {
			$hash_lng->{$lng}++;
		}
	}
	
	return $c;
	
}





# =============================
# Function: get the length of tag.
# Input: a tag file and reference of hash.
# Return: the number of fasta.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub getTagLength {
	
	my $usage = "<fasta file > < reference of hash >";
	my $infile = shift or die $usage;
	my $hash_lng = shift or die $usage;

	my $c=0;
	my $count;
	my $seq = "";
	
	open IN, $infile || die;
	while (<IN>) {
		chomp;
		if (/^>/) {
			$seq =~ s/\s//g;
			my $lng = length( $seq );
			if ( $lng > 0 ){
				unless( exists $hash_lng->{$lng} ){
					$hash_lng->{$lng} = $count;
				} else {
					$hash_lng->{$lng}+= $count;
				}
			}
			$seq = "";			
			$count = &getTagFreq($_);
			$c++;
		} else {
			$_=~s/\s//g;
			$seq .= $_;
		}
	}
	close IN;	
	
	$seq =~ s/\s//g;
	my $lng = length( $seq );
	if ( $lng > 0 ){
		unless( exists $hash_lng->{$lng} ){
			$hash_lng->{$lng} = $count;
		} else {
			$hash_lng->{$lng}+= $count;
		}
	}
	
	return $c;
	
}



# =============================
# Function: get the length distribution of tag or fasta.
# Input: reference of array of fasta filenames.
#		 filename for output length distribution of tag.
#		 filename for output length distribution (%) of tag.
#		 tag or fasta: 1 (for tag) or 0 (for fasta).
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub lengthDistribution {
	
	my $usage = "
	< reference of array of fasta filenames > 
	< output filename for length distribution >
	< output filename for the percent of length distribution>
	[ tag flag] 
	";
	
	my $fasta_arrays = shift or die $usage;
	my $outfile_length_distribution = shift or die $usage;
	my $outfile_percent_length_distribution = shift or die $usage;
	my $tag = shift;
	
	my @samples = ();
	my %hash_lng = ();
	my %hash_total = ();
	my @suffix = qw( .fa .txt );
	
	foreach my $fastaFile ( @$fasta_arrays ){
		
		unless(-e $fastaFile ){
			print STDERR "Error:$fastaFile not exists\n";
			die;
		}
		
		my $sample = basename($fastaFile,@suffix);
		push(@samples,$sample);
		
		my %hash = ();	
		if( defined $tag ){
			&getTagLength($fastaFile,\%hash);
		} else {
			&getFastaLength($fastaFile,\%hash);
		}

		for my $lng ( sort {$a<=>$b} keys %hash ){
			$hash_lng{$lng}{$sample} = $hash{$lng};
			unless(exists $hash_total{$sample}){
				$hash_total{$sample} = $hash{$lng};
			} else {
				$hash_total{$sample}+= $hash{$lng};
			}
		}
		
	}

	# ************************************************ #

	my $header = "Length";
	for my $sample ( @samples ){
		$header .= "\t$sample";
	}

	# ************************************************ #

	open  OUT,">$outfile_length_distribution" or die;
	print OUT "$header\n";
	
	open  PER,">$outfile_percent_length_distribution" or die;
	print PER "$header\n";
	
	foreach my $lng (sort { $a<=>$b } keys %hash_lng){
		my $str = $lng;
		my $str_per = $lng;
		for my $sample ( @samples ){
			if ( exists $hash_lng{$lng}{$sample} ){
				$str .= "\t$hash_lng{$lng}{$sample}";
				my $per  = &round($hash_lng{$lng}{$sample}/$hash_total{$sample},6);
				$str_per.= "\t$per";			
			} else {
				$str .= "\t0";
				$str_per .= "\t0";
			}
		}
		print OUT "$str\n";
		print PER "$str_per\n";
	}
	close OUT;
	close PER;
	
}		






# =============================
# Function: extract the miRNA and pre-miRNA from miRBase database
# Input: reference species
#		 known mature miRNA from miRBase
#		 known precursor from miRBase
#		 outfile for extracted mature 
#        outfile for extracted precursor
#        uncheck (1): uncheck all known miRNAs and pre-miRNA for reference species
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub extractSeqInMiRBase {
	
	my $species 				= shift or die;
	my $mirbase_mature 			= shift or die;
	my $mirbase_precursor 		= shift or die;
	my $this_species_mature 	= shift or die;
	my $this_species_precursor 	= shift or die;
	my $uncheck					= shift;
	
	unless(defined $uncheck){
		$uncheck = 0;
	}
	
	unless(-e $mirbase_mature){
		print STDERR "Error:$mirbase_mature not exists\n";
		die;
	}
	unless(-e $mirbase_precursor){
		print STDERR "Error:$mirbase_precursor not exists\n";
		die;
	}
	
	my %hash_mature_seq = ();
	&readfasta($mirbase_mature,\%hash_mature_seq );

	my %hash_precursor_seq = ();
	&readfasta($mirbase_precursor,\%hash_precursor_seq );

	open MIRBASE,">$this_species_mature" or die;	
	for my $id ( sort keys %hash_mature_seq ){
		my $seq = $hash_mature_seq{$id};
		$seq = uc( $seq );
		$seq =~ s/U/T/ig;
		unless( not ($seq =~/^([A|C|G|T|U|a|c|g|t|u]+)$/) ){
			unless($uncheck){
				if ( $id =~ /^$species/i and $id =~/(\S+)/ ){		
					print MIRBASE ">$1\n$seq\n";
				}
			} else {
				if( $id !~ /^$species/i and $id =~/(\S+)/ ){	
					print MIRBASE ">$1\n$seq\n";
				}
			}	
		}	
	}
	close MIRBASE;

	open MIRBASE,">$this_species_precursor" or die;
	for my $id ( sort keys %hash_precursor_seq ){
		my $seq = $hash_precursor_seq{$id};
		$seq = uc( $seq );
		$seq =~ s/u/T/ig;
		unless( not ($seq =~/^([A|C|G|T|U|a|c|g|t|u]+)$/) ){
			unless($uncheck){
				if ( $id =~ /^$species/i and $id =~/(\S+)/ ){		
					print MIRBASE ">$1\n$seq\n";
				}
			} else {
				if( $id !~ /^$species/i and $id =~/(\S+)/ ){	
					print MIRBASE ">$1\n$seq\n";
				}
			}	
		}	
	}
	close MIRBASE;
	
}




# =============================
# Function: extract the miRNA and pre-miRNA from miRBase database
# Input: reference species
#		 known mature miRNA from miRBase
#		 known precursor from miRBase
#		 outfile for extracted mature 
#        outfile for extracted precursor
#        uncheck (1): uncheck all known miRNAs and pre-miRNA for reference species
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub extractSeqInMiRBase1 {
	
	my $ref_species		  = shift or die;
	my $mirbase_mature 	  = shift or die;
	my $mirbase_precursor = shift or die;

	my $uncheck = 0;	
	my $outfile_mature;
	my $outfile_precursor;
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/mature/i) {$outfile_mature=shift @_}
		if ($argument=~/precursor/i) {$outfile_precursor=shift @_}
		if ($argument=~/uncheck/i) {$uncheck=shift @_}
	}
	
	
	
	unless(-e $mirbase_mature){
		print STDERR "Error:$mirbase_mature not exists\n";
		die;
	}
	unless(-e $mirbase_precursor){
		print STDERR "Error:$mirbase_precursor not exists\n";
		die;
	}
	
	my %hash_mature_seq = ();
	&readfasta($mirbase_mature,\%hash_mature_seq );

	my %hash_precursor_seq = ();
	&readfasta($mirbase_precursor,\%hash_precursor_seq );


	

	if( defined $outfile_mature ){
		
		open MIRBASE,">$outfile_mature" or die;	
		for my $id ( sort keys %hash_mature_seq ){
			my $seq = $hash_mature_seq{$id};
			$seq = uc( $seq );
			$seq =~ s/u/T/ig;
			unless( not ($seq =~/^([A|C|G|T|U|a|c|g|t|u]+)$/) ){
				unless($uncheck){
					if ( &is_array_ref( $ref_species ) ){
						for my $species ( @$ref_species	){
							if ( $id =~ /^$species/i and $id =~/(\S+)/ ){		
								print MIRBASE ">$1\n$seq\n";
								last;
							}
						}
					} else {
						if ( $id =~ /^$ref_species/i and $id =~/(\S+)/ ){		
							print MIRBASE ">$1\n$seq\n";							
						}
					}
				} else {
					if ( &is_array_ref( $ref_species ) ){
						for my $species ( @$ref_species	){
							if( $id !~ /^$species/i and $id =~/(\S+)/ ){	
								print MIRBASE ">$1\n$seq\n";
								last;
							}
						}
					} else {
						if( $id !~ /^$ref_species/i and $id =~/(\S+)/ ){	
							print MIRBASE ">$1\n$seq\n";
						}
					}
				}	
			}	
		}
		close MIRBASE;
		
	}


	if( defined $outfile_precursor ){

		open MIRBASE,">$outfile_precursor" or die;
		for my $id ( sort keys %hash_precursor_seq ){
			my $seq = $hash_precursor_seq{$id};
			$seq = uc( $seq );
			$seq =~ s/u/T/ig;
			unless( not ($seq =~/^([A|C|G|T|U|a|c|g|t|u]+)$/) ){
				unless($uncheck){
					if ( &is_array_ref( $ref_species ) ){
						for my $species ( @$ref_species	){
							if ( $id =~ /^$species/i and $id =~/(\S+)/ ){		
								print MIRBASE ">$1\n$seq\n";
								last;
							}
						}
					} else {
						if ( $id =~ /^$ref_species/i and $id =~/(\S+)/ ){		
							print MIRBASE ">$1\n$seq\n";
						}
					}
				} else {
					if ( &is_array_ref( $ref_species ) ){
						for my $species ( @$ref_species	){
							if( $id !~ /^$species/i and $id =~/(\S+)/ ){	
								print MIRBASE ">$1\n$seq\n";
								last;
							}
						}
					} else {
						if( $id !~ /^$ref_species/i and $id =~/(\S+)/ ){	
							print MIRBASE ">$1\n$seq\n";
						}
					}
				}	
			}	
		}
		close MIRBASE;
		
	}
	
}




# =============================
# Function: extract required sequence from Rfam database
# Input: reference species
#		 Rfam fasta
#		 outfile for extracted sequence
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub extractSeqInRfam {
	
	my $hash_accession 	= shift or die;
	my $species 		= shift or die;
	my $infile_Rfam 	= shift or die;
	my $outfile_fasta 	= shift or die;
	
	unless(-e $infile_Rfam){
		print STDERR "Error:$infile_Rfam not exists\n";
		die;
	}
	
	my %hash_seq = ();
	&readfasta($infile_Rfam,\%hash_seq);
	
	open RFAM,">$outfile_fasta" or die;	
	for my $id ( keys %hash_seq ){
		my $seq =$hash_seq{$id};
		$seq =~ tr/[acgtuUn\.]/[ACGTTTNN]/;
		for my $accession ( keys %{ $hash_accession } ){
			if($id =~ /$accession/ && $id =~ /$species/){
				print RFAM ">$id\n$seq\n";
			}
		}
	}
	close RFAM;
	
}
	
	
	
	
# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================	
	
sub is_array_ref {
    if($_[0] and Scalar::Util::reftype($_[0])
             and Scalar::Util::reftype($_[0]) eq "ARRAY") {
        return 1;
    }
    else {
        return 0;
    }
}




# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub is_hash_ref {
    if($_[0] and Scalar::Util::reftype($_[0])
             and Scalar::Util::reftype($_[0]) eq "HASH") {
        return 1;
    }
    else {
        return 0;
    }
}



# =============================
# Function: preprocess the repeat and mRNA sequence from UCSC Table Browser
# Input: a fasta file 
#		 outfile for preprocessed fasta sequence
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub preproSeqInUCSC {
	
	my $infile	= shift or die;
	my $outfile = shift or die;

	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}

	open IN, "<$infile" or die;
	open UCSC,">$outfile" or die;
	
	my $c = 0;
	my ($seqId,$seq);
	while (defined (my $line=<IN>)) {
		chomp($line);
		next if $line=~/^\s*$/;
		if ($line =~/^>/) {		
			$c++;
			if (defined $seq and length($seq) > 0 ){
				my $new_id = $seqId;			
				if( $seqId =~ /(\S+)/ ){			
					my $name = $1;
					if($name=~/rmsk_(\S+)/i){
						$name = $1;
					} elsif($name=~/all_mrna_(\S+)/i){
						$name = $1;
					} elsif ($name=~/refGene_(\S+)/i){
						$name = $1;
					}
					my ($coor,$strand);
					if($seqId=~/\brange=(\S+)/){
						$coor = $1;
					} 
					if($seqId=~/\bstrand=(\S+)/){
						$strand = $1;
					} 
					if(defined $coor and defined $strand){
						$new_id = $name.":".$coor.":".$strand;
					} else {
						$new_id = $name;
					}
				}
				$seq =~ tr/[acgtuUn\.]/[ACGTTTNN]/;			
				my $i = 0;
				my $str = "";
				while($i < length($seq) ){		
					my $subseq = substr($seq,$i,50);
					$str .= "$subseq\n";
					$i += 50;
				}
				print UCSC ">$new_id\n$str";
				
			}
			
			$seqId = substr($line,1);
			$seq = "";
			
		} else {
			$line =~s/\s*//g;
			$seq .=$line;
		}
	}
	close IN;	
	
	if ( defined $seq and length($seq) > 0 ){
		$c++;
		my $new_id = $seqId;
		if( $seqId =~ /(\S+)/ ){		
			my $name = $1;
			if($name=~/rmsk_(\S+)/i){
				$name = $1;
			} elsif($name=~/all_mrna_(\S+)/i){
				$name = $1;
			} elsif ($name=~/refGene_(\S+)/i){
				$name = $1;
			}
			my ($coor,$strand);
			if($seqId=~/\brange=(\S+)/){
				$coor = $1;
			} 
			if($seqId=~/\bstrand=(\S+)/){
				$strand = $1;
			} 
			if(defined $coor and defined $strand){
				$new_id = $name.":".$coor.":".$strand;
			} else {
				$new_id = $name;
			}
		}
		$seq =~ tr/[acgtuUn\.]/[ACGTTTNN]/;			
		my $i = 0;
		my $str = "";
		while($i < length($seq) ){		
			my $subseq = substr($seq,$i,50);
			$str .= "$subseq\n";
			$i += 50;
		}
		print UCSC ">$new_id\n$str";
		
	}	
	close UCSC;
	
	return $c;
}
	
	
	


# =============================
# Function: count the position of occurrences of pattern 
#           And the number of occurrences of pattern
# Input: tolerant mismatch
#		 pattern
#        subject sequence
#        reference of hash for saving the position of occurrences of pattern
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub matchPattern {
	
	my $nmismatch 	= shift;
	my $pattern 	= shift or die;
	my $subjectSeq 	= shift or die;
	my $hash_pos 	= shift or die;
	
	unless( defined $nmismatch ){
		print STDERR "Error:number of mismatch not defined\n";
		die;
	}
	
	my @pattern 	= split '', $pattern;
	my @subjectSeq 	= split '', $subjectSeq;

	my $lng = length( $pattern );
	my %err_count;
	for my $i ( 0 .. @subjectSeq - $lng ) {
		my $n_err = 0 ;
		for my $j ( 0 .. @pattern - 1 ) {
			$n_err++ if( $pattern[$j] ne $subjectSeq[$i+$j] );
		}
		next if ( $n_err > $nmismatch );
		$err_count{$i} = $n_err;
	}

	foreach( keys %err_count ) {
		if( $err_count{$_} == $nmismatch ){
			my $value = substr( $subjectSeq, $_, $lng );		
			$hash_pos->{ $_ } = $value;
		}
	}
}




# =============================
# Function: calculate the position preference of base.
# Input: a fasta file 
#		 outfile for base position bias.
#        outifle for percent of base position bias.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub basePositionBias {
	
	my $fastaFile 	= shift or die;
	my $outfile_bias = shift or die;
	my $outfile_percent_bias = shift or die;
	
	unless(-e $fastaFile){
		print STDERR "Error:$fastaFile not exists\n";
		die;
	}

	my %hash_seq = ();
	&readfasta($fastaFile,\%hash_seq);
	
	
	my %hash_base = ();
	my @bases = qw( A C G U N );
	
	for my $id ( sort keys %hash_seq ){
		
		my $seq = $hash_seq{$id};
		$seq =~ tr/[acgtuTn\.]/[ACGUUUNN]/;		
		
		for my $base ( @bases ){
			
			my %hash_pos = ();
			&matchPattern(0,$base,$seq,\%hash_pos);
			
			for my $p ( sort {$a<=>$b} keys %hash_pos ){
				$p++;
				unless(exists $hash_base{$p}{$base}){
					$hash_base{$p}{$base} = 1;
				} else {
					$hash_base{$p}{$base}++;
				}
			}
		}
	}
	
	
	open  OUT,">$outfile_bias" or die;
	print OUT "Position\tA\tC\tG\tU\tN\n";
	
	my %hash_tot = ();
	for my $p ( sort {$a<=>$b} keys %hash_base ){
		my $line = $p;
		my $sum = 0;
		for my $base ( @bases ){
			if (exists $hash_base{$p}{$base}){
				$line .= "\t$hash_base{$p}{$base}";
				$sum += $hash_base{$p}{$base};
			} else {
				$line .= "\t0";
			}
		}
		print OUT "$line\n";
		$hash_tot{$p} = $sum;
	}
	close OUT;		
	
	
	open  OUT,">$outfile_percent_bias" or die;
	print OUT "Position\tA\tC\tG\tU\tN\n";
	
	for my $p ( sort {$a<=>$b} keys %hash_base ){
		my $line = $p;
		for my $base ( @bases ){
			if (exists $hash_base{$p}{$base}){				
				my $per = &round($hash_base{$p}{$base}/$hash_tot{$p},6);
				$line .= "\t$per";
			} else {
				$line .= "\t0";
			}
		}
		print OUT "$line\n";
	}
	close OUT;	
	
}

	


# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub get_lower_case {
	
	my $infile = shift or die;

	my $outfile;
	my $hash_input;
	my $min_length = 1;
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/file/i) {$outfile=shift @_}
		if ($argument=~/hash/i) {$hash_input=shift @_}
		if ($argument=~/min_length/i) {$min_length=shift @_}
	}
	
	
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}

	
    my $c = 0;
    my %hash_seq = ();
    my($seqId,$seq,$beg,$id,$str,@ids) = ();
    
    open FASTA,"<$infile" or die;
    
    while (<FASTA>) {
        chomp;
        if (/^>(\S+)/){
		    $seq = "";
		    $seqId = $1;
		    while (<FASTA>){
				chomp;
				if (/^>(\S+)/){	
					$c++;
					my $i = 0;
					while ( $i < length($seq) ){
						$beg = $i + 1;
						$str = "";					
						while ( $i < length($seq) ){
							my $ch = substr($seq,$i,1);
							if ( $ch =~ /[a-z]/ ){
								$str .= $ch;
								$i++;
							} else {
								if( ($i - $beg) >= ($min_length - 1) ){								
									$id = "$seqId:$beg:$i";
									push(@ids,$id);
									$str =~ tr/[acgtuUn\.]/[ACGTTTNN]/;								
									$hash_seq{$id} = $str;
									$str = "";
								}
								last;
							}
						}	
						$i++;									
					}	
					
					if ( $str ne "" ){
						$id = "$seqId:$beg:$i";
						push(@ids,$id);
						$str =~ tr/[acgtuUn\.]/[ACGTTTNN]/;								
						$hash_seq{$id} = $str;
						$str = "";
					}
		
					$seqId = $1;	
					$seq  = "";
					$c++;
					next;	
										
				}
				
				s/\s*//g;
				$seq .=  $_;
				
		    }
		}
    }
    
    close FASTA;
    
	my $i = 0;
	while ( $i < length($seq) ){
		$beg = $i + 1;
		$str = "";					
		while ( $i < length($seq) ){
			my $ch = substr($seq,$i,1);
			if ( $ch =~ /[a-z]/ ){
				$str .= $ch;
				$i++;
			} else {
				if( ($i - $beg) >= ($min_length - 1) ){									
					$id = "$seqId:$beg:$i";
					push(@ids,$id);
					$str =~ tr/[acgtuUn\.]/[ACGTTTNN]/;								
					$hash_seq{$id} = $str;		
					$str = "";
				}
				last;
			}
		}	
		$i++;									
	}			
	
	if ( $str ne "" ){
		$id = "$seqId:$beg:$i";
		push(@ids,$id);
		$str =~ tr/[acgtuUn\.]/[ACGTTTNN]/;								
		$hash_seq{$id} = $str;	
		$str = "";
	}
	
	if( defined $outfile ){
		open UCSC,">$outfile" or die;
		for my $id ( @ids ){
			my $j = 0;
			my $str_out = "";
			while($j < length($hash_seq{$id}) ){		
				my $subseq = substr($hash_seq{$id},$j,50);
				$str_out .= "$subseq\n";
				$j += 50;
			}
			print UCSC ">$id\n$str_out";
		}	
		close UCSC;
	}
	
	if ( defined $hash_input and &is_hash_ref($hash_input) ){
		%$hash_input = %hash_seq;
	}
	
	$c++;
	
    return $c;
    
}	



	
	
# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub getSeqByCoordinate {
	
	my $seq 	= shift or die;
	my $beg 	= shift;
	my $end 	= shift or die;
	my $strand  = shift or die;
	
	unless( defined $beg ){
		print STDERR "Error:beg not defined\n";
		die;
	}
	
	my $zero_one_base = 1;
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/zero_one_base/i) {$zero_one_base=shift @_}
	}
	
	
	my $subseq;
	my $lng = $end - $beg + 1;
	if ( $zero_one_base ){	
		$subseq = substr($seq,$beg-1,$lng);
	} else {
		$subseq = substr($seq,$beg,$lng);
	}


	if ( $strand eq "-" ){
		$subseq = &comp($subseq);		
		$subseq = reverse $subseq;
	} 
	
	$subseq =~ tr/[acgtuUn\.]/[ACGTTTNN]/;		
	
	return $subseq;
	
}						
			
			
			
			
# =============================
# Input: fasta  and adapter sequence
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub clip_adapter {
	
	my $infile = shift or die;
	my $adapter = shift or die;
	
	unless( -e $infile ){
		print STDERR "Error:$infile not exists\n";
		die;
	}
	
	my $outfile;
	my $hash_input;
	my $collapse;
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/file/i) {$outfile=shift @_}
		if ($argument=~/hash/i) {$hash_input=shift @_}
		if ($argument=~/collapse/i) {$collapse=shift @_}
	}
	
	my %hash_seq = ();
	&readfasta($infile,\%hash_seq);
	
	my $prefix = substr($adapter,0,6);
	$prefix =~ tr/[acgtuUn\.]/[ACGTTTNN]/;
	
	
	my %hash_clip = ();	
	for my $id ( keys %hash_seq ){
		
		my $seq = $hash_seq{$id};
		$seq =~ tr/[acgtuUn\.]/[ACGTTTNN]/;
		
		my $flag = 1;	
		if( $seq =~ /(\w{18,})$prefix/ ){
			$hash_clip{$id} = $1;
			$flag = 0;
		} else {			
			while( $flag && ( length( $prefix ) > 0 ) ){ 
			    chop $prefix;	    
			    if( $seq =~ /(\w+)$prefix$/ ){
					$hash_clip{$id} = $1;
					$flag = 0;
			    }
			}
		}
		
		if( $flag ){
			$hash_clip{$id} = $seq;
		}
	}
	
	
	# collapse sequences
	if ( defined $collapse ) {			
		my $unique_seq_sum = 0;
		my %hash_unique_seq = ();
		for my $id ( keys %hash_clip ){		
			my $seq = $hash_clip{$id};
			$seq =~ tr/[acgtuUn\.]/[ACGTTTNN]/;		
			unless(exists $hash_unique_seq{$seq}){				
				$hash_unique_seq{$seq} = 1;
				$unique_seq_sum++;
			} else {
				$hash_unique_seq{$seq}++;
			}
		}
		
		my $sn = 1;
		%hash_clip = ();
		my $n = &order_of_magnitude($unique_seq_sum);
		for my $seq ( sort {$hash_unique_seq{$b} <=> $hash_unique_seq{$a}} 
			keys %hash_unique_seq ){
			my $freq = $hash_unique_seq{$seq};
			my $header = sprintf("%0$n"."d",$sn);
			$header = "t$header";
			my $id = "$header $freq";
			
			unless(exists $hash_clip{$id}){
				$hash_clip{$id} = $seq;
			} else {
				print STDERR "Error:$id repeat\n";
				die;
			}
			$sn++;
		}
	}
		
		
	if( defined $outfile ){
		open CLIP,">$outfile" or die;
		for my $id ( keys %hash_clip ){
			print CLIP ">$id\n$hash_clip{$id}\n";
		}	
		close CLIP;
	}
	
	
	if ( defined $hash_input and &is_hash_ref($hash_input) ){
		%$hash_input = %hash_clip;
	}
		
}






# =============================
# Input: fasta  and adapter sequence
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================
					
sub is_fastq_fasta {
	
	my $infile = shift or die;
	
	unless( -e $infile ){
		print STDERR "Error:$infile not exists\n";
		die;
	}
		
	my $str;
	open IN,"<$infile" or die;
	while( <IN> ){
		chomp;
		next if /^\s*$/;
		if( /^>/ ){
			$str = "fasta";
			last;
		} elsif ( /^\@/ ) {
			$str = "fastq";
			last;
		} else {
			print STDERR "Error:$infile format error\n";
			print STDERR "Only fastq and fasta format are valid\n"; 
			die;
		}
	}
	close IN;
	
	return $str;
	
}





# =============================
# Input: fasta  and adapter sequence
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub getSeqByID {
	
	my $infile 	= shift or die;
	my $inSeqId	= shift or die;
	  
	my $outfile;
	my $result_seq;
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/file/i) {$outfile=shift @_}
		if ($argument=~/seq/i) {$result_seq=shift @_}
	}
	  
	  
	unless(-e $infile){
		print STDERR "Error:$infile not exists";
		die;
	}
	  
    my $c = 0;
    my ($outSeq,$seq);
   
    open FASTA,"<$infile" or die;
    while (<FASTA>) {
        chomp;
        if (/^\s*>\s*($inSeqId)\s*$/i){
		    $seq = "";
		    while (<FASTA>){
				chomp;
				if (/^>(\S+)/){									
					$outSeq	= $seq;													
					$seq = "";
					$c++;
					last;
				}
				s/\s*//g;
				$seq .=  $_;
		    }
		    $c++;
		}
		last if $c;
    }
    
    close FASTA;
    
	if( defined $seq && length( $seq ) > 0 ){		
		$outSeq	= $seq;	
	}
	
    if( defined $outfile && defined $outSeq ){	
		my $j = 0;
		my $str_out = "";
		open SEQ,">$outfile" or die;
		while($j < length($outSeq) ){		
			my $subseq = substr($outSeq,$j,50);
			$str_out .= "$subseq\n";
			$j += 50;
		}
		print SEQ ">$inSeqId\n$str_out";			
		close SEQ;
	}
	
	if ( defined $result_seq && defined $outSeq ){
		$result_seq = $outSeq;
	}
	
}



# =============================
# Input:  a numerical value.
# Return: order of magnitude of this numerical value.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub order_of_magnitude {
	
	my $usage = "< numerical value >";
	my $value = shift;
	
	die $usage unless defined $value;
	
	unless ( &is_numeric($value) ){
		print STDERR "Error:$value not a numeric\n";
		die;
	}
	
	if ( $value < 1 ){
		return 0;
	} else {
		my $n = 0;
		while( $value >= 1 ){
			$value = int( $value / 10 );
			$n++;
		}
		return $n;
	}
}
		
		



# =============================
# Input: a numerical value and the numbers after the decimal point.
# Return: rounded value of this numerical value.
# Date: 2013-2-20
# =============================

sub round {
	
	my $usage = "<  numerical value > < the numbers after the decimal point >";
    my $val = shift;
    my $col = shift;
    
    unless(defined $val){
    	die $usage;
    }
    
    unless(defined $col){
    	die $usage;
    }
    
    unless( &is_numeric($val) ){
    	print STDERR "Error:$val not a numeric";
		die;
	}
	
	unless( &is_numeric($col) ){
    	print STDERR "Error:$col not a numeric";
		die;
	}
    
    my $r = 10 ** $col;
    my $a = ($val > 0) ? 0.5 : -0.5;
    
    return int($val * $r + $a) / $r;
    
}





# =============================
# Input:  a parameter of any type.
# Return: 1 (numeric) or 0 (non-numeric). 
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub is_numeric {
	
	use Scalar::Util qw(looks_like_number);
	
    my $v = shift;

    if( looks_like_number( $v ) ){  	
        return 1;
    } else {
        return 0;
    }
    
}	



# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub getCoordinateRelativeLocation{
	
	my $strand 	= shift;
	my $beg 	= shift;
	my $end 	= shift;
	my $st 		= shift;
	my $ed 		= shift;
	
	my $t = $ed - $st + 1;
	my $lng = $end - $beg + 1;
	
	my $left_offset  = $st;
	my $right_offset = $lng - $left_offset - $t;
	
	my ($mb,$me);
	if ( $strand eq "-" ){
		($left_offset,$right_offset) = ($right_offset,$left_offset);
		$mb = $beg + $left_offset + 1;
		$me = $end - $right_offset + 1;
	} else {
		$mb = $beg + $left_offset - 1;
		$me = $end - $right_offset - 1;
	}
	 
	
	return ($mb,$me);
	
}
	



# =============================
# Function: find conserved miRNAs
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub family {
	
	my $seed_start 	 = shift or die;
	my $seed_end   	 = shift or die;
	my $hash_mature  = shift or die;
	my $hash_family  = shift or die;
	my $outfile      = shift;
	
	$seed_start--;
	my $n_base = $seed_end - $seed_start;
	
	for my $id ( sort keys %$hash_mature ){
		my $seq = uc( $hash_mature->{$id} );
		$seq =~ tr/T/U/;
		$seq =~ s/^\s*//g;
		$seq =~ s/\s*$//g; 		
		unless( not ($seq =~/^([A|C|G|U]+)$/) ){
			my $seed = substr($seq,$seed_start,$n_base);
			unless(exists $hash_family->{$seed}){
				$hash_family->{$seed} = $id;
			} else {
				$hash_family->{$seed}.= "\t$id";
			}
		}		
	}
	
	if( defined $outfile ){
		open OUT,">$outfile" or die;
		for my $id ( sort keys %$hash_family ){	
			print OUT "$id\t$hash_family->{$id}\n";
		}
		close OUT;
	}
	
}
	
	
	
	
# =============================
# Function: length distribution
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================
	
sub lng_dis {
	
	my $infile 	 = shift or die;
	my $hash_lng = shift or die;
	  
	unless(-e $infile){
		print STDERR "Error:$infile not exists";
		die;
	}
	  
    my $c = 0;
    my($seqId, $seq) = ();
    
    open FASTA,"<$infile" or die;
    while (<FASTA>) {
        chomp;
        if (/^>(\S+)/){
		    $seq = "";
		    $seqId = $1;
		    while (<FASTA>){
				chomp;
				if (/^>(\S+)/){					
				
					unless(exists $hash_lng->{$seqId}){		
						$hash_lng->{$seqId} = length($seq);	
					} else {
						print STDERR "Error:$seqId repeat\n";
						die;
					}	
													
					$seqId = $1;
					
					$seq  = "";
					$c++;
					next;
				}
				s/\s*//g;
				$seq .=  $_;
		    }
		}
    }
    
    close FASTA;
    
    
	unless(exists $hash_lng->{$seqId}){		
		$hash_lng->{$seqId} = length($seq);	
	} else {
		print STDERR "Error:$seqId repeat\n";
		die;
	}
														
	$c++;
	
    return $c;
    
}	
	
#################################################################
1;