#!/usr/bin/perl
use strict;
use warnings;
use Cwd;
use File::Copy;
use File::Basename;
use Getopt::Long  qw(:config bundling);

# My Library
use Bowtie;
use miRNA;
use MyFile;
use miRDeep2;
use Mireap;
use Feature;
use Redundancy;
use Repeat;
use Category;
use piRNA;

# ============== Default parameters ================ #
my $program				= $0;
my $species				= "dre";
my $mode 				= "union";
my $prepro				= 1;
my $novel				= 1;
my $known				= 0;
my $minLen				= 18;
my $maxLen				= 35;
my $minFrq				= 3;
my $probT				= 0.5;
my $threads				= 1;
my $mismatches			= 1;
my $Mireap 				= 1;
my $miRDeep2 			= 0;
my $BEG_DIF  			= -2;
my $END_DIF  			= 5;
my $seed_start			= 2;
my $seed_end			= 8;
my $no_scoring			= 0;
my $no_map_Rfam			= 0;
my $no_map_repeat		= 0;
my $no_map_mRNA			= 0;
my $no_map_piRNA		= 0;
my $mismatches_seed		= 0;
my $mapping_loc			= 5;
my $help 				= 0;
my $piRNA_cluster_dist  = 1000;
my $piRNA_cluster_height= 10;
my $is_piRNA_cluster	= 1;
my $is_category			= 1;
my $is_recall			= 1;
my $is_rescue			= 1;
my $is_clean_tmp_file	= 0;
# ================= Required Path/Files ================ #
my $dir;
my $data_path;
my $miRDeep2_path;
my $Mireap_path;
my $Bowtie_path;
my $LIBSVM_path;
my $genome_fasta;
my $hairpin_fasta;
my $mature_fasta;
my $Rfam_fasta;
my $repeat_fasta;
my $mRNA_fasta;
my $piRNA_fasta;
my $gff3_file;
# ============================================== #
my $header = "";
my $species_Latin;
my $mir_same_file;
my $mir_most_file;
my $mir_total_file;
my $mir_same_norm_file;
my $mir_most_norm_file;
my $mir_total_norm_file;
my $this_species_mature;
my $this_species_precursor;
my $known_precursor_struct;
my $this_species_gff3;
my $repeat_dir;
my $piRNA_dir;
# ================= Hash/Array ================ #
my @tools;
my @samples;
my @ncRNAIdFile;
my @SVMmodel;
my @scaleModel;
my @reads = ();
my @sampleNames = ();
my @mapped_reads = ();
my @unmapped_reads = ();
my @unmapped_reads_tmp = ();
my @relatedSpecies;
my @suffix = qw(.fa .txt);
my %hash_species_Latin;
my %hash_species_miRBase;
my %hash_predict = ();
my %hash_known_mature = ();
my %hash_genome_mapped = ();
my %hash_rRNA_accession = ('RF00001'=>1,
'RF00002'=>1,'RF01118'=>1,'RF01960'=>1);
my %hash_tRNA_accession = ('RF01852'=>1,'RF00005'=>1);
# ================= Bowtie CMD ================ #
my $genome_cmd;
my $miRBase_cmd;
my $Rfam_cmd;
my $repeat_cmd;
my $mRNA_cmd;
my $piRNA_cmd;
my $Mireap_cmd;
my $miRDeep2_cmd;
my $novel_miR_cmd;
# ============================================== #

GetOptions( 

	"help|h!"			=> \$help,
	"tools|T=s"			=> \@tools,
	"samples|F=s" 		=> \@samples,
	"relatedSpecies|R=s"=> \@relatedSpecies,
	"species|S=s" 		=> \$species, 
	"mode|M=s"			=> \$mode,
	"prepro|r=i"		=> \$prepro,
	"known|K=i"			=> \$known,
	
	"minLen|l=i"		=> \$minLen,
	"maxLen|L=i"		=> \$maxLen,
	"minFrq|q=i"		=> \$minFrq,
	
	"dir|d=s"			=> \$dir,
	"data_path|I=s"		=> \$data_path,
	"Bowtie_path|B=s"	=> \$Bowtie_path,
	"Mireap_path|E=s"	=> \$Mireap_path,
	"miRDeep2_path|D=s"	=> \$miRDeep2_path,
	"LIBSVM_path|C=s"	=> \$LIBSVM_path,
	
	"threads|p=i"		=> \$threads,
	"mismatches|v=i"	=> \$mismatches,
	"genome_cmd=s"		=> \$genome_cmd,
	"miRBase_cmd=s"		=> \$miRBase_cmd,
	"Rfam_cmd=s"		=> \$Rfam_cmd,
	"repeat_cmd=s"		=> \$repeat_cmd,
	"mRNA_cmd=s"		=> \$mRNA_cmd,
	"piRNA_cmd=s"		=> \$piRNA_cmd,
	"Mireap_cmd=s"		=> \$Mireap_cmd,
	"miRDeep2_cmd=s"	=> \$miRDeep2_cmd,
	"novel_map_cmd=s"	=> \$novel_miR_cmd,
	
	"genome_fasta=s"	=> \$genome_fasta,
	"mature_fasta=s"	=> \$mature_fasta,
	"hairpin_fasta=s"	=> \$hairpin_fasta,
	"Rfam_fasta=s"		=> \$Rfam_fasta,
	"repeat_fasta=s"	=> \$repeat_fasta,
	"mRNA_fasta=s"		=> \$mRNA_fasta,
	"piRNA_fasta=s"		=> \$piRNA_fasta,
	"gff3=s"			=> \$gff3_file,
	
	"SVMmodel|V=s"		=> \@SVMmodel,
	"scaleModel|H=s"	=> \@scaleModel,
	"probT|P=f"			=> \$probT,
	"no_scoring!"		=> \$no_scoring,
	
	"no_map_Rfam!"		=> \$no_map_Rfam,
	"no_map_repeat!"	=> \$no_map_repeat,
	"no_map_mRNA!"		=> \$no_map_mRNA,
	"no_map_piRNA!"		=> \$no_map_piRNA,

	"del_tmp|w=i"		=> \$is_clean_tmp_file,
);



# ********** Required paramters *********** #

if ( $help ) {
	&usage(); exit;
}

if( $known ){
	$novel = 0;
}	

unless(scalar @samples){
	print STDERR "Error:read fasta not defined\n";
	&usage(); exit;	
}

unless( @tools ){
	if ( $novel ){
		print STDERR "\n\n\nPlease select tools for microRNA prediction\n";
		print STDERR "You can specify it by typing in command:\n";
		print STDERR "-T Mireap or -T miRDeep2 or -T Mireap -T miRDeep2\n\n\n";
		&usage(); exit;
	}
} else {
	for my $tool ( @tools ){
		if($tool =~/Mireap/i){
			$Mireap = 1;
		}elsif($tool =~/miRDeep2/i){
			$miRDeep2 = 1;
		}
	}
	if( $Mireap ==0 || $miRDeep2 == 0 ){
		$mode = "union";
	}
}

unless ( $no_scoring ) {
	if ( @SVMmodel ) {
		my @SVMmodel_tmp = ();
		for my $model ( @SVMmodel ) {
			if ( -e $model ){
				$model = MyFile::getAbsPath($model);
				push(@SVMmodel_tmp,$model);
			} else {		
				print STDERR "Error:$model not exists\n";
				print STDERR "You can specify it by -V option\n";
				print STDERR "Note that the full pathname of given file is required\n";
				print STDERR "If it was not in the current directory\n\n\n";
				&usage(); exit;		
			}
		}
		@SVMmodel = ();
		@SVMmodel = @SVMmodel_tmp;
	} else {
		print STDERR "\n\nYou are required to specify one or more SVM modles\n";
		print STDERR "You can specify it by -V option\n";
		print STDERR "Note that the full pathname of given file is required\n";
		print STDERR "If it was not in the current directory\n\n\n";
		&usage(); exit;
	}

	if ( @scaleModel ) {
		my @scaleModel_tmp = ();
		for my $model ( @scaleModel ) {
			if ( -e $model ){
				$model = MyFile::getAbsPath($model);
				push(@scaleModel_tmp,$model);
			} else {		
				print STDERR "Error:$model not exists\n";
				print STDERR "You can specify it by -H option\n";
				print STDERR "Note that the full pathname of given file is required\n";
				print STDERR "If it was not in the current directory\n\n\n";
				&usage(); exit;		
			}
		}
		@scaleModel = ();
		@scaleModel = @scaleModel_tmp;
	} else {
		print STDERR "\n\nYou are required to specify one or more scaled modles\n";
		print STDERR "You can specify it by -H option\n";
		print STDERR "Note that the full pathname of given file is required\n";
		print STDERR "If it was not in the current directory\n\n\n";
		&usage(); exit;
	}
}



# ******** species name transform ******** #

$species_Latin = &get_species($species,
\%hash_species_Latin,\%hash_species_miRBase);

for my $other_species ( @relatedSpecies ){
	unless( exists $hash_species_miRBase{$other_species} ){
		print STDERR "\n\nError:$other_species is not a name of species in miRBase database\n";
		print STDERR "Please use name of species in miRBase\n";
		print STDERR "Please look at organisms in current directory to get it\n\n\n";
		&usage(); exit;
	}
}


# ********* Default Paths/Files ********** #

$program =~ s/\.pl$//i;
my $current_dir = getcwd;
unless( defined $dir ){
	$dir = "$current_dir/$program"."-".time;
} else {
	$dir = MyFile::getAbsPath($dir);
}

unless( defined $miRDeep2_path ){
	$miRDeep2_path = "$current_dir/Packages/mirdeep2";
} else {
	$miRDeep2_path = MyFile::getAbsPath($miRDeep2_path);
}

unless( defined $Mireap_path ){
	$Mireap_path = "$current_dir/Packages/mireap_0.2";
} else {
	$Mireap_path = MyFile::getAbsPath($Mireap_path);
}

unless( defined $Bowtie_path ){
	$Bowtie_path = "$current_dir/Packages/mirdeep2/essentials/bowtie-0.12.5";
} else {
	$Bowtie_path = MyFile::getAbsPath($Bowtie_path);
}

unless( defined $LIBSVM_path ){
	$LIBSVM_path = "$current_dir/Packages/libsvm-3.16";
} else {
	$LIBSVM_path = MyFile::getAbsPath($LIBSVM_path);
}

unless( defined $data_path ){
	$data_path = "$current_dir/Data";
} else {
	$data_path = MyFile::getAbsPath($data_path);
}



# ********** Check required files ********* #

unless(-x "$Bowtie_path/bowtie"){
	print STDERR "bowtie not exists in $Bowtie_path\n";
	print STDERR "You can specify it by typing in command:\n";
	print STDERR "-B Bowtie path\n";
	print STDERR "Note that the full pathname of given file is required\n";
	print STDERR "If it was not in the current directory\n\n\n";
	&usage(); exit;
}

unless(-e "$Mireap_path/bin/mireap.pl"){
	print STDERR "Error:mireap.pl not exists in $Mireap_path/bin\n";
	print STDERR "You can specify it by typing in command:\n";
	print STDERR "-E Mireap path\n";
	print STDERR "Note that the full pathname of given file is required\n";
	print STDERR "If it was not in the current directory\n\n\n";
	&usage(); exit;
}

unless(-x "$LIBSVM_path/svm-train"){
	print STDERR "Error:svm-train not exists in $LIBSVM_path\n";
	print STDERR "You can specify it by typing in command:\n";
	print STDERR "-C LIBSVM path\n";
	print STDERR "Note that the full pathname of given file is required\n";
	print STDERR "If it was not in the current directory\n\n\n";
	&usage(); exit;
}

unless(-e "$miRDeep2_path/miRDeep2.pl"){
	print STDERR "Error:miRDeep2.pl not exists in $miRDeep2_path\n";
	print STDERR "You can specify it by typing in command:\n";
	print STDERR "-D miRDeep2 path\n";
	print STDERR "Note that the full pathname of given file is required\n";
	print STDERR "If it was not in the current directory\n\n\n";
	&usage(); exit;
}


unless ( defined $genome_fasta ){
	$genome_fasta = "$data_path/$species.genome.fa";
	unless(-e $genome_fasta){
		print STDERR "Error:$genome_fasta not exists\n";
		die;
	}
} else {
	$genome_fasta = MyFile::getAbsPath($genome_fasta);
}

unless( defined $hairpin_fasta ){
	$hairpin_fasta = "$data_path/hairpin.fa";
	unless(-e $hairpin_fasta){
		print STDERR "Error:$hairpin_fasta not exists\n";
		die;
	}
} else {
	$hairpin_fasta = MyFile::getAbsPath($hairpin_fasta);
}

unless( defined $mature_fasta ){
	$mature_fasta = "$data_path/mature.fa";
	unless(-e $mature_fasta){
		print STDERR "Error:$mature_fasta not exists\n";
		die;
	}
} else {
	$mature_fasta = MyFile::getAbsPath($mature_fasta);
}
	
unless( defined $Rfam_fasta ){
	$Rfam_fasta = "$data_path/Rfam.fasta";
	unless(-e $Rfam_fasta){
		print STDERR "Error:$Rfam_fasta not exists\n";
		die;
	}
} else {
	$Rfam_fasta = MyFile::getAbsPath($Rfam_fasta);
}

unless( defined $repeat_fasta ){
	$repeat_fasta = "$data_path/$species.repeat.fa";
	unless(-e $repeat_fasta){
		print STDERR "Error:$repeat_fasta not exists\n";
		die;
	}
} else {
	$repeat_fasta = MyFile::getAbsPath($repeat_fasta);
}

unless( defined $mRNA_fasta ){
	$mRNA_fasta = "$data_path/$species.mRNA.fa";
	unless(-e $mRNA_fasta){
		print STDERR "Error:$mRNA_fasta not exists\n";
		die;
	}
} else {
	$mRNA_fasta = MyFile::getAbsPath($mRNA_fasta);
}

unless( defined $piRNA_fasta ){
	$piRNA_fasta = "$data_path/$species.piRNA.fa";
} else {
	$piRNA_fasta = MyFile::getAbsPath($piRNA_fasta);
}

unless( defined $this_species_gff3 ){
	$this_species_gff3 = "$data_path/$species.gff3";
} else {
	$this_species_gff3 = MyFile::getAbsPath($gff3_file);
}

my $snRNA_snoRNA_file = "snRNA_snoRNA";
unless(-e $snRNA_snoRNA_file){
	print STDERR "Error:$snRNA_snoRNA_file not exists\n";
	die;
} else {
	push(@ncRNAIdFile,$snRNA_snoRNA_file);
}



# ********** Default Bowtie cmd ********* #

unless ( defined $genome_cmd ){
	$genome_cmd = "bowtie -p $threads -t -f -v $mismatches";
}

unless ( defined $miRBase_cmd ){
	$miRBase_cmd = "bowtie -p $threads -t -f -v $mismatches -a --best --strata --norc";
}

unless ( defined $Rfam_cmd ){
	$Rfam_cmd = "bowtie -p $threads -t -f -v $mismatches -a --best --strata --norc";
}

unless ( defined $repeat_cmd ){
	$repeat_cmd = "bowtie -p $threads -t -f -v $mismatches";
}

unless ( defined $mRNA_cmd ){
	$mRNA_cmd = "bowtie -p $threads -t -f -v $mismatches -a --best --strata --norc";
}

unless ( defined $piRNA_cmd ){
	$piRNA_cmd = "bowtie -p $threads -t -f -v $mismatches -a --best --strata --norc";
}

unless ( defined $Mireap_cmd ){
	$Mireap_cmd = "bowtie -p $threads -t -f -v 0 -m 5 --best --strata";
}

unless ( defined $novel_miR_cmd ){
	$novel_miR_cmd = "bowtie -p $threads -t -f -v 0 -a --best --strata --norc";
}





# ************************************************************** #
# Start
# ************************************************************** #


MyFile::makeDir($dir);

my $record_parameter = "$dir/$program-log.txt";
open PARAM,">$record_parameter" or die;

my $start_time = MyFile::start(\*PARAM);

print PARAM "Species:$species ($species_Latin)\n";
print PARAM "Destination folder:$dir\n";
print PARAM "Data folder:$data_path\n";
print PARAM "mode:$mode\n";
print PARAM "prepro:$prepro\n";
print PARAM "minLen:$minLen\n";
print PARAM "maxLen:$maxLen\n";
print PARAM "minFrq:$minFrq\n";
print PARAM "probT:$probT\n";
print PARAM "threads:$threads\n";
print PARAM "mismatches:$mismatches\n";
print PARAM "\ngenome_cmd:$genome_cmd\n";
print PARAM "\nmiRBase_cmd:$miRBase_cmd\n";
print PARAM "\nRfam_cmd:$Rfam_cmd\n";
print PARAM "\nrepeat_cmd:$repeat_cmd\n";
print PARAM "\nmRNA_cmd:$mRNA_cmd\n";
print PARAM "\npiRNA_cmd:$piRNA_cmd\n";




# ************************************************************** #
# preprocessing
# ************************************************************** #


my $prepro_stat = "$dir/stat.txt";
open  PREPRO,">$prepro_stat" or die;
print PREPRO "read_file\ttotal_tag\ttotal_read\tremaining_tag\tremaining_read\n";
for my $sample ( @samples ){
	
	# ϢĴ
	$sample = MyFile::getAbsPath($sample);
	unless( -e $sample ){
		print STDERR "Error:$sample not exists\n";
		die;
	}
	my $basename = basename($sample,@suffix);	
	push(@sampleNames,$basename);
	$header .= "\t$basename";
	
	
	# reads
	my %hash_seq = ();
	MyFasta::readfasta($sample,\%hash_seq);
		
	
	# readsʽͳ
	my $n_tag = 0; my $n_read = 0;
	my $tot_tag = 0; my $tot_read = 0;
	
	my $read_file = "$dir/$basename.fa";
	open OUT,">$read_file" or die;
	
	for my $id ( keys %hash_seq ){
		# ͳreads
		my $tag_name = MyFasta::getTagName($id);
		my $tag_freq = MyFasta::getTagFreq($id);
		my $tag_id   = $tag_name."_x".$tag_freq;
		$tot_tag++;
		$tot_read += $tag_freq;

		# RNAתΪDNA
		my $seq = $hash_seq{$id};
		$seq =~ s/U/T/ig;
		
		# ˵ַreads
		unless( not ($hash_seq{$id} =~/^([A|C|G|T|N|U|a|c|g|t|n|u]+)$/) ){
			if( $prepro ){
				# ֻreads
				my $lng = length($hash_seq{$id});	
			 	if ( ($tag_freq >= $minFrq) && ($lng >= $minLen) && ($lng <= $maxLen) ){
					$n_tag++;
					$n_read += $tag_freq;
					print OUT ">$tag_id\n$seq\n";
				}
			} else {
				$n_tag++;
				$n_read += $tag_freq;
				print OUT ">$tag_id\n$seq\n";
			}
		}
	}
	close OUT;
	
	# ¼Ԥreadsļ
	push(@reads,$read_file);
	
	# ͳƽ
	print PREPRO "$basename\t$tot_tag\t$tot_read\t$n_tag\t$n_read\n";
}
close PREPRO;

		
			
# ************************************************************** #
# genome mapping
# ************************************************************** #




print "\nGenome mapping ...\n";
print PARAM "\nGenome mapping ...\n";

my $genome_dir = "$dir/genome";
MyFile::makeDir($genome_dir,1);


my $genome_ref_basename = basename($genome_fasta,@suffix);
my $genome_stat = "$genome_dir/$genome_ref_basename.stat.txt";


# readsȶԵο
Bowtie::mapper($genome_fasta,\@reads,$genome_cmd,$Bowtie_path,
'dest_dir'=>$genome_dir,'genome_stat'=>$genome_stat);


for my $read_file ( @reads ){
	my $ref_basename  = basename($genome_fasta,@suffix);
	my $read_basename = basename($read_file,@suffix);
	my $mapped_file   = "$genome_dir/$ref_basename.$read_basename.mapped.fa";
	my $unmapped_file = "$genome_dir/$ref_basename.$read_basename.unmapped.fa";
	my $mapping_file  = "$genome_dir/$ref_basename.$read_basename.mapping.txt";
	my $stat_file 	  = "$genome_dir/$ref_basename.stat.txt";
	
	# ¼mapped readRPM׼
	my %hash_row = ();
	MyMath::toMatrix($stat_file,\%hash_row);
	for my $row ( sort keys %hash_row ){
		$hash_genome_mapped{$read_basename} = $hash_row{mapped_read}{$read_basename};
	}
	
	
	# mapped readsһmapping
	unlink($read_file);
	copy($mapped_file,$read_file);
	push(@mapped_reads,$read_file);
	
	if ( $is_clean_tmp_file ){
		unlink($mapped_file);
		unlink($unmapped_file);
		unlink($mapping_file);
	}
	
}


	
	
# ************************************************************** #
# length distribution 
# ************************************************************** #


# readĳȷֲ
my $lng_dis_dir = $genome_dir;
my $tag_length_dist = "$lng_dis_dir/tag_length_distribution.txt";
my $tag_length_dist_percent = "$lng_dis_dir/tag_length_distribution_percent.txt";
MyFasta::lengthDistribution(\@mapped_reads,$tag_length_dist,$tag_length_dist_percent);


# tagĳȷֲ
my $read_length_dist = "$lng_dis_dir/read_length_distribution.txt";
my $read_length_dist_percent = "$lng_dis_dir/read_length_distribution_percent.txt";
MyFasta::lengthDistribution(\@mapped_reads,$read_length_dist,$read_length_dist_percent,1);





# ************************************************************** #
# miRBase mapping
# ************************************************************** #


print "\nIdentify known miRNAs ...\n";
print PARAM "\nIdentify known miRNAs ...\n";

my $known_miRNA_dir = "$dir/known";
MyFile::makeDir($known_miRNA_dir,1);


# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
my %hash_known_precursor = ();
my %hash_known_expression = ();
my %hash_known_mature_sequenced = ();
my %hash_known_expression_sort = ();
my %hash_family_sequenced = ();
my %hash_known_family = ();
my %hash_known_family_express = ();
my $known_miRNA_family = "$known_miRNA_dir/known_family.txt";
$this_species_mature = "$known_miRNA_dir/$species.mature.fa";
$this_species_precursor = "$known_miRNA_dir/$species.hairpin.fa";
$mir_same_file  = "$known_miRNA_dir/miRNA.miRBase.txt";
$mir_most_file  = "$known_miRNA_dir/miRNA.most-abundant.txt";
$mir_total_file = "$known_miRNA_dir/miRNA.total.txt";
$mir_same_norm_file  = "$known_miRNA_dir/miRNA.miRBase.RPM.txt";
$mir_most_norm_file  = "$known_miRNA_dir/miRNA.most-abundant.RPM.txt";
$mir_total_norm_file = "$known_miRNA_dir/miRNA.total.RPM.txt";
$known_precursor_struct = "$known_miRNA_dir/$species.struct.txt";
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #



# =========================================================
# =========== ֪miRNAʶͱ׵Ľ ================
# =========================================================


# miRBaseصmaturehairpinļȡֵָmiRNApre-miRNA
MyFasta::extractSeqInMiRBase($species,$mature_fasta,
$hairpin_fasta,$this_species_mature,$this_species_precursor,0);
MyFasta::readfasta($this_species_mature,\%hash_known_mature);
MyFasta::readfasta($this_species_precursor,\%hash_known_precursor);


# RNAfoldԿ֪pre-miRNAнṹԤ
die if system("perl fold_linux.pl $this_species_precursor $known_precursor_struct RNA");


# Ѳreads ȶԵ֪pre-miRNA
Bowtie::mapper($this_species_precursor,\@mapped_reads,$miRBase_cmd,
$Bowtie_path,'dest_dir'=>$known_miRNA_dir,'del_index'=>1,
'genome_stat'=>$genome_stat);


# miRNAı
for my $read_file ( @mapped_reads ){
	
	# mappingļ
	my $ref_basename  = basename($this_species_precursor,@suffix);
	my $read_basename = basename($read_file,@suffix);
	my $sample = $read_basename;
	my $mapping_file  = "$known_miRNA_dir/$ref_basename.$read_basename.mapping.txt";
	my $unmapped_file = "$known_miRNA_dir/$ref_basename.$read_basename.unmapped.fa";	
	my $mapped_file	  = "$known_miRNA_dir/$ref_basename.$read_basename.mapped.fa";
	
	# miRNAȶԵǰpre-miRNA棬miRNAıˮƽ
	my %hash_parse = (); my %hash_mature_tag = ();
	my $outfile_profile_file = "$known_miRNA_dir/$read_basename.aln.txt";
	if(-e $this_species_gff3){
		miRNA::parser($Bowtie_path,\%hash_parse,\%hash_mature_tag,$this_species_mature,
		$this_species_precursor,$mapping_file,$known_precursor_struct,$outfile_profile_file,
		'gff3'=>$this_species_gff3,'dest_dir'=>$known_miRNA_dir,'BEG_DIF'=>$BEG_DIF,'END_DIF'=>$END_DIF);
	} else {
		miRNA::parser($Bowtie_path,\%hash_parse,\%hash_mature_tag,$this_species_mature,
		$this_species_precursor,$mapping_file,$known_precursor_struct,$outfile_profile_file,
		'dest_dir'=>$known_miRNA_dir,'BEG_DIF'=>$BEG_DIF,'END_DIF'=>$END_DIF);
	}
	
	
	# ÿmiRNAı
	my $title = "miRNA\ttotal count\tmost count\tmiRBase count\t";
	$title .= "ratio of most to total\tratio of miRBase to total\t";
	$title .= "same/differenct\tmiRBase sequence\tmost abundant sequence";
	my $most_seq_file = "$known_miRNA_dir/$read_basename.most-abundant-sequence.txt";
	open  OUT,">$most_seq_file" or die;
	print OUT "$title\n";
	
	for my $mir ( sort keys %hash_parse ){
		
		if(exists $hash_parse{$mir}{same}){
			$hash_known_expression{$mir}{$sample}{same} = $hash_parse{$mir}{same};
		} else {
			$hash_known_expression{$mir}{$sample}{same} = 0;
		}		
		$hash_known_expression{$mir}{$sample}{most} = $hash_parse{$mir}{most_count};
		$hash_known_expression{$mir}{$sample}{total} = $hash_parse{$mir}{total};
	
		# miRNAıˮƽڿ̽ıˮƽԤ(minFrq)miRNA
		if ( $hash_known_expression{$mir}{$sample}{most} < $minFrq ){
			next;
		}
		
		my $most_r = MyMath::round($hash_parse{$mir}{most_count}/$hash_parse{$mir}{total},6);
		my $same_r = MyMath::round($hash_known_expression{$mir}{$sample}{same}/$hash_parse{$mir}{total},6);
		
		my $mirBase_seq = $hash_known_mature{$mir};
		my $most_seq = $hash_parse{$mir}{most_seq};
		$mirBase_seq =~ s/T/U/ig;
		$most_seq =~ s/T/U/ig;
		
		my $str = "$mir\t$hash_parse{$mir}{total}\t$hash_parse{$mir}{most_count}\t";
		$str .= "$hash_known_expression{$mir}{$sample}{same}\t$most_r\t$same_r\t";
		if( $mirBase_seq eq $most_seq){
			$str .= "S\t$mirBase_seq\t$most_seq";
		} else {
			$str .= "D\t$mirBase_seq\t$most_seq";
		}
		print OUT "$str\n";
		
	}
	close OUT;
	
	# unmapped readsһmapping
	unlink($read_file);
	copy($unmapped_file,$read_file);
	push(@unmapped_reads,$read_file);
	
	
	if ( $is_clean_tmp_file ){
		unlink($mapped_file);
		unlink($unmapped_file);
		unlink($mapping_file);
	}
	
}



# =========================================================
# ================ ˵¹۲ֵmiRNA =================
# =========================================================



# miRNAıˮƽڿ̽ıˮƽԤ(minFrq)miRNA
for my $mir ( sort keys %hash_known_expression ){
	my $exp_flag = 0;
	for my $sample ( @sampleNames ){
		my $t = 0;
		if ( exists $hash_known_expression{$mir}{$sample} ){
			$t = $hash_known_expression{$mir}{$sample}{most};
		}
		if ( $t >= $minFrq ){
			$exp_flag++;
		}
	}	
	if ( $exp_flag ){
		$hash_known_mature_sequenced{$mir} = $hash_known_mature{$mir};
	}
}



# =========================================================
# =================== ֪miRNAĽ =================
# =========================================================


# еһԣԲпɹ۲쵽֪miRNAּ
MyFasta::family($seed_start,
$seed_end,\%hash_known_mature_sequenced,
\%hash_known_family,$known_miRNA_family);


# ĸ˳ԼеһԱ
for my $seed ( sort keys %hash_known_family ){		
	my @members = split /\t/,$hash_known_family{$seed};
	@members = sort {$a cmp $b} @members;	
	my $n = 0;
	for my $mir ( @members ){
		$hash_family_sequenced{$seed}[$n] = $mir;
		$n++;
	}
	my $family_name = $members[0];
	$hash_known_expression_sort{$family_name} = $seed;
}



# =========================================================
# ======== ֶmiRNAڸбˮƽ =====
# =========================================================


open  SAME, ">$mir_same_file" or die;
open  MOST, ">$mir_most_file" or die;
open  TOTAL,">$mir_total_file" or die;
print SAME  "miRNA$header\n";
print MOST  "miRNA$header\n";
print TOTAL "miRNA$header\n";
# ĸ˳򣬷ּmiRNA
for my $family ( sort keys %hash_known_expression_sort ){	
	my $seed = $hash_known_expression_sort{$family};
	for my $mir ( @{ $hash_family_sequenced{$seed} } ){
		my $str_same  = "";
		my $str_most  = "";
		my $str_total = "";
		# λĳmiRNAڸеıˮƽ
		for my $sample ( @sampleNames ){	
			if ( exists $hash_known_expression{$mir}{$sample} ){
				$str_same  .= "\t$hash_known_expression{$mir}{$sample}{same}";
				$str_most  .= "\t$hash_known_expression{$mir}{$sample}{most}";
				$str_total .= "\t$hash_known_expression{$mir}{$sample}{total}";	
			} else {
				$str_same  .= "\t0";
				$str_most  .= "\t0";
				$str_total .= "\t0";
			}
		}
		# 
		my $mirBase_seq = $hash_known_mature{$mir};
		$mirBase_seq =~ s/T/U/ig;
		$seed =~ s/T/U/ig;
		print SAME "$mir$str_same\n";
		print MOST "$mir$str_most\n";
		print TOTAL"$mir$str_total\n";
	}	
}
close SAME;
close MOST;
close TOTAL;


# =========================================================
# ========= ֶRPM׼֮ıˮƽ ========
# =========================================================


open  SAME, ">$mir_same_norm_file" or die;
open  MOST, ">$mir_most_norm_file" or die;
open  TOTAL,">$mir_total_norm_file" or die;
print SAME  "miRNA$header\n";
print MOST  "miRNA$header\n";
print TOTAL "miRNA$header\n";
for my $family ( sort keys %hash_known_expression_sort ){
	my $seed = $hash_known_expression_sort{$family};
	for my $mir ( @{ $hash_family_sequenced{$seed} } ){
		my $str_same  = "";
		my $str_most  = "";
		my $str_total = "";
		for my $sample ( @sampleNames ){
			# RPM׼	
			if((exists $hash_genome_mapped{$sample}) && ($hash_genome_mapped{$sample} > 0)){
				my $s = 0;
				my $m = 0;
				my $t = 0;
				if( exists $hash_known_expression{$mir}{$sample} ){
					$s = ($hash_known_expression{$mir}{$sample}{same}/$hash_genome_mapped{$sample} ) * 1000000;
					$m = ($hash_known_expression{$mir}{$sample}{most}/$hash_genome_mapped{$sample} ) * 1000000;
					$t = ($hash_known_expression{$mir}{$sample}{total}/$hash_genome_mapped{$sample}) * 1000000;			
					$s = MyMath::round($s,6);
					$m = MyMath::round($m,6);
					$t = MyMath::round($t,6);
				}					
				$str_same  .= "\t$s";
				$str_most  .= "\t$m";
				$str_total .= "\t$t";	
			} 
		}
		# 
		my $mirBase_seq = $hash_known_mature{$mir};
		$mirBase_seq =~ s/T/U/ig;
		$seed =~ s/T/U/ig;
		print SAME "$mir$str_same\n";
		print MOST "$mir$str_most\n";
		print TOTAL"$mir$str_total\n";	
	}
}
close SAME;
close MOST;
close TOTAL;


# =========================================================
# ================ miRNAı ==================
# =========================================================


for my $seed ( sort keys %hash_known_family ){
	my @members = split /\t/,$hash_known_family{$seed};
	@members = sort {$a cmp $b} @members;
	my $family_name = $members[0];
	for my $sample ( @sampleNames ){		
		my $s = 0;
		my $m = 0;
		my $t = 0;	
		# ڲԱı֮Ϊı	
		for my $mir ( @members ){		
			if( exists $hash_known_expression{$mir}{$sample} ){			
				$s += $hash_known_expression{$mir}{$sample}{same};
				$m += $hash_known_expression{$mir}{$sample}{most};
				$t += $hash_known_expression{$mir}{$sample}{total};
			}
		}	
		# ¼ı				
		unless(exists $hash_known_family_express{$family_name}{$sample}{same}){
			$hash_known_family_express{$family_name}{$sample}{same}  = $s;
			$hash_known_family_express{$family_name}{$sample}{most}  = $m;
			$hash_known_family_express{$family_name}{$sample}{total} = $t;
		} else {
			print STDERR "Error:($family_name,$sample) repeat\n";
			die;
		}							
	}
}


# =========================================================
# ============ miRNAΪ׼raw ==========
# =========================================================


$mir_same_file  = "$known_miRNA_dir/family.miRBase.txt";
$mir_most_file  = "$known_miRNA_dir/family.most-abundant.txt";
$mir_total_file = "$known_miRNA_dir/family.total.txt";
open  SAME, ">$mir_same_file" or die;
open  MOST, ">$mir_most_file" or die;
open  TOTAL,">$mir_total_file" or die;	
print SAME  "miRNA$header\n";
print MOST  "miRNA$header\n";
print TOTAL "miRNA$header\n";

for my $family ( sort keys %hash_known_family_express ){
	my $str_same  = $family;
	my $str_most  = $family;
	my $str_total = $family;
	for my $sample ( @sampleNames ){	
		if( exists $hash_known_family_express{$family}{$sample} ){
			$str_same  .= "\t$hash_known_family_express{$family}{$sample}{same}";
			$str_most  .= "\t$hash_known_family_express{$family}{$sample}{most}";
			$str_total .= "\t$hash_known_family_express{$family}{$sample}{total}";	
		} else {
			$str_same  .= "\t0";
			$str_most  .= "\t0";
			$str_total .= "\t0";
		}
	}	
	print SAME "$str_same\n";
	print MOST "$str_most\n";
	print TOTAL"$str_total\n";
}
close SAME;
close MOST;
close TOTAL;


# =========================================================
# ============ RPM׼֮miRNA ==========
# =========================================================


$mir_same_norm_file  = "$known_miRNA_dir/family.miRBase.RPM.txt";
$mir_most_norm_file  = "$known_miRNA_dir/family.most-abundant.RPM.txt";
$mir_total_norm_file = "$known_miRNA_dir/family.total.RPM.txt";
open  SAME, ">$mir_same_norm_file" or die;
open  MOST, ">$mir_most_norm_file" or die;
open  TOTAL,">$mir_total_norm_file" or die;
print SAME  "miRNA$header\n";
print MOST  "miRNA$header\n";
print TOTAL "miRNA$header\n";	

for my $family ( sort keys %hash_known_family_express ){
	my $str_same  = $family;
	my $str_most  = $family;
	my $str_total = $family;
	for my $sample ( @sampleNames ){	
		if((exists $hash_genome_mapped{$sample}) && ($hash_genome_mapped{$sample} > 0)){			
			my $s = 0;
			my $m = 0;
			my $t = 0;	
			if( exists $hash_known_family_express{$family}{$sample} ){
				$s = ($hash_known_family_express{$family}{$sample}{same} /$hash_genome_mapped{$sample}) * 1000000;
				$m = ($hash_known_family_express{$family}{$sample}{most} /$hash_genome_mapped{$sample}) * 1000000;
				$t = ($hash_known_family_express{$family}{$sample}{total}/$hash_genome_mapped{$sample}) * 1000000;
				
				$s = MyMath::round($s,6);
				$m = MyMath::round($m,6);
				$t = MyMath::round($t,6);
			} 					
			$str_same  .= "\t$s";
			$str_most  .= "\t$m";
			$str_total .= "\t$t";
		} 
	}
	print SAME "$str_same\n";
	print MOST "$str_most\n";
	print TOTAL"$str_total\n";
}
close SAME;
close MOST;
close TOTAL;


# =========================================================
# ========== ʶ֪miRNA˳ =========
# =========================================================


if( $known ){
	MyFile::end($start_time,\*PARAM);	
	print "\n***************************************\n";
	print "\n***************** END *****************\n";
	print "\n***************************************\n\n\n";
	print PARAM "\n***************************************\n";
	print PARAM "\n***************** END *****************\n";
	print PARAM "\n***************************************\n\n\n";
	close PARAM;
	exit;
}





# ************************************************************** #
# Rfam mapping
# ************************************************************** #



unless ( $no_map_Rfam ) {
		
	print "\nRfam mapping ...\n";
	print PARAM "\nRfam mapping ...\n";
	
	my $Rfam_dir = "$dir/Rfam";
	MyFile::makeDir($Rfam_dir,1);


	# =========================================================
	# =============== RfamȡrRNAtRNA ==============
	# =========================================================
	
	
	my $rRNA_file = "$Rfam_dir/$species.rRNA.fa";
	MyFasta::extractSeqInRfam(\%hash_rRNA_accession,
	$species_Latin,$Rfam_fasta,$rRNA_file);
	my $tRNA_file = "$Rfam_dir/$species.tRNA.fa";
	MyFasta::extractSeqInRfam(\%hash_tRNA_accession,
	$species_Latin,$Rfam_fasta,$tRNA_file);
	
	
	# =========================================================
	# ================== ʶrRNA =================
	# =========================================================


	Bowtie::mapper($rRNA_file,\@unmapped_reads,$Rfam_cmd,
	$Bowtie_path,'dest_dir'=>$Rfam_dir,'del_index'=>1,
	'genome_stat'=>$genome_stat);		
	
	# unmapped readsһmapping
	for my $read_file ( @unmapped_reads ){		
		my $ref_basename  = basename($rRNA_file,@suffix);
		my $read_basename = basename($read_file,@suffix);
		my $unmapped_file = "$Rfam_dir/$ref_basename.$read_basename.unmapped.fa";
		my $mapped_file	  = "$Rfam_dir/$ref_basename.$read_basename.mapped.fa";
		my $mapping_file  = "$Rfam_dir/$ref_basename.$read_basename.mapping.txt";
		
		unlink($read_file);
		copy($unmapped_file,$read_file);
		push(@unmapped_reads_tmp,$read_file);
		
		if ( $is_clean_tmp_file ){
			unlink($mapped_file);
			unlink($unmapped_file);
			unlink($mapping_file);
		}
		
	}

	@unmapped_reads = ();
	@unmapped_reads = @unmapped_reads_tmp;
	@unmapped_reads_tmp = ();


	
	# =========================================================
	# ================== ʶtRNA =================
	# =========================================================
	
	
	Bowtie::mapper($tRNA_file,\@unmapped_reads,$Rfam_cmd,
	$Bowtie_path,'dest_dir'=>$Rfam_dir,'del_index'=>1,
	'genome_stat'=>$genome_stat);			
		
	# unmapped readsһmapping
	for my $read_file ( @unmapped_reads ){		
		my $ref_basename  = basename($tRNA_file,@suffix);
		my $read_basename = basename($read_file,@suffix);
		my $unmapped_file = "$Rfam_dir/$ref_basename.$read_basename.unmapped.fa";
		my $mapped_file	  = "$Rfam_dir/$ref_basename.$read_basename.mapped.fa";
		my $mapping_file  = "$Rfam_dir/$ref_basename.$read_basename.mapping.txt";
		
		
		unlink($read_file);
		copy($unmapped_file,$read_file);
		push(@unmapped_reads_tmp,$read_file);	
		
		if ( $is_clean_tmp_file ){
			unlink($mapped_file);
			unlink($unmapped_file);
			unlink($mapping_file);
		}
		
	}

	@unmapped_reads = ();
	@unmapped_reads = @unmapped_reads_tmp;
	@unmapped_reads_tmp = ();
		
	
	# =========================================================
	# ============= ʶsnRNA/snoRNA ==============
	# =========================================================


	for my $ncRNA_ID_file ( @ncRNAIdFile ){
		unless(-e $ncRNA_ID_file){
			print STDERR "Skip:because $ncRNA_ID_file never exist\n";
			next;
		}
		
		
		# ncRNARfamе¼
		my %hash_accession = ();
		open IN,"<$ncRNA_ID_file" or die;
		while(<IN>){
			chomp;
			my $ac = MyFasta::trim($_);
			$hash_accession{$ac} = 1;
		}
		close IN;
		
		
		# ݵ¼ŴRfamȡncRNA
		my $basename = basename($ncRNA_ID_file,@suffix);
		my $snRNA_snoRNA_file = "$Rfam_dir/$species.$basename.fa";
		MyFasta::extractSeqInRfam(\%hash_accession,
		$species_Latin,$Rfam_fasta,$snRNA_snoRNA_file);
			
			
		# readsȶԵncRNA		
		Bowtie::mapper($snRNA_snoRNA_file,\@unmapped_reads,$Rfam_cmd,
		$Bowtie_path,'dest_dir'=>$Rfam_dir,'del_index'=>1,
		'genome_stat'=>$genome_stat);		
		
			
		# unmapped readsһmapping
		for my $read_file ( @unmapped_reads ){
			my $ref_basename  = basename($snRNA_snoRNA_file,@suffix);
			my $read_basename = basename($read_file,@suffix);
			my $unmapped_file = "$Rfam_dir/$ref_basename.$read_basename.unmapped.fa";
			my $mapped_file	  = "$Rfam_dir/$ref_basename.$read_basename.mapped.fa";
			my $mapping_file  = "$Rfam_dir/$ref_basename.$read_basename.mapping.txt";
	
			
			unlink($read_file);
			copy($unmapped_file,$read_file);
			push(@unmapped_reads_tmp,$read_file);
			
			if ( $is_clean_tmp_file ){
				unlink($mapped_file);
				unlink($unmapped_file);
				unlink($mapping_file);
			}
			
		}

		@unmapped_reads = ();
		@unmapped_reads = @unmapped_reads_tmp;
		@unmapped_reads_tmp = ();
		%hash_accession = ();		
	}
	
} 
	

	
			
# ************************************************************** #
# repeat mapping
# ************************************************************** #

# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
my %hash_repeat_mapping = ();
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #

unless ( $no_map_repeat ){

	print "\nRepeat mapping ...\n";
	print PARAM "\nRepeat mapping ...\n";

	$repeat_dir = "$dir/repeat";
	MyFile::makeDir($repeat_dir,1);


	# =========================================================
	# ================ ʶrepeat =================
	# =========================================================

	Bowtie::mapper($repeat_fasta,\@unmapped_reads,
	$repeat_cmd,$Bowtie_path,'dest_dir'=>$repeat_dir,
	'genome_stat'=>$genome_stat);	

	# unmapped readsһmapping
	for my $read_file ( @unmapped_reads ){	
		my $ref_basename  = basename($repeat_fasta,@suffix);
		my $read_basename = basename($read_file,@suffix);
		my $mapped_file   = "$repeat_dir/$ref_basename.$read_basename.mapped.fa";
		my $unmapped_file = "$repeat_dir/$ref_basename.$read_basename.unmapped.fa";
		my $mapping_file  = "$repeat_dir/$ref_basename.$read_basename.mapping.txt";
		
		unlink($read_file);
		copy($unmapped_file,$read_file);
		push(@unmapped_reads_tmp,$read_file);
		
		$hash_repeat_mapping{$mapping_file} = $mapped_file;
		
		if ( ($is_recall == 0) && ($is_rescue == 0 ) && $is_clean_tmp_file ){
			unlink($mapped_file);
			unlink($unmapped_file);
			unlink($mapping_file);
		}
		
	}
	
	@unmapped_reads = ();
	@unmapped_reads = @unmapped_reads_tmp;
	@unmapped_reads_tmp = ();
	
}




# ************************************************************** #
# mRNA mapping
# ************************************************************** #


unless ( $no_map_mRNA ){

	print "\nmRNA mapping ...\n";
	print PARAM "\nmRNA mapping ...\n";

	my $mRNA_dir = "$dir/mRNA";
	MyFile::makeDir($mRNA_dir,1);


	# =========================================================
	# ================== ʶmRNA =================
	# =========================================================


	Bowtie::mapper($mRNA_fasta,\@unmapped_reads,
	$mRNA_cmd,$Bowtie_path,'dest_dir'=>$mRNA_dir,
	'genome_stat'=>$genome_stat);	

	# unmapped readsһmapping
	for my $read_file ( @unmapped_reads ){	
		my $ref_basename  = basename($mRNA_fasta,@suffix);
		my $read_basename = basename($read_file,@suffix);
		my $unmapped_file = "$mRNA_dir/$ref_basename.$read_basename.unmapped.fa";
		my $mapped_file	  = "$mRNA_dir/$ref_basename.$read_basename.mapped.fa";
		my $mapping_file  = "$mRNA_dir/$ref_basename.$read_basename.mapping.txt";
	
		
		unlink($read_file);
		copy($unmapped_file,$read_file);
		push(@unmapped_reads_tmp,$read_file);	
		
		if ( $is_clean_tmp_file ){
			unlink($mapped_file);
			unlink($unmapped_file);
			unlink($mapping_file);
		}
	}
		
	@unmapped_reads = ();
	@unmapped_reads = @unmapped_reads_tmp;
	@unmapped_reads_tmp = ();
	
}





# ************************************************************** #
# piRNA mapping
# ************************************************************** #


if( (not($no_map_piRNA)) && (-e $piRNA_fasta) ){
	
	print "\npiRNA mapping ...\n";
	print PARAM "\npiRNA mapping ...\n";
	
	$piRNA_dir = "$dir/piRNA";
	MyFile::makeDir($piRNA_dir,1);
	

	# =========================================================
	# ============= ʶ֪piRNA ==============
	# =========================================================
	
	
	Bowtie::mapper($piRNA_fasta,\@unmapped_reads,
	$piRNA_cmd,$Bowtie_path,'dest_dir'=>$piRNA_dir,
	'genome_stat'=>$genome_stat);	

	# unmapped readsһmapping
	my %hash_mapping_files = ();
	for my $read_file ( @unmapped_reads ){		
		my $ref_basename  = basename($piRNA_fasta,@suffix);
		my $read_basename = basename($read_file,@suffix);
		my $mapped_file = "$piRNA_dir/$ref_basename.$read_basename.mapped.fa";
		my $unmapped_file = "$piRNA_dir/$ref_basename.$read_basename.unmapped.fa";
		my $mapping_file = "$piRNA_dir/$ref_basename.$read_basename.mapping.txt";
		$hash_mapping_files{$mapping_file} = $read_basename;
		
		unlink($read_file);
		copy($unmapped_file,$read_file);
		push(@unmapped_reads_tmp,$read_file);
		
		if ( ($is_rescue == 0 ) && $is_clean_tmp_file ){
			unlink($mapped_file);
			unlink($unmapped_file);
			unlink($mapping_file);
		}
	}
		
	@unmapped_reads = ();
	@unmapped_reads = @unmapped_reads_tmp;
	@unmapped_reads_tmp = ();
	
	
	# =========================================================
	# =============== piRNA cluster =============
	# =========================================================
	
	
	if ( $is_piRNA_cluster && ($is_rescue == 0) ){
		#multi-mappingunique mappingreads
		my %hash_piRNA = ();
		my $cluster_file = "$piRNA_dir/";
		piRNA::parser(\%hash_mapping_files,\%hash_piRNA,
		$cluster_file,'unique'=>0,'dist'=>$piRNA_cluster_dist,
		'height'=>$piRNA_cluster_height,'parse_coor'=>1,
		'MIN_LEN'=>$minLen,'MAX_LEN'=>$maxLen);
	}
			
}




# ************************************************************** #
# recall ncRNAs
# ************************************************************** #


# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
my $repeat2rRNA_stat;
my $repeat2tRNA_stat;
my $repeat2snRNA_snoRNA_stat;
my $repeat2piRNA_stat;
my @real_repeat_read_files = ();
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #

if ( $is_recall && (not ($no_map_repeat)) ){

	print "\nrecall ncRNA ...\n";
	print PARAM "\nrecall ncRNA ...\n";
	
	my $recall_dir = "$repeat_dir/recall";
	MyFile::makeDir($recall_dir,1);
	

	my @rescuedTyps = ();
	push(@rescuedTyps,"rRNA");
	push(@rescuedTyps,"tRNA");
	push(@rescuedTyps,"snRNA");

	my %hash_repeat_reads = ();
	Repeat::rescue(\@sampleNames,\@rescuedTyps,\%hash_repeat_mapping,
	\%hash_repeat_reads,$recall_dir);
	
	
	if (-e "$recall_dir/rRNA/rRNA.stat.txt"){
		$repeat2rRNA_stat = "$recall_dir/rRNA/rRNA.stat.txt";
	}
	
	if (-e "$recall_dir/tRNA/tRNA.stat.txt"){
		$repeat2tRNA_stat = "$recall_dir/tRNA/tRNA.stat.txt";
	}
	
	if (-e "$recall_dir/snRNA/snRNA.stat.txt"){
		$repeat2snRNA_snoRNA_stat = "$recall_dir/snRNA/snRNA.stat.txt";
	}
	
	
	for my $sample ( @sampleNames ){
		my $repeat_basename  = basename($repeat_fasta,@suffix);
		my $real_repeat_reads = "$recall_dir/repeat/$repeat_basename.$sample.mapped.fa";
		if (-e $real_repeat_reads){
			push(@real_repeat_read_files,$real_repeat_reads);
		}
	}
	
	
	
	
	# ɾʱļ
	for my $read_file ( @unmapped_reads ){		
		my $repeat_basename  = basename($repeat_fasta,@suffix);
		my $read_basename = basename($read_file,@suffix);
		my $mapped_file   = "$repeat_dir/$repeat_basename.$read_basename.mapped.fa";
		my $unmapped_file = "$repeat_dir/$repeat_basename.$read_basename.unmapped.fa";
		my $mapping_file  = "$repeat_dir/$repeat_basename.$read_basename.mapping.txt";
		
		if ( $is_recall && $is_clean_tmp_file ){
			unlink($mapped_file);
			unlink($unmapped_file);
			unlink($mapping_file);
		}
	}
	
	
	# ɾʱļ
	for my $rescuedType ( @rescuedTyps ) {
	for my $read_file ( @unmapped_reads ){		
		my $repeat_basename  = basename($repeat_fasta,@suffix);
		my $read_basename = basename($read_file,@suffix);
		my $mapped_file = "$recall_dir/$rescuedType/$repeat_basename.$read_basename.mapped.fa";
		my $mapping_file = "$recall_dir/$rescuedType/$repeat_basename.$read_basename.mapping.txt";
	
		if ( $is_recall && $is_clean_tmp_file ){
			unlink($mapped_file);
			unlink($mapping_file);
		}
	}}
	
	
	# ɾʱļ
	for my $read_file ( @unmapped_reads ){		
		my $repeat_basename  = basename($repeat_fasta,@suffix);
		my $read_basename = basename($read_file,@suffix);
		my $mapped_file = "$recall_dir/repeat/$repeat_basename.$read_basename.mapped.fa";
		my $mapping_file = "$recall_dir/repeat/$repeat_basename.$read_basename.mapping.txt";
	
		if ( $is_recall && $is_clean_tmp_file ){
			unlink($mapping_file);
			if ( $is_rescue == 0 ){
				unlink($mapped_file);
			}
		}
	}
	
}



# ************************************************************** #
# rescue piRNAs
# ************************************************************** #


if ( $is_rescue && (not ($no_map_piRNA) ) && (-e $piRNA_fasta) ){
	
	print "\nrescue piRNA ...\n";
	print PARAM "\nrescue piRNA ...\n";

	my $rescue_piRNA_dir = "$piRNA_dir/rescue";
	MyFile::makeDir($rescue_piRNA_dir,1);


	Bowtie::mapper($piRNA_fasta,\@real_repeat_read_files,
	$piRNA_cmd,$Bowtie_path,'dest_dir'=>$rescue_piRNA_dir);	
	
	
	my $piRNA_basename = basename($piRNA_fasta,@suffix);
	if (-e "$rescue_piRNA_dir/$piRNA_basename.stat.txt"){
		$repeat2piRNA_stat = "$rescue_piRNA_dir/$piRNA_basename.stat.txt";
	}
	
	
	# =========================================================
	# =============== piRNA cluster =============
	# =========================================================
	
	
	if ( $is_piRNA_cluster ){
		
		my $piRNA_cluster_dir = "$piRNA_dir/cluster";
		MyFile::makeDir($piRNA_cluster_dir,1);
		
		# ԭʼmappingļrescue֮mappingļϲ
		my %hash_mapping_files = ();
		for my $read_file ( @unmapped_reads ){		
			my $ref_basename    = basename($piRNA_fasta,@suffix);
			my $repeat_basename = basename($repeat_fasta,@suffix);
			my $read_basename   = basename($read_file,@suffix);
			
			my $mapping_file = "$ref_basename.$read_basename.mapping.txt";
			my $rescued_mapping_file = "$ref_basename.$repeat_basename.$read_basename.mapped.mapping.txt";
			
			my $merged_mapping_file = "$piRNA_cluster_dir/$read_basename.mapping.txt";
			open CLUSTEROUT,">$merged_mapping_file" or die;
			
			open CLUSTERIN,"<$piRNA_dir/$mapping_file" or die;
			while(<CLUSTERIN>){
				print CLUSTEROUT $_;
			}
			close CLUSTERIN;
				
			open CLUSTERIN,"<$rescue_piRNA_dir/$rescued_mapping_file" or die;
			while(<CLUSTERIN>){
				print CLUSTEROUT $_;
			}
			close CLUSTERIN;		
			close CLUSTEROUT;	
			
			$hash_mapping_files{$merged_mapping_file} = $read_basename;
			
		}
				
		
		# ѱpiRNAо
		my %hash_piRNA = ();
		my $cluster_file = "$piRNA_cluster_dir/";
		piRNA::parser(\%hash_mapping_files,\%hash_piRNA,
		$cluster_file,'unique'=>0,'dist'=>$piRNA_cluster_dist,
		'height'=>$piRNA_cluster_height,'parse_coor'=>1,
		'MIN_LEN'=>$minLen,'MAX_LEN'=>$maxLen);
		
		# ɾʱļ
		for my $merged_mapping_file ( keys %hash_mapping_files ){
			unlink($merged_mapping_file);
		}
		
	}
	
	
	# ɾʱļ
	if ( $is_recall && $is_rescue && $is_clean_tmp_file ){
		for my $real_repeat_file ( @real_repeat_read_files ){
			unlink($real_repeat_file);
		}
	}
	
	
	# ɾʱļ
	for my $read_file ( @unmapped_reads ){		
		my $ref_basename  = basename($piRNA_fasta,@suffix);
		my $read_basename = basename($read_file,@suffix);
		my $mapped_file   = "$piRNA_dir/$ref_basename.$read_basename.mapped.fa";
		my $unmapped_file = "$piRNA_dir/$ref_basename.$read_basename.unmapped.fa";
		my $mapping_file  = "$piRNA_dir/$ref_basename.$read_basename.mapping.txt";
	
		if ( $is_rescue && $is_clean_tmp_file ){
			unlink($mapped_file);
			unlink($unmapped_file);
			unlink($mapping_file);
		}
	}
	
	
	# ɾʱļ
	for my $read_file ( @unmapped_reads ){		
		my $piRNA_basename  = basename($piRNA_fasta,@suffix);
		my $repeat_basename = basename($repeat_fasta,@suffix);
		my $read_basename = basename($read_file,@suffix);
		my $mapped_file   = "$rescue_piRNA_dir/$piRNA_basename.$repeat_basename.$read_basename.mapped.mapped.fa";
		my $unmapped_file = "$rescue_piRNA_dir/$piRNA_basename.$repeat_basename.$read_basename.mapped.unmapped.fa";
		my $mapping_file  = "$rescue_piRNA_dir/$piRNA_basename.$repeat_basename.$read_basename.mapped.mapping.txt";
		
		if ( $is_rescue && $is_clean_tmp_file ){
			unlink($mapped_file);
			unlink($unmapped_file);
			unlink($mapping_file);
		}
	}
	
}
	
	
	
	
# ************************************************************** #
# category
# ************************************************************** #


if ( $is_category ){
	
	my $category_dir = "$dir/category";
	MyFile::makeDir($category_dir,1);

	
	# ======= category ===========
	my @categories = ();
	push(@categories,"genome");
	push(@categories,"miRNA");
	push(@categories,"rRNA");
	push(@categories,"tRNA");
	push(@categories,"snRNA/snoRNA");
	push(@categories,"repeat");
	push(@categories,"mRNA");
	
	
	# ======= stat files ==========
	my @stat_files = ();
	my %hash_stat_category = ();
	my $genome_basename = basename($genome_fasta,@suffix);
	push(@stat_files,"$dir/genome/$genome_basename.stat.txt");
	$hash_stat_category{"$dir/genome/$genome_basename.stat.txt"} = "genome";
	
	push(@stat_files,"$dir/known/$species.hairpin.stat.txt");
	$hash_stat_category{"$dir/known/$species.hairpin.stat.txt"} = "miRNA";
		
	push(@stat_files,"$dir/Rfam/$species.rRNA.stat.txt");
	$hash_stat_category{"$dir/Rfam/$species.rRNA.stat.txt"} = "rRNA";
	
	push(@stat_files,"$dir/Rfam/$species.tRNA.stat.txt");
	$hash_stat_category{"$dir/Rfam/$species.tRNA.stat.txt"} = "tRNA";
	
	push(@stat_files,"$dir/Rfam/$species.snRNA_snoRNA.stat.txt");
	$hash_stat_category{"$dir/Rfam/$species.snRNA_snoRNA.stat.txt"} = "snRNA/snoRNA";
	
	my $repeat_basename = basename($repeat_fasta,@suffix);
	push(@stat_files,"$dir/repeat/$repeat_basename.stat.txt");
	$hash_stat_category{"$dir/repeat/$repeat_basename.stat.txt"} = "repeat";
	
	my $mRNA_basename = basename($mRNA_fasta,@suffix);
	push(@stat_files,"$dir/mRNA/$mRNA_basename.stat.txt");
	$hash_stat_category{"$dir/mRNA/$mRNA_basename.stat.txt"} = "mRNA";
			
	my $piRNA_basename = basename($piRNA_fasta,@suffix);
	if(-e "$dir/piRNA/$piRNA_basename.stat.txt"){
		push(@categories,"piRNA");
		push(@stat_files,"$dir/piRNA/$piRNA_basename.stat.txt");
		$hash_stat_category{"$dir/piRNA/$piRNA_basename.stat.txt"} = "piRNA";
	}
	
	
	if (-e $repeat2piRNA_stat){
		Category::class($category_dir,\@stat_files,\@categories,\%hash_stat_category,
		\@sampleNames,$repeat2rRNA_stat,$repeat2tRNA_stat,$repeat2snRNA_snoRNA_stat,
		$repeat2piRNA_stat,$repeat_basename);
	} else {
		Category::class($category_dir,\@stat_files,\@categories,\%hash_stat_category,
		\@sampleNames,$repeat2rRNA_stat,$repeat2tRNA_stat,$repeat2snRNA_snoRNA_stat);
	}

}




# ************************************************************** #
# unknown sequences
# ************************************************************** #


	
my $unknown_dir = "$dir/unknown";
MyFile::makeDir($unknown_dir,1);

for my $read_file ( @unmapped_reads ){
	my $read_basename = basename($read_file,@suffix);
	my $unmapped_file = "$unknown_dir/$read_basename.fa";
	
	
	###########################
	my %hash_seq = ();
	MyFasta::readfasta($read_file,\%hash_seq);
	open UNKNOWN,">$unmapped_file" or die;
	for my $id ( sort keys %hash_seq ){
		my $seq = $hash_seq{$id};
		if ( length( $seq ) <= 32 ){
			print UNKNOWN ">$id\n$seq\n";
		}
	}
	close UNKNOWN;
	###########################
		
		
	my $unknown_file = "$unknown_dir/$read_basename-unknown.fa";
	copy($read_file,$unknown_file);
	
	
	push(@unmapped_reads_tmp,$unmapped_file);
	unlink($read_file);
}

@unmapped_reads = ();
@unmapped_reads = @unmapped_reads_tmp;
@unmapped_reads_tmp = ();




# ************************************************************** #
# miRDeep2 
# ************************************************************** #



if ( $miRDeep2 ){
	
	print "\nUse miRDeep2 to predict novel miRNAs ...\n";
	print PARAM "\nUse miRDeep2 to predict novel miRNAs ...\n";

	my $miRDeep2_result_dir = "$dir/miRDeep2";
	MyFile::makeDir($miRDeep2_result_dir,1);


	# =========================================================
	# ==== ֵָmiRNApre-miRNA ======
	# =========================================================
	
	my $other_species_mature  = "$miRDeep2_result_dir/other_species_mature.fa";
	my $other_species_hairpin = "$miRDeep2_result_dir/other_species_hairpin.fa";
	if ( @relatedSpecies ){
		MyFasta::extractSeqInMiRBase1(\@relatedSpecies,$mature_fasta,$hairpin_fasta,
		'mature'=>$other_species_mature,'precursor'=>$other_species_hairpin);
	} else {
		MyFasta::extractSeqInMiRBase1($species,$mature_fasta,$hairpin_fasta,
		'mature'=>$other_species_mature,'precursor'=>$other_species_hairpin,'uncheck'=>1);
	}


	# ÿΪmiRDeep2
	for my $read_file ( @unmapped_reads ){
		
		# Ϊÿһļ
		my $read_basename = basename($read_file,@suffix);
		my $miRDeep2_read_fasta = "$miRDeep2_result_dir/$read_basename.fa";
		my $sample = $read_basename;

		my $dest_dir = "$miRDeep2_result_dir/$sample";
		MyFile::makeDir($dest_dir,1);
		
		
		if( defined $miRDeep2_cmd ){	
			
			# ָbowtiereadsȶԵ
			my @tmp_miRDeep2 = ();
			my $tmp_file = "$dest_dir/read.fa";
			copy($read_file,$tmp_file);
			push(@tmp_miRDeep2,$tmp_file);
			Bowtie::mapper($genome_fasta,\@tmp_miRDeep2,$miRDeep2_cmd,
			$Bowtie_path,'dest_dir'=>$dest_dir,'analysis'=>0);
		
			# ѡȡƥreadsnovel miRNAԤ
			my $ref_basename  = basename($genome_fasta,@suffix);
			my $read_basename = basename($tmp_file,@suffix);	
			my $mapped_file   = "$dest_dir/$ref_basename.$read_basename.mapped.fa";
			unless(-e $mapped_file){
				print STDERR "Error:$mapped_file not exists\n";
				die;
			}
			
			# readsfastaʽתΪmiRDeep2ָĸʽ
			miRDeep2::toMiRDeep2($species,$mapped_file,$miRDeep2_read_fasta);
			
		} else {
			
			# readsfastaʽתΪmiRDeep2ָĸʽ
			miRDeep2::toMiRDeep2($species,$read_file,$miRDeep2_read_fasta);
		}
		
		
		# miRDeep2miRNAʶ
		miRDeep2::doMiRDeep2($genome_fasta,$miRDeep2_read_fasta,
		$species,$this_species_mature,$this_species_precursor,
		$other_species_mature,$miRDeep2_path,$dest_dir);
		
		
		# ɾmiRDeep2װļ£Ľļ
		my $flag = 0; my @files = ();
		MyFile::getFileListInCurrentDir($dest_dir,\@files);
		for my $file ( @files ){
			if($file =~ /^result_/ && $file =~ /\.html$/){		
				my $html_file = "$dest_dir/$file";
				my $parsed_file = "$miRDeep2_result_dir/$read_basename.miRDeep2.txt";
				miRDeep2::parser($species,$html_file,$parsed_file,\%hash_predict,'sample'=>$sample);				
				unless(-e $parsed_file){
					print STDERR "Error:$parsed_file not exists\n";
					die;
				}
				$flag++;
				last;
			}
		}
		unless( $flag ){
			print STDERR "Error:miRDeep2 no results ($sample)\n";
			die;
		}
	} 
} 
	
	
	
	

# ************************************************************** #
# Mireap
# ************************************************************** #	
	
if ( $Mireap ) {

	print "\nUse Mireap to predict novel miRNAs ...\n";
	print PARAM "\nUse Mireap to predict novel miRNAs ...\n";

	my $Mireap_result_dir = "$dir/Mireap";
	MyFile::makeDir($Mireap_result_dir,1);


	# =========================================================
	# ================= MIREAPʶµmiRNA =================
	# =========================================================
	
	Bowtie::mapper($genome_fasta,\@unmapped_reads,$Mireap_cmd,
	$Bowtie_path,'dest_dir'=>$Mireap_result_dir,'analysis'=>0);	

	for my $read_file ( @unmapped_reads ){
		my $ref_basename  = basename($genome_fasta,@suffix);
		my $read_basename = basename($read_file,@suffix);
		my $mapping_file  = "$Mireap_result_dir/$ref_basename.$read_basename.mapping.txt";
		my $sample = $read_basename;
				
		
		# readsfastaʽתΪMIREAPָĸʽ
		my %hash_seq = ();
		MyFasta::readfasta($read_file,\%hash_seq);
		my $Mireap_read_fasta = "$Mireap_result_dir/$read_basename.fa";
		open OUT,">$Mireap_read_fasta" or die;
		for my $id ( keys %hash_seq ){	
			my $tag_name = MyFasta::getTagName($id);
			my $tag_freq = MyFasta::getTagFreq($id);
			my $new_id = "$tag_name $tag_freq";
			print OUT ">$new_id\n$hash_seq{$id}\n";	
		}
		close OUT;
		
	
		# bowtieʽתΪMIREAPʽ	
		my $mireap_file = "$Mireap_result_dir/$read_basename.map.txt";
		Bowtie::bowtie2Mireap($mapping_file,$mireap_file);
		
		
		# MIREAPʶµmiRNA
		Mireap::doMireap($genome_fasta,$Mireap_read_fasta,
		$mireap_file,$Mireap_path,$Mireap_result_dir,'sample'=>$sample);
		
		
		# MIREAPԤ
		my $gff_file = "$Mireap_result_dir/mireap-$sample.gff";
		unless(-e $gff_file){
			print STDERR "Error:$gff_file not exists\n";
			die;
		}
		my $aln_file = "$Mireap_result_dir/mireap-$sample.aln";
		unless(-e $aln_file){
			print STDERR "Error:$aln_file not exists\n";
			die;
		}
		my $parsed_file = "$Mireap_result_dir/$read_basename.Mireap.txt";
		Mireap::parser($gff_file,$aln_file,$parsed_file,\%hash_predict);
		
	}
} 
		

		
# ************************************************************** #
# predict results
# ************************************************************** #	


my $predict_dir = "$dir/predict";
MyFile::makeDir($predict_dir,1);


# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
my $total_predict_mature = 0;
my $predict_mature_fasta = "$predict_dir/predict_mature.fa";
my $predict_precursor_fasta = "$predict_dir/predict_precursor.fa";
my $predict_precursor_struct = "$predict_dir/predict_precursor.struct.txt";
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #


# ԤmiRNA
for my $mature ( sort keys %{ $hash_predict{mature} } ){
	$total_predict_mature++;
}


# ûԤmiRNA˳
unless( $total_predict_mature ){	
	MyFile::end($start_time,\*PARAM);
	print "\n***************************************\n";
	print "\n***************** END *****************\n";
	print "\n***************************************\n\n\n";
	print PARAM "\n***************************************\n";
	print PARAM "\n***************** END *****************\n";
	print PARAM "\n***************************************\n\n\n";
	close PARAM;
	exit;
}
	
	
# ԤmiRNApre-miRNA
open OUT,">$predict_mature_fasta" or die;
for my $mature ( sort keys %{ $hash_predict{mature} } ){
	print OUT ">$mature\n$hash_predict{mature}{$mature}{seq}\n";
}
close OUT;

open OUT,">$predict_precursor_fasta" or die;
for my $precursor ( sort keys %{ $hash_predict{precursor} } ){
	print OUT ">$precursor\n$hash_predict{precursor}{$precursor}{seq}\n";
}
close OUT;

# Ԥpre-miRNAнṹԤ
die if system("perl fold_linux.pl $predict_precursor_fasta $predict_precursor_struct RNA");





# ************************************************************** #
# Delete known miRNAs in the predict results
# ************************************************************** #	



my $del_known_dir = "$dir/delKnown";
MyFile::makeDir($del_known_dir,1);



# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
my %hash_novel = ();
my @predict_matures = ();
my $total_redundant_mature = 0;
my $redundant_mature_fasta = "$del_known_dir/redundant_mature.fa";
my $redundant_precursor_fasta = "$del_known_dir/redundant_precursor.fa";
my $redundant_precursor_struct = "$del_known_dir/redundant_precursor.struct.txt";
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #


# ԤmiRNAȶ֪ǰpre-miRNA
push(@predict_matures,$predict_mature_fasta);
Bowtie::mapper($this_species_precursor,\@predict_matures,$miRBase_cmd,
$Bowtie_path,'dest_dir'=>$del_known_dir,'del_index'=>1,'analysis'=>0);


for my $read_file ( @predict_matures ){
	my $ref_basename  = basename($this_species_precursor,@suffix);
	my $read_basename = basename($read_file,@suffix);
	my $mapping_file  = "$del_known_dir/$ref_basename.$read_basename.mapping.txt";
	
	# ֪miRNAȶԵǰpre-miRNA棬ʶԤmiRNA֪miRNA
	my %hash_parse = ();
	my %hash_mature_tag = ();
	my $outfile_profile_file = "$del_known_dir/$read_basename.aln.txt";
	if(-e $this_species_gff3){
		miRNA::parser($Bowtie_path,\%hash_parse,\%hash_mature_tag,$this_species_mature,
		$this_species_precursor,$mapping_file,$known_precursor_struct,
		$outfile_profile_file,'gff3'=>$this_species_gff3,'dest_dir'=>$del_known_dir);
	} else {
		miRNA::parser($Bowtie_path,\%hash_parse,\%hash_mature_tag,$this_species_mature,
		$this_species_precursor,$mapping_file,$known_precursor_struct,
		$outfile_profile_file,'dest_dir'=>$del_known_dir);
	}
	
	
	# ɾԤmiRNAе֪miRNA
	my %hash_predict_mature = ();
	MyFasta::readfasta($predict_mature_fasta,\%hash_predict_mature);
	
	my %hash_known_tag = ();
	for my $mature ( sort keys %hash_mature_tag ){
		for my $tag ( sort keys %{ $hash_mature_tag{$mature} } ){
			unless(exists $hash_known_tag{$tag}){
				$hash_known_tag{$tag} = 1;
			}
		}
	}		
	
	for my $tag ( sort keys %hash_predict_mature ){
		unless(exists $hash_known_tag{$tag}){
			my $precursor = $hash_predict{mature}{$tag}{parent};
			$hash_novel{mature}{$tag} = 1;
			unless(exists $hash_novel{precursor}{$precursor}){
				$hash_novel{precursor}{$precursor} = 1;
			}
			$total_redundant_mature++;
		}
	}
	
}


# ûµmiRNA˳
unless( $total_redundant_mature ){
	
	if ( $is_clean_tmp_file ){
		MyFile::rm($del_known_dir);
	}

	MyFile::end($start_time,\*PARAM);
	print "\n***************************************\n";
	print "\n***************** END *****************\n";
	print "\n***************************************\n\n\n";
	print PARAM "\n***************************************\n";
	print PARAM "\n***************** END *****************\n";
	print PARAM "\n***************************************\n\n\n";
	close PARAM;
	exit;
} 



# µmiRNApre-miRNA
open OUT,">$redundant_mature_fasta" or die;
for my $mature ( keys %{ $hash_novel{mature} } ){
	print OUT ">$mature\n$hash_predict{mature}{$mature}{seq}\n";
}
close OUT;

open OUT,">$redundant_precursor_fasta" or die;
for my $precursor ( keys %{ $hash_novel{precursor} } ){
	print OUT ">$precursor\n$hash_predict{precursor}{$precursor}{seq}\n";
}
close OUT;

# µǰpre-miRNAнṹԤ
die if system("perl fold_linux.pl $redundant_precursor_fasta $redundant_precursor_struct RNA");

	



# ************************************************************** #
# Scoring
# ************************************************************** #	




# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
my %hash_positive = ();
my $positive_mature_fasta;
my $positive_precursor_fasta;
my $positive_precursor_struct;
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #


unless ( $no_scoring ) {

	print "\nScoring ...\n";
	print PARAM "\nScoring ...\n";

	my $scoring_dir = "$dir/scoring";
	MyFile::makeDir($scoring_dir,1);


	# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
	my %hash_score_sum = ();
	my $total_positive_mature = 0;
	$positive_mature_fasta = "$scoring_dir/positive_mature.fa";
	$positive_precursor_fasta = "$scoring_dir/positive_precursor.fa";
	$positive_precursor_struct = "$scoring_dir/positive_precursor.struct.txt";
	# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #

	# öSVMģͶµpre-miRNAд
	my $SVMmodel_num = (scalar @SVMmodel) - 1;
	for my $m ( 0 .. $SVMmodel_num ){	
		my $model_dir = "$scoring_dir/model$m";
		MyFile::makeDir($model_dir,1);
		
		# ZmirPµmiRNAд
		my $SVM_model = $SVMmodel[$m];
		my $scaled_model = $scaleModel[$m];
		my $scoring_out_file = "$model_dir/out.txt";
		die if system("perl zmirp_predict.pl -L $LIBSVM_path ".
		"-P $redundant_precursor_fasta -M $SVM_model -H $scaled_model -D $model_dir");
		
		# ZmirPԤ
		my %hash_scoring = ();
		MyMath::toMatrix($scoring_out_file,\%hash_scoring);
		for my $precursor ( sort keys %hash_scoring ){
			my $prob = $hash_scoring{$precursor}{1};
			if( $prob >= $probT ){
				unless(exists $hash_score_sum{$precursor}){
					$hash_score_sum{$precursor} = $prob;
				} else {
					$hash_score_sum{$precursor}+= $prob;
				}
			}
		}
	}

	# öSVMģ͵Ԥƽֵ
	$SVMmodel_num++;
	for my $precursor ( sort keys %hash_score_sum ){
		my $avg_prob = $hash_score_sum{$precursor}/$SVMmodel_num;
		# ֻZmirPֵֵΪԵԤ
		if( $avg_prob >= $probT ){
			$hash_positive{precursor}{$precursor} = MyMath::round($avg_prob,6);
		}
	}

	
	# ͳԵġµmiRNAԤĿ
	for my $precursor ( keys %{ $hash_positive{precursor} } ){
		for my $mature ( keys %{ $hash_novel{mature} } ){
			my $parent = $hash_predict{mature}{$mature}{parent};
			if( $parent eq $precursor ){
				unless(exists $hash_positive{mature}{$mature}){
					$hash_positive{mature}{$mature} = 1;
					$total_positive_mature++;
				}
			}
		}
	}


	if ( $total_positive_mature ){
		# ԵġµmiRNAǰpre-miRNA
		open OUT,">$positive_mature_fasta" or die;
		for my $mature ( keys %{ $hash_positive{mature} } ){
			print OUT ">$mature\n$hash_predict{mature}{$mature}{seq}\n";
		}
		close OUT;
		
		open OUT,">$positive_precursor_fasta" or die;
		for my $precursor ( keys %{ $hash_positive{precursor} } ){
			print OUT ">$precursor\n$hash_predict{precursor}{$precursor}{seq}\n";
		}
		close OUT;
		
		# Եġµǰpre-miRNAнṹԤ
		die if system("perl fold_linux.pl $positive_precursor_fasta $positive_precursor_struct RNA");
		
	} else {
		
		# ûԵĽֱµmiRNApre-miRNAк
		print "\nWarn:novel miRNAs are used directly for subsequent analysis\n";
		print PARAM "\nWarn:novel miRNAs are used directly for subsequent analysis\n";
		%hash_positive = %hash_novel;
		$positive_mature_fasta = $redundant_mature_fasta;
		$positive_precursor_fasta = $redundant_precursor_fasta;
		$positive_precursor_struct = $redundant_precursor_struct;
		
	}
	
} else {

	# ZmirPµmiRNAɸѡֱеµmiRNAк
	%hash_positive = %hash_novel;
	$positive_mature_fasta = $redundant_mature_fasta;
	$positive_precursor_fasta = $redundant_precursor_fasta;
	$positive_precursor_struct = $redundant_precursor_struct;
	
}





# ************************************************************** #
# remove redundancy 
# ************************************************************** #	



my $del_redundant_dir = "$dir/delRedundancy";
MyFile::makeDir($del_redundant_dir,1);


# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
my @positive_matures = ();
my %hash_nonredundant = ();
my %hash_predict_tool = ();
my %hash_intersection = ();
my $total_Mireap_mature = 0;
my $total_miRDeep2_mature = 0;
my $total_intersection_mature = 0;
my $total_nonredundancy_mature = 0;
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #


# ԵġµmiRNAȶԵԵġµǰpre-miRNA
push(@positive_matures,$positive_mature_fasta);
Bowtie::mapper($positive_precursor_fasta,\@positive_matures,$miRBase_cmd,
$Bowtie_path,'dest_dir'=>$del_redundant_dir,'del_index'=>1,'analysis'=>0);


for my $read_file ( @positive_matures ){
	my $ref_basename  = basename($positive_precursor_fasta,@suffix);
	my $read_basename = basename($read_file,@suffix);
	my $mapping_file  = "$del_redundant_dir/$ref_basename.$read_basename.mapping.txt";
	
	# ȥԵġµmiRNAеmiRNA
	my %hash_count = ();
	my %hash_parse = ();
	my %hash_mature_tag = ();
	my %hash_precursor_score = ();
	my %hash_precursor_mature = ();
	my $outfile_profile_file = "$del_redundant_dir/$read_basename.aln.txt";
	
	for my $precursor ( keys %{ $hash_positive{precursor} } ){
		$hash_precursor_score{$precursor} = $hash_positive{precursor}{$precursor};
	}
	
	for my $mature ( keys %{ $hash_positive{mature} } ){
		$hash_count{$mature} = $hash_predict{mature}{$mature}{count};
		my $precursor = $hash_predict{mature}{$mature}{parent};
		$hash_precursor_mature{$precursor}{$mature} = $hash_predict{mature}{$mature}{site};
	}
	
	Redundancy::remove(\%hash_count,\%hash_precursor_score,\%hash_precursor_mature,
	$positive_mature_fasta,$positive_precursor_fasta,$mapping_file,
	$positive_precursor_struct,$outfile_profile_file,\%hash_parse,
	\%hash_mature_tag,'BEG_DIF'=>$BEG_DIF,'END_DIF'=>$END_DIF);

	# ͳƷġԵġµmiRNA
	for my $mature ( sort keys %hash_mature_tag ){
		$hash_nonredundant{mature}{$mature} = 1;
		my $precursor = $hash_predict{mature}{$mature}{parent};		
		$hash_nonredundant{precursor}{$precursor} = 1;			
		$total_nonredundancy_mature++;	
	}
	
	# ԤġԵġµmiRNAĹ
	for my $mature ( sort keys %{ $hash_nonredundant{mature} } ){
		my $tool = $hash_predict{mature}{$mature}{tool};
		for my $tag ( sort keys %{ $hash_mature_tag{$mature} } ){	
			my $tag_tool = $hash_predict{mature}{$tag}{tool};
			if($tool !~ /$tag_tool/){
				$tool .= ";$tag_tool";
			}
		}
		$hash_predict_tool{$mature} = $tool;
	}
				
}


# ͳmiRDeep2MIREAPԤġġԵġµmiRNAĿ
for my $mature ( sort keys %{ $hash_nonredundant{mature} } ){
	my $tool = $hash_predict_tool{$mature};
	if( $tool =~ /Mireap/i and $tool =~/miRDeep2/i ){
		my $precursor = $hash_predict{mature}{$mature}{parent};
		$hash_intersection{mature}{$mature} = 1;
		$hash_intersection{precursor}{$precursor} = 1;
		$total_intersection_mature++;
	} elsif ($tool =~ /Mireap/ ){
		$total_Mireap_mature++;
	} elsif ($tool =~ /miRDeep2/ ){
		$total_miRDeep2_mature++;
	} else {
		print STDERR "Error:lack prediction tool ($mature)\n";
		die;
	}
}


# ġԵġµmiRNAĿΪ㣬˳
unless( $total_nonredundancy_mature ){
	
	if ( $is_clean_tmp_file ){
		MyFile::rm($del_known_dir);
		MyFile::rm($del_redundant_dir);
	}

	MyFile::end($start_time,\*PARAM);
	print "\n***************************************\n";
	print "\n***************** END *****************\n";
	print "\n***************************************\n\n\n";
	print PARAM "\n***************************************\n";
	print PARAM "\n***************** END *****************\n";
	print PARAM "\n***************************************\n\n\n";
	close PARAM;
	exit;	
} 





# ************************************************************** #
# novel microRNA expression
# ************************************************************** #	



print "\nNovel miRNA expression profile ...\n";
print PARAM "\nNovel miRNA expression profile ...\n";

my $novel_miRNA_dir = "$dir/novel";
MyFile::makeDir($novel_miRNA_dir,1);


# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #
my %hash_novel_mature = ();
my %hash_novel_expression = ();
my %hash_novel_family = ();
my %hash_novel_family_express = ();
my %hash_novel_expression_sort = ();
my %hash_family_sequenced_novel = ();
my %hash_novel_mature_sequenced = ();
my %hash_nonredundant_struct = ();
my $nonredundant_mature = "$novel_miRNA_dir/nonredundant_mature.fa";
my $nonredundant_precursor = "$novel_miRNA_dir/nonredundant_precursor.fa";
my $nonredundant_precursor_struct = "$novel_miRNA_dir/nonredundant_precursor.struct.txt";
my $novel_miRNA_family = "$novel_miRNA_dir/novel_family.txt";
my $novel_miRNA_list = "$novel_miRNA_dir/novel_microRNA_list.txt";
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ #


# miRDeep2MIREAPĹͬԤķġԵġµmiRNAк
if( ($mode !~/union/i) && $Mireap && $miRDeep2 && $total_intersection_mature){	
	%hash_nonredundant = ();
	%hash_nonredundant = %hash_intersection;	
} 
	

# ġԵġµmiRNAǰpre-miRNA
open OUT,">$nonredundant_mature" or die;
for my $mature ( sort keys %{ $hash_nonredundant{mature} } ){
	print OUT ">$mature\n$hash_predict{mature}{$mature}{seq}\n";
}
close OUT;

open OUT,">$nonredundant_precursor" or die;
for my $precursor ( sort keys %{ $hash_nonredundant{precursor} } ){
	print OUT ">$precursor\n$hash_predict{precursor}{$precursor}{seq}\n";
}
close OUT;

# ġԵġµǰpre-miRNAнṹԤ
die if system("perl fold_linux.pl $nonredundant_precursor $nonredundant_precursor_struct RNA");
	

# ġԵġµmiRNA
MyFasta::readfasta($nonredundant_mature,\%hash_novel_mature);


# δ֪readsȶԵġԵġµǰpre-miRNA
Bowtie::mapper($nonredundant_precursor,\@unmapped_reads,$novel_miR_cmd,
$Bowtie_path,'dest_dir'=>$novel_miRNA_dir,'del_index'=>1);


# ġԵġµmiRNAı
for my $read_file ( @unmapped_reads ){
	my $ref_basename  = basename($nonredundant_precursor,@suffix);
	my $read_basename = basename($read_file,@suffix);
	my $mapping_file  = "$novel_miRNA_dir/$ref_basename.$read_basename.mapping.txt";
	my $unmapped_file = "$novel_miRNA_dir/$ref_basename.$read_basename.unmapped.fa";
	my $mapped_file   = "$novel_miRNA_dir/$ref_basename.$read_basename.mapped.fa";
	my $sample = $read_basename;
	
	
	# ѷġԵġµmiRNAȶԵǰpre-miRNA棬miRNAıˮƽ
	my %hash_parse = ();
	my %hash_mature_tag = ();
	my %hash_precursor_mature = ();
	my $outfile_profile_file = "$novel_miRNA_dir/$read_basename.aln.txt";
	
	for my $mature ( keys %{ $hash_nonredundant{mature} } ){
		my $precursor = $hash_predict{mature}{$mature}{parent};
		$hash_precursor_mature{$precursor}{$mature} = $hash_predict{mature}{$mature}{site};
	}
	
	miRNA::parser($Bowtie_path,\%hash_parse,\%hash_mature_tag,$nonredundant_mature,
	$nonredundant_precursor,$mapping_file,$nonredundant_precursor_struct,
	$outfile_profile_file,'annotation'=>\%hash_precursor_mature,'dest_dir'=>$novel_miRNA_dir);

	
	# ÿзġԵġµmiRNAı
	my $title = "miRNA\ttotal count\tmost count\tstandard count\t";
	$title .= "ratio of most to total\tratio of standard to total\t";
	$title .= "same/differenct\tmiRBase sequence\tmost abundant sequence";
	my $most_seq_file = "$novel_miRNA_dir/$read_basename.most-abundant-sequence.txt";
	open  OUT,">$most_seq_file" or die;
	print OUT "$title\n";
	
	for my $mir ( sort keys %hash_parse ){
		if(exists $hash_parse{$mir}{same}){
			$hash_novel_expression{$mir}{$sample}{same} = $hash_parse{$mir}{same};
		} else {
			$hash_novel_expression{$mir}{$sample}{same} = 0;
		}
		$hash_novel_expression{$mir}{$sample}{total} = $hash_parse{$mir}{total};
		$hash_novel_expression{$mir}{$sample}{most} = $hash_parse{$mir}{most_count};
		
		# ˮƽڿ̽ıˮƽԤ(minFrq)miRNA
		if ( $hash_novel_expression{$mir}{$sample}{most} < $minFrq ){
			next;
		}
		
		# 
		my $most_r = MyMath::round($hash_parse{$mir}{most_count}/$hash_parse{$mir}{total},6);
		my $same_r = MyMath::round($hash_novel_expression{$mir}{$sample}{same}/$hash_parse{$mir}{total},6);
		
		my $strandard_seq = $hash_novel_mature{$mir};
		my $most_seq = $hash_parse{$mir}{most_seq};
		$strandard_seq =~ s/T/U/ig;
		$most_seq =~ s/T/U/ig;
		
		my $str = "$mir\t$hash_parse{$mir}{total}\t$hash_parse{$mir}{most_count}\t";
		$str .= "$hash_novel_expression{$mir}{$sample}{same}\t$most_r\t$same_r\t";
		if( $strandard_seq eq $most_seq ){
			$str .= "S\t$strandard_seq\t$most_seq";
		} else {
			$str .= "D\t$strandard_seq\t$most_seq";
		}
		print OUT "$str\n";
	}
	close OUT;
	
	if ( $is_clean_tmp_file ){
		unlink($mapped_file);
		unlink($unmapped_file);
		unlink($mapping_file);
	}

}



# =========================================================
# ==== ˵¹۲ֵġġԵġµmiRNA =====
# =========================================================

for my $mir ( sort keys %hash_novel_expression ){
	my $exp_flag = 0;
	for my $sample ( @sampleNames ){
		my $t = 0;
		if ( exists $hash_novel_expression{$mir}{$sample} ){
			$t = $hash_novel_expression{$mir}{$sample}{most};
		}
		if ( $t >= $minFrq ){
			$exp_flag++;
		}
	}	
	if( $exp_flag ){
		$hash_novel_mature_sequenced{$mir} = $hash_novel_mature{$mir};
	}
}
	


# =========================================================
# ========= ġԵġµmiRNAĽ ==========
# =========================================================

MyFasta::family($seed_start,
$seed_end,\%hash_novel_mature_sequenced,
\%hash_novel_family,$novel_miRNA_family);

for my $seed ( sort keys %hash_novel_family ){	
	my @members = split /\t/,$hash_novel_family{$seed};
	@members = sort {$a cmp $b} @members;	
	my $n = 0;
	for my $mir ( @members ){
		$hash_family_sequenced_novel{$seed}[$n] = $mir;
		$n++;
	}	
	my $family_name = $members[0];
	$hash_novel_expression_sort{$family_name} = $seed;
}



# =========================================================
# = ֶġԵġµmiRNAڱˮƽ =
# =========================================================

$mir_same_file  = "$novel_miRNA_dir/miRNA.standard.txt";
$mir_most_file  = "$novel_miRNA_dir/miRNA.most-abundant.txt";
$mir_total_file = "$novel_miRNA_dir/miRNA.total.txt";
open  SAME, ">$mir_same_file" or die;
open  MOST, ">$mir_most_file" or die;
open  TOTAL,">$mir_total_file" or die;
print SAME  "miRNA$header\n";
print MOST  "miRNA$header\n";
print TOTAL "miRNA$header\n";

# ĸ˳򣬷ּmiRNA
for my $family ( sort keys %hash_novel_expression_sort ){
	my $seed = $hash_novel_expression_sort{$family};
	for my $mir ( @{ $hash_family_sequenced_novel{$seed} } ){
		my $str_same  = "";
		my $str_most  = "";
		my $str_total = "";
		# λĳڸеıˮƽ
		for my $sample ( @sampleNames ){	
			if( exists 	$hash_novel_expression{$mir}{$sample} ){
				$str_same  .= "\t$hash_novel_expression{$mir}{$sample}{same}";
				$str_most  .= "\t$hash_novel_expression{$mir}{$sample}{most}";
				$str_total .= "\t$hash_novel_expression{$mir}{$sample}{total}";	
			} else {
				$str_same  .= "\t0";
				$str_most  .= "\t0";
				$str_total .= "\t0";
			}
		}	
		# 
		my $most_seq = $hash_novel_mature{$mir};
		$most_seq =~ s/T/U/ig;
		$seed =~ s/T/U/ig;		
		print SAME "$mir$str_same\n";
		print MOST "$mir$str_most\n";
		print TOTAL"$mir$str_total\n";
	}
}
close SAME;
close MOST;
close TOTAL;



# =========================================================
# ========= ֶRPM׼֮ıˮƽ ========
# =========================================================

$mir_same_norm_file  = "$novel_miRNA_dir/miRNA.standard.RPM.txt";
$mir_most_norm_file  = "$novel_miRNA_dir/miRNA.most-abundant.RPM.txt";
$mir_total_norm_file = "$novel_miRNA_dir/miRNA.total.RPM.txt";
open  SAME, ">$mir_same_norm_file" or die;
open  MOST, ">$mir_most_norm_file" or die;
open  TOTAL,">$mir_total_norm_file" or die;
print SAME  "miRNA$header\n";
print MOST  "miRNA$header\n";
print TOTAL "miRNA$header\n";	
	
for my $family ( sort keys %hash_novel_expression_sort ){	
	my $seed = $hash_novel_expression_sort{$family};	
	for my $mir ( @{ $hash_family_sequenced_novel{$seed} } ){		
		my $str_same  = "";
		my $str_most  = "";
		my $str_total = "";
		for my $sample ( @sampleNames ){	
			if((exists $hash_genome_mapped{$sample}) && ($hash_genome_mapped{$sample} > 0)){				
				my $s = 0;
				my $m = 0;
				my $t = 0;	
				# RPM׼			
				if( exists $hash_novel_expression{$mir}{$sample} ){
					$s = ($hash_novel_expression{$mir}{$sample}{same}/$hash_genome_mapped{$sample} ) * 1000000;
					$m = ($hash_novel_expression{$mir}{$sample}{most}/$hash_genome_mapped{$sample} ) * 1000000;
					$t = ($hash_novel_expression{$mir}{$sample}{total}/$hash_genome_mapped{$sample}) * 1000000;					
					$s = MyMath::round($s,6);
					$m = MyMath::round($m,6);
					$t = MyMath::round($t,6);
				} 						
				$str_same  .= "\t$s";
				$str_most  .= "\t$m";
				$str_total .= "\t$t";				
			}
		}
		# 
		my $most_seq = $hash_novel_mature{$mir};
		$most_seq =~ s/T/U/ig;
		$seed =~ s/T/U/ig;
		print SAME "$mir$str_same\n";
		print MOST "$mir$str_most\n";
		print TOTAL"$mir$str_total\n";	
	}
}
close SAME;
close MOST;
close TOTAL;



# =========================================================
# ================ miRNAı ==================
# =========================================================

for my $seed ( sort keys %hash_novel_family ){
	my @members = split /\t/,$hash_novel_family{$seed};
	@members = sort {$a cmp $b} @members;
	my $family_name = $members[0];
	for my $sample ( @sampleNames ){		
		my $s = 0;
		my $m = 0;
		my $t = 0;		
		# ڲԱı֮Ϊı	
		for my $mir ( @members ){		
			if( exists $hash_novel_expression{$mir}{$sample} ){			
				$s += $hash_novel_expression{$mir}{$sample}{same};
				$m += $hash_novel_expression{$mir}{$sample}{most};
				$t += $hash_novel_expression{$mir}{$sample}{total};
			}
		}		
		# ¼ı		
		unless(exists $hash_novel_family_express{$family_name}{$sample}{same}){
			$hash_novel_family_express{$family_name}{$sample}{same}  = $s;
			$hash_novel_family_express{$family_name}{$sample}{most}  = $m;
			$hash_novel_family_express{$family_name}{$sample}{total} = $t;
		} else {
			print STDERR "Error:($family_name,$sample) repeat\n";
			die;
		}						
	}
}
		

# =========================================================
# ============ miRNAı׼raw ==========
# =========================================================
	

$mir_same_file  = "$novel_miRNA_dir/family.standard.txt";
$mir_most_file  = "$novel_miRNA_dir/family.most-abundant.txt";
$mir_total_file = "$novel_miRNA_dir/family.total.txt";
open  SAME, ">$mir_same_file" or die;
open  MOST, ">$mir_most_file" or die;
open  TOTAL,">$mir_total_file" or die;
print SAME  "miRNA$header\n";
print MOST  "miRNA$header\n";
print TOTAL "miRNA$header\n";

for my $family ( sort keys %hash_novel_family_express ){	
	my $str_same  = $family;
	my $str_most  = $family;
	my $str_total = $family;
	for my $sample ( @sampleNames ){	
		if( exists $hash_novel_family_express{$family}{$sample} ){
			$str_same  .= "\t$hash_novel_family_express{$family}{$sample}{same}";
			$str_most  .= "\t$hash_novel_family_express{$family}{$sample}{most}";
			$str_total .= "\t$hash_novel_family_express{$family}{$sample}{total}";	
		} else {
			$str_same  .= "\t0";
			$str_most  .= "\t0";
			$str_total .= "\t0";
		}
	}
	print SAME "$str_same\n";
	print MOST "$str_most\n";
	print TOTAL"$str_total\n";
}
close SAME;
close MOST;
close TOTAL;



# =========================================================
# ============ RPM׼֮miRNA ==========
# =========================================================

$mir_same_norm_file  = "$novel_miRNA_dir/family.standard.RPM.txt";
$mir_most_norm_file  = "$novel_miRNA_dir/family.most-abundant.RPM.txt";
$mir_total_norm_file = "$novel_miRNA_dir/family.total.RPM.txt";
open  SAME, ">$mir_same_norm_file" or die;
open  MOST, ">$mir_most_norm_file" or die;
open  TOTAL,">$mir_total_norm_file" or die;	
print SAME  "miRNA$header\n";
print MOST  "miRNA$header\n";
print TOTAL "miRNA$header\n";	

for my $family ( sort keys %hash_novel_family_express ){	
	my $str_same  = $family;
	my $str_most  = $family;
	my $str_total = $family;	
	for my $sample ( @sampleNames ){	
		if((exists $hash_genome_mapped{$sample}) && ($hash_genome_mapped{$sample} > 0)){		
			my $s = 0;
			my $m = 0;
			my $t = 0;			
			if( exists $hash_novel_family_express{$family}{$sample} ){
				$s = ($hash_novel_family_express{$family}{$sample}{same} /$hash_genome_mapped{$sample}) * 1000000;
				$m = ($hash_novel_family_express{$family}{$sample}{most} /$hash_genome_mapped{$sample}) * 1000000;
				$t = ($hash_novel_family_express{$family}{$sample}{total}/$hash_genome_mapped{$sample}) * 1000000;			
				$s = MyMath::round($s,6);
				$m = MyMath::round($m,6);
				$t = MyMath::round($t,6);
			} 				
			$str_same  .= "\t$s";
			$str_most  .= "\t$m";
			$str_total .= "\t$t";
		}
	}
	print SAME "$str_same\n";
	print MOST "$str_most\n";
	print TOTAL"$str_total\n";
}
close SAME;
close MOST;
close TOTAL;



# =========================================================
# ============ RPM׼֮miRNA ==========
# =========================================================

# ġԵġµpre-miRNAĽṹ
Feature::parse_struct($nonredundant_precursor_struct,
\%hash_nonredundant_struct);

# ġԵġµmiRNAıˮƽZmirPֵСܡԤ⹤ߵϢ
open  OUT,">$novel_miRNA_list" or die;
print OUT "miRNA\tpre-miRNA\tprobability\tMFE(kcal/mol)\tbifurcation\t";
print OUT "tool\tmature coordinate\tprecursor coordinate\n";
for my $mature ( sort keys %{ $hash_nonredundant{mature} } ){
	my $mature_coor = $hash_predict{mature}{$mature}{coor};
	my $precursor = $hash_predict{mature}{$mature}{parent};
	my $prob = $hash_positive{precursor}{$precursor};
	my $precursor_coor = $hash_predict{precursor}{$precursor}{coor};
	my $mfe = MyMath::round($hash_nonredundant_struct{$precursor}{mfe},2);
	my $struct = $hash_nonredundant_struct{$precursor}{struct};
	my $bifurcation = Feature::is_bifurcation($struct);
	my $tool = $hash_predict_tool{$mature};
	my $str = "$mature\t$precursor\t$prob\t$mfe\t$bifurcation\t";
	$str .= "$tool\t$mature_coor\t$precursor_coor";
	print OUT "$str\n";
}
close OUT;




# ************************************************************** #
# End Program
# ************************************************************** #	

if ( $is_clean_tmp_file ){
	MyFile::rm($del_known_dir);
	MyFile::rm($del_redundant_dir);
}

MyFile::end($start_time,\*PARAM);
print "\n***************************************\n";
print "\n***************** END *****************\n";
print "\n***************************************\n\n\n";
print PARAM "\n***************************************\n";
print PARAM "\n***************** END *****************\n";
print PARAM "\n***************************************\n\n\n";
close PARAM;
exit;




# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub get_species {
	
	my $species = shift or die;
	my $hash_species_Latin = shift or die;
	my $hash_species_miRBase = shift or die;
	
	
	my $miRBase_organisms_file = "organisms";
	unless(-e $miRBase_organisms_file){
		print STDERR "Error:$miRBase_organisms_file not exists\n";
		die;
	}

	open IN,"<$miRBase_organisms_file" or die;
	while(<IN>){
		chomp;
		next if /^\s*$/;
		my $line = MyFasta::trim($_);
		my @tmp = split/\t/,$line;
		unless(exists $hash_species_miRBase->{$tmp[0]}){
			$hash_species_miRBase->{$tmp[0]} = $tmp[1];
		} else {
			print STDERR "Error:$tmp[0] repeat in $miRBase_organisms_file\n";
			die;
		}
		unless(exists $hash_species_Latin->{$tmp[1]}){
			$hash_species_Latin->{$tmp[1]} = $tmp[0];
		} else {
			print STDERR "Error:$tmp[1] repeat in $miRBase_organisms_file\n";
			die;
		}
	}
	close IN;

	my $species_Latin = "";
	unless (exists $hash_species_miRBase->{$species}){
		print STDERR "\n\nError:$species is not a name of species in miRBase database\n";
		print STDERR "Please use name of species in miRBase\n";
		print STDERR "Please look at 'organisms' in current directory to get it\n\n\n";
		die;
	} else {
		$species_Latin = $hash_species_miRBase->{$species};
		unless (exists $hash_species_Latin->{$species_Latin}){
			print STDERR "Error:miRBase name and Latin name inconsistent\n";
			die;
		}
	}
	
	return $species_Latin;
}


# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================


sub usage {
	
my $usage = << "USAGE";

Version: 1.0
Contact: Yao Yuangen <yygen89\@163.com>

Usage:

	********************************************************************
	
	Options:
	
	--species/-S <species> : the three letter acronym of specified species in miRBase database. More details about it, please read organisms file in current CSZ distribution.
	
	--samples/-F <fasta> : fasta files containing the deep sequencing data in the specified format
	
	--data_path/-I <path> : the path to directory containing all required data.
	
	--tools/-T <Mireap and/or miRDeep2> : the names of tools using to identify novel miRNAs in CSZ
	
	--mode/-M [union/intersection] : the way to combine predicted results of MIREAP and miRDeep2. This is optional and the default value is union.
	
	--prepro/-r [1/0] : preprocess reads before mapping it against reference genome if this function is activated (default: 1) where 1 stand for activation 
	
	--known/-K [1/0] : quit immediately after detecting all known miRNAs if this function is activated (default: 0)
	
	--probT/-P [float] : the probability threshold. If ZmirP score is less than this probability threshold, then this miRNA is filtered directly out from downstream analysis (default: 0.5). 
	
	--minLen/-l [int] : the minimum of read length. If the length of reads is less than the lower bound of read length, then this reads will be directly filtered out (default: 18 nt).
	
	--maxLen/-L [int] : the maximum of read length. If the length of reads is greater than the upper limit of read length, this reads will be directly filtered out (default: 35 nt).
	
	--minFrq/-q [int] : the observed frequency of read. If the occurrence of read is less than this lower bound of observed frequency of read, then it will be removed out (default: 3).
	
	--Bowtie_path/-B [path] : the path to Bowtie (default: ./Packages/mirdeep2/essentials/bowtie-0.12.5)
	
	--Mireap_path/-E [path] : the path to Mireap (default: ./Packages/mireap_0.2)
	
	--miRDeep2_path/-D [path] : the path to miRDeep2 (default: ./Packages/mirdeep2)
	
	--LIBSVM_path/-C [path] : the path to LIBSVM (default: ./Packages/libsvm-3.16)
	
	--SVMmodel/-M <SVM model> : the SVM model file containing parameters and support vectors
	
	--scaleModel/-H <scaled model> : the scaled model file containing parameters for scaling input data for LIBSVM
	
	--dir/-d [dir] : the result directory. This is the optional.
	
	--del_tmp/-w [1/0] : delete temporary files (default: 0)
	
	--help/-h : print help information and quit
	
	--genome_fasta [fasta] : FASTA file containing reference genome sequences downloaded from UCSC database
	
	--mature_fasta [fasta] : FASTA file containing sequences of all mature miRNA sequences deposited in miRBase database
	
	--hairpin_fasta [fasta] : FASTA file containing sequences of all miRNA hairpins deposited in miRBase database 
	
	--Rfam_fasta [fasta]	: FASTA file containing all kinds of non-coding RNA deposited in Rfam database
	
	--repeat_fasta [fasta] : FASTA file containing repeat sequences extracted from reference genome
	
	--mRNA_fasta [fasta]	 : FASTA file containing sequences of RefSeq mRNA downloaded from UCSC database
	
	--piRNA_fasta [fasta]	 : FASTA file containing sequences of piRNAs extracted from piRNABank database
	
	Optional PARAMETERS for Bowtie:

	--threads/-p [int] : launch <int> parallel search threads (default: 1). For more details about it, please read manual of Bowtie
	
	--mismatches/-v [int] : report alignments with at most <int> mismatches (default: 1) For more details about it, please read manual of Bowtie
	
	--genome_cmd [cmd] : the command of Bowtie for reads mapping against reference genome, enclosed in "" if more than one word (default: "bowtie -p threads -t -f -v mismatches") where threads and mismatches stand for respective the number of threads and mismatches
	
	--miRBase_cmd [cmd] : the command of Bowtie for reads mapping against miRBase, enclosed in "" if more than one word (default: "bowtie -p threads -t -f -v mismatches -a --best --strata --norc")
	
	--piRNA_cmd [cmd]	: the command of Bowtie for reads mapping against piRNAs, enclosed in "" if more than one word (default: "bowtie -p threads -t -f -v mismatches -a --best --strata --norc")

	--Rfam_cmd [cmd] : the command of Bowtie for reads mapping against ncRNAs deposited in Rfam, enclosed in "" if more than one word (default: "bowtie -p threads -t -f -v mismatches -a --best --strata --norc")
	
	--repeat_cmd [cmd] : the command of Bowtie for reads mapping against repeat sequences, enclosed in "" if more than one word (default: "bowtie -p threads -t -f -v mismatches")
	
	--mRNA_cmd [cmd] : the command of Bowtie for reads mapping against mRNAs, enclosed in "" if more than one word (default: "bowtie -p threads -t -f -v mismatches -a --best --strata --norc")
	
	********************************************************************
	
	perl $0 [options] -I <data path> -T <Mireap> and/or -T <miRDeep2> 
	-V <SVM model> -H <scaled model> -F <sample_1.fa> -F <sample_2.fa> ...

	********************************************************************
		
USAGE
print $usage;

}

