#!/usr/bin/perl
use strict;
use warnings;
use RNA; # Vienna RNA package perl interface



my $usage = "< fasta file > < outfile > [DNA or RNA]";
my $infile_fasta 	= shift or die $usage;
my $outfile_struct	= shift or die $usage;
my $DNA_or_RNA 		= shift;

if ( defined $DNA_or_RNA ){
	if( $DNA_or_RNA =~ /DNA/i ){
		&fold_linux_DNA($infile_fasta,$outfile_struct);
	} else {
		&fold_linux_RNA($infile_fasta,$outfile_struct);
	}
} else {
	&fold_linux_RNA($infile_fasta,$outfile_struct);
}





# =============================
# Function: fold the sequence with RNAfold
# Input: infile for fasta
#		 outfile for secondary structure
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub fold_linux {
	
	my $infile_fasta 	= shift or die;
	my $outfile_struct	= shift or die;
	
	my %hash_seq = ();
	&readfasta($infile_fasta,\%hash_seq);
	
	open OUT,">$outfile_struct" or die;
	for my $id ( keys %hash_seq ){
		my $seq = $hash_seq{$id};
		$seq = uc ( $seq );
		$seq =~ s/^\s*//g;
		$seq =~ s/\s*$//g;
		unless( not ($seq =~/^([A|C|G|T|U|N]+)$/) ){
			my ($struct,$mfe) = RNA::fold($seq);
			$mfe = &round($mfe,6);
			print OUT ">$id\n$seq\n$struct ($mfe)\n";
		}	
	}
	close OUT;
}



# =============================
# Function: fold the sequence with RNAfold
# Input: infile for fasta
#		 outfile for secondary structure
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub fold_linux_DNA {
	
	my $infile_fasta 	= shift or die;
	my $outfile_struct	= shift or die;
	
	my %hash_seq = ();
	&readfasta($infile_fasta,\%hash_seq);
	
	open OUT,">$outfile_struct" or die;
	for my $id ( keys %hash_seq ){
		my $seq = $hash_seq{$id};
		$seq = uc ( $seq );
		$seq =~ s/U/T/ig;
		$seq =~ s/^\s*//g;
		$seq =~ s/\s*$//g;
		unless( not ($seq =~/^([A|C|G|T|N]+)$/) ){
			my ($struct,$mfe) = RNA::fold($seq);
			$mfe = &round($mfe,6);
			print OUT ">$id\n$seq\n$struct ($mfe)\n";
		}	
	}
	close OUT;
}



# =============================
# Function: fold the sequence with RNAfold
# Input: infile for fasta
#		 outfile for secondary structure
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub fold_linux_RNA {
	
	my $infile_fasta 	= shift or die;
	my $outfile_struct	= shift or die;
	
	my %hash_seq = ();
	&readfasta($infile_fasta,\%hash_seq);
	
	open OUT,">$outfile_struct" or die;
	for my $id ( keys %hash_seq ){
		my $seq = $hash_seq{$id};
		$seq = uc ( $seq );
		$seq =~ s/T/U/ig;
		$seq =~ s/^\s*//g;
		$seq =~ s/\s*$//g;
		unless( not ($seq =~/^([A|C|G|U|N]+)$/) ){
			my ($struct,$mfe) = RNA::fold($seq);
			$mfe = &round($mfe,6);
			print OUT ">$id\n$seq\n$struct ($mfe)\n";
		}	
	}
	close OUT;
}





# =============================
# Function: read one or more fasta sequencs into hash table.
# Input: a fasta file and reference of hash table.
# Return: the number of fasta sequence.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub readfasta {
	
	my $usage = "< fasta file > < hash reference >";
	my $infile = shift or die $usage;
	my $hash_seq = shift or die $usage;
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists";
		die;
	}
	
	open IN, $infile || die;
	
	my $c=0;
	my $seqId;
	while (defined (my $line=<IN>)) {
		chomp($line);
		next if $line=~/^\s*$/;
		if ($line =~/^>/) {
			$line =~s/\s*$//g;	
			$line =~s/^\s*//g;	
			$seqId = substr($line,1);
			$c++;
		} else {
			$line =~s/\s*//g;
			$hash_seq->{$seqId}.=$line;
		}
	}
	close IN;
	
	return $c;
	
}




# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub round {
	
	my $usage = "<  numerical value > < the numbers after the decimal point >";
    my $val = shift;
    my $col = shift;
    
    
    my $r = 10 ** $col;
    my $a = ($val > 0) ? 0.5 : -0.5;
    
    return int($val * $r + $a) / $r;
    
}



#######################################################

