package miRDeep2;
#!/usr/bin/perl
use strict;
use warnings;
use Cwd;
use File::Copy;
use File::Basename;


# My Library
use MyFile;
use MyFasta;



# =============================
# Function: change the format to required format met miRDeep2
# Input: a fasta file and 
# 		 outfile for paresed fasta 
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub toMiRDeep2 {
	
	my $species = shift or die;
	my $infile_fasta = shift or die;
	my $outfile_fasta = shift or die;
		
	unless(-e $infile_fasta){
		print STDERR "Error:$infile_fasta not exists\n";
		die;
	}
	
	my %hash_seq = ();
	MyFasta::seqkey($infile_fasta,\%hash_seq);
	
		
	my $sn = 1;
	open MIRDEEP,">$outfile_fasta" or die;
	for my $seq ( sort keys %hash_seq ){
		my $freq = $hash_seq{$seq};
		my $new_id = $species."_".$sn."_x".$freq;
		print MIRDEEP ">$new_id\n$seq\n";
		$sn++;
	}
	close MIRDEEP;
	
}
		



# =============================
# Function: use miRDeep2 to predict novel microRNAs
# Input: reference fasta
#		 basename of bowtie index
#		 read fasta
#		 species
#		 mature fasta for reference species
#		 precursor fasta for reference species
#		 mature fasta for other species
#		 miRDeep2 bin path
# 		 outfile for paresed fasta 
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub doMiRDeep2 {
	
	my $ref_genome  			= shift or die;
	my $read_fasta 				= shift or die;
	my $this_species			= shift or die;
	my $this_species_mature 	= shift or die;
	my $this_species_precursor 	= shift or die;
	my $other_species_mature 	= shift or die;
	my $mirdeep2_path 			= shift or die;
	my $dest_dir			 	= shift or die;
	
	unless(-e $ref_genome){
		print STDERR "Error:$ref_genome not exists\n";
		die;
	}
	unless(-e $read_fasta){
		print STDERR "Error:$read_fasta not exists\n";
		die;
	}
	unless(-e $this_species_mature){
		print STDERR "Error:$this_species_mature not exists\n";
		die;
	}
	unless(-e $this_species_precursor){
		print STDERR "Error:$this_species_precursor not exists\n";
		die;
	}
	unless(-e $other_species_mature){
		print STDERR "Error:$other_species_mature not exists\n";
		die;
	}
	
	my @suffix = qw(.fa .txt);
	my $ref_dirname = dirname($ref_genome);
	my $ref_basename = basename($ref_genome,@suffix);
	my $ebwt_base_name = "$ref_dirname/$ref_basename";
	my $read_dirname = dirname($read_fasta);
	my $read_basename = basename($read_fasta,@suffix);
	my $arf_file = "$read_dirname/$read_basename.arf";
	
	
	# check Bowtie indexes
	my $build_flag = 1;
	for my $i ( 1 .. 4 ){
		unless( -e $ebwt_base_name.".$i.ebwt" ){
			$build_flag = 0;
		}
	}
	unless( -e $ebwt_base_name.".rev.1.ebwt" ){
		$build_flag = 0;
	}
	unless( -e $ebwt_base_name.".rev.2.ebwt" ){			
		$build_flag = 0;
	}
	
	unless( $build_flag ){	
		print STDERR "Error:$ebwt_base_name not exists\n";
		die;
	}
	
	
	my $current_dir = getcwd;
	chdir($mirdeep2_path);
	
	
	unless(-e "miRDeep2.pl"){
		print STDERR "Error: error miRDeep2 path ($mirdeep2_path)\n";
		die;
	}
	unlink("mapper.log") if -e "mapper.log";
	unlink($arf_file) if -e $arf_file;
	
	
	
	my @tmp_files = ();
	my @expressions = qw(result_ expression_ mirdeep_runs pdfs_ miRNAs_expressed dir_perform_);
	MyFile::getFileListInCurrentDir($mirdeep2_path,\@tmp_files,\@expressions);
	
	for my $file ( @tmp_files ){
		MyFile::rm($file);
	}
	
	
	die if system("mapper.pl $read_fasta -c -p $ebwt_base_name -t $arf_file");
	
	die if system("quantifier.pl -p $this_species_precursor ".
	"-m $this_species_mature -r $read_fasta -t $this_species");
	
	die if system("miRDeep2.pl $read_fasta $ref_genome $arf_file $this_species_mature ".
	"$other_species_mature $this_species_precursor -t $this_species 2 -v");
	
	
	@tmp_files = ();
	MyFile::getFileListInCurrentDir($mirdeep2_path,\@tmp_files,\@expressions);
	for my $file ( @tmp_files ){
		if( -d $file ){
			MyFile::copyDir($file,"$dest_dir/$file");
		} else {
			move($file,$dest_dir);
		}
		MyFile::rm($file);
	}
	unlink("mapper.log") if -e "mapper.log";
	unlink($arf_file) if -e $arf_file;
	
	
	chdir($current_dir);
	
}	
	




# =============================
# Function: parse the result of miRDeep2
# Input: species
#		 a fasta file and 
# 		 outfile for paresed fasta 
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub parser {
	
	my $species			= shift or die;
	my $infile_html 	= shift or die;
	my $outfile_parse 	= shift or die;
	my $hash_sequence 	= shift or die;
	
	
	my $sample;
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/sample/i) {$sample=shift @_}
	}
	

	unless(-e $infile_html){
		print STDERR "Error:$infile_html not exists\n";
		die;
	}


	# +++++++++++++++++++++++++++++++++++++ #
	
	
	my @suffix = qw(.html .txt .fa);
	my $dirname = dirname($infile_html);
	my $basename = basename($infile_html,@suffix);


	my $txt_file = "$dirname/$basename.txt";
	MyFile::html2txt($infile_html,$txt_file);
	
	
	# +++++++++++++++++++++++++++++++++++++ #
	
	my $str = "";
	my %hash_parse = ();
	
	
	
	open IN,"<$txt_file" or die;
	while(defined( my $line = <IN> )){
		chomp($line);
		next if $line=~/^\s*$/;	
		
		$str .= "$line\t";	
		
		if ( $str =~/\S+:\d+\.\.\d+:\S+/ ){	
			
			my $str_tail = $';		
					
			my ($id,$precursor_count,$mature_count,$loop_count,$star_count);
			my ($mature_name,$mature_seq,$star_seq,$precursor_seq,$coordinate);
			
			if($str =~ /\bchr\d+_|\bZv9_/){
				$str = $&.$';
			} else {
				$str = $str_tail;
				next;
			}
			
			$str = MyFasta::trim($str);

			if( $str =~/yes|no/ ){
				
				my $pre_str = $`;
				$pre_str = MyFasta::trim($pre_str);
				my @tmp = split /\s+/,$pre_str;		
				$star_count   = pop( @tmp );
				$loop_count   = pop( @tmp );
				$mature_count = pop( @tmp );
				$precursor_count  = pop( @tmp );
			
				my $flag = 0;
				for my $tmp_str ( @tmp ){
					if($tmp_str =~ /\bchr\d+_|\bZv9_/){	
						$id = $tmp_str;
						$flag++;
						last;
					}
				}
			
				unless( $flag ){
					print STDERR "Error:$str\n";
					die;
				}
				$str = $';
			}

			my $flag = 0;
			$mature_name = "novel";	
			$str = MyFasta::trim($str);
			
			if( $str =~/\bblast\b/i ){
				
				my $pre_str = $`;
				if(length($pre_str)){
					$pre_str = MyFasta::trim($pre_str);
					my @tmpA = split /\s+/,$pre_str;
					for my $tmp_str ( @tmpA ){
						if($tmp_str =~ /\b$species/){
							$tmp_str = MyFasta::trim($tmp_str);
							$mature_name = $tmp_str;
						} 	
					}
				}
				
				my $suffix_str = $'; 	
				$suffix_str = MyFasta::trim($suffix_str);
				my @tmp = split /\s+/,$suffix_str;
				$mature_seq 	= shift( @tmp );
				$star_seq   	= shift( @tmp );
				$precursor_seq 	= shift( @tmp );
				$coordinate  	= shift( @tmp );
				$flag = 1;
			}
			
			
			unless(exists $hash_parse{$id}{total_count}){
				$hash_parse{$id}{mature_name}	= $mature_name;
				$hash_parse{$id}{star_count} 	= $star_count;
				$hash_parse{$id}{mature_count}	= $mature_count;
				$hash_parse{$id}{total_count} 	= $precursor_count;
				$hash_parse{$id}{coordinate} 	= $coordinate;
				$hash_parse{$id}{star_seq}		= $star_seq;
				$hash_parse{$id}{mature_seq} 	= $mature_seq;	
				$hash_parse{$id}{precursor_seq} = $precursor_seq;
				
			} else {
				print STDERR "Error:$id repeat\n";
				die;
			}
			
			unless( $flag ){
				print STDERR "Error:$str\n";
				die;
			}
			
			$str = $str_tail;
		} 			
	} 
	close IN;
	
	
	# +++++++++++++++++++++++++++++++++++++ #
	
	open  MIRDEEP,">$outfile_parse" or die;
	print MIRDEEP "Name\tknown/novel\tmature position\tstar position\t";
	print MIRDEEP "total read count\tmature read count\tstar read count\t";
	print MIRDEEP "mature sequence\tstar sequence\tprecursor sequence\t";
	print MIRDEEP "mature coordiante\tstar coordinate\tprecursor coordinate\n";
	
	for my $id ( sort keys %hash_parse ){
		
		my $mature_name		= $hash_parse{$id}{mature_name};
		my $star_count		= $hash_parse{$id}{star_count};
		my $mature_count	= $hash_parse{$id}{mature_count};
		my $precursor_count	= $hash_parse{$id}{total_count};
		
		my $star_seq		= $hash_parse{$id}{star_seq};
		my $mature_seq		= $hash_parse{$id}{mature_seq};
		my $precursor_seq	= $hash_parse{$id}{precursor_seq};
		
	
		my $coordinate = $hash_parse{$id}{coordinate};
		if ( $coordinate =~ /(\S+):(\d+)\.\.(\d+):(\S+)/ ){
			
		
			my $chrom 	= $1;
			my $pb 		= $2;
			my $pe 		= $3;
			my $strand 	= $4;
			
			if ( $strand eq "-" ){
				$pb++;
			} else {
				$pe--;
			}
			
			if ( $pb >= $pe ){
				print STDERR "Error:coordinate ($id)\n";
				die;
			}
				
			my $mature_beg = index( $precursor_seq, $mature_seq );
			my $mature_end = $mature_beg + length( $mature_seq ) - 1;
			if ( $mature_beg < 0 or $mature_end > length( $precursor_seq ) ){
				print STDERR "Error:coordinate ($id)\n";
				die;
			}
			
			my $left_offset  = $mature_beg;
			my $right_offset = length($precursor_seq) - $left_offset - length($mature_seq);
			($left_offset,$right_offset) = ($right_offset,$left_offset) if $strand eq "-";
							
			my $mb = $pb + $left_offset;
			my $me = $pe - $right_offset;
			
			
			my $star_beg = index( $precursor_seq, $star_seq );
			my $star_end = $star_beg + length( $star_seq ) - 1;
			if ( $star_beg < 0 or $star_end > length( $precursor_seq ) ){
				print STDERR "Error:coordinate ($id)\n";
				die;
			}
			
			$left_offset  = $star_beg;
			$right_offset = length($precursor_seq) - $left_offset - length($star_seq);
			($left_offset,$right_offset) = ($right_offset,$left_offset) if $strand eq "-";
							
			my $sb = $pb + $left_offset;
			my $se = $pe - $right_offset;
			
			
			my $star_coor = "$chrom:$sb:$se:$strand";
			my $mature_coor = "$chrom:$mb:$me:$strand";
			my $precursor_coor = "$chrom:$pb:$pe:$strand";
			
			
			my $mature_arm = "3p";
			my $star_arm = "5p";
			if ( $mature_beg < int(length($precursor_seq)/2) ){
				$mature_arm = "5p";
				$star_arm = "3p";
			} 
			
			$mature_beg++;
			$mature_end++;
			$star_beg++;
			$star_end++;
			
			my $str = "$id\t$mature_name\t$mature_beg:$mature_end\t$star_beg:$star_end\t";
			$str .= "$precursor_count\t$mature_count\t$star_count\t";
			$str .= "$mature_seq\t$star_seq\t$precursor_seq\t$mature_coor\t$star_coor\t$precursor_coor";		
			print MIRDEEP "$str\n";
			
			
			my ($star,$mature,$precursor);		
			if ( defined $sample ){
				$star = "$sample-$id-$star_arm";
				$mature = "$sample-$id-$mature_arm";
				$precursor = "$sample-$id";
			} else {					
				$star = "$id-$star_arm";
				$mature = "$id-$mature_arm";
				$precursor = $id;
			}

			
			unless(exists $hash_sequence->{mature}{$mature}{seq}){
				$mature_seq = uc ( $mature_seq );
				$mature_seq =~ s/U/T/ig;
				$hash_sequence->{mature}{$mature}{seq} 		= $mature_seq;
				$hash_sequence->{mature}{$mature}{count} 	= $mature_count;
				$hash_sequence->{mature}{$mature}{site} 	= "$mature_beg:$mature_end";
				$hash_sequence->{mature}{$mature}{coor}		= $mature_coor;
				$hash_sequence->{mature}{$mature}{tool}		= "miRDeep2";
				$hash_sequence->{mature}{$mature}{parent}	= $precursor;
			} else {
				print STDERR "Error:$mature repeat\n";
				die;
			}
			
			
			unless(exists $hash_sequence->{precursor}{$precursor}{seq}){
				$precursor_seq = uc ( $precursor_seq );
				$precursor_seq =~ s/U/T/ig;
				$hash_sequence->{precursor}{$precursor}{seq} 	= $precursor_seq;
				$hash_sequence->{precursor}{$precursor}{coor}	= $precursor_coor;
				$hash_sequence->{precursor}{$precursor}{count}	= $precursor_count;
				$hash_sequence->{precursor}{$precursor}{tool}	= "miRDeep2";
			} else {
				print STDERR "Error:$precursor repeat\n";
				die;
			}
			
					
			if( $star_count > 0 ){								
				unless(exists $hash_sequence->{mature}{$star}{seq}){
					$star_seq = uc ( $star_seq );
					$star_seq =~ s/U/T/ig;
					$hash_sequence->{mature}{$star}{seq} 	= $star_seq;
					$hash_sequence->{mature}{$star}{count} 	= $star_count;
					$hash_sequence->{mature}{$star}{site} 	= "$star_beg:$star_end";
					$hash_sequence->{mature}{$star}{coor}	= $star_coor;
					$hash_sequence->{mature}{$star}{tool}	= "miRDeep2";
					$hash_sequence->{mature}{$star}{parent}	= $precursor;
				}  else {
					print STDERR "Error:$star repeat\n";
					die;
				}				
			}
				
		} else {
			
			print STDERR "Error:format of coordiante ($id)\n";
			die;
			
		}
		
	}
	
	close MIRDEEP;
	
	
	unless( %hash_parse ){
		print STDERR "warn:No predicted novel microRNAs in miRDeep2\n";
	}
	
}


	
###############################################################
1;
	
	